// jcpplisp.cpp

#include "jcpplisp.h"

char lineStr[BUFLEN];

CELL *NILCell,*TCell;
CELL *QUOTE,*CAR,*CDR,*CONS,*PRINT,*PRIN1,*SET,*NIL,*T;
CELL *BACKQUOTE,*BACK_QUOTE,*COMMA,*AST;
CELL *DEFUN,*AMPREST,*LET,*LETAST,*SETQ,*PROG,*GO,*RETURN;
CELL *AND,*OR,*ONEPLUS,*ONEMINUS,*COND,*NUMBEREQUAL,*DEFMACRO,*AMPBODY;
CELL *ATOM,*LIST,*EQ,*EVAL,*NULLP,*PROGN,*IF,*TREAD,*TOPEN,*COLDIRECTION,*COLINPUT,*COLOUTPUT;
CELL *TCLOSE,*PROG1,*PLUS,*MINUS,*MINUSP,*EQL,*CHAR,*STRINGP,*LENGTH,*COERCE,*STRING,*PRINC;
CELL *FUNCTION,*LAMBDA,*SPACES,*TERPRI,*STRINGEQUAL,*READLINE,*GC,*BYE,*QUIT,*EXIT,*DEFPARAMETER;
CELL *DEFVAR,*CONSP,*NUMBERP,*TIMES,*DIVIDE,*VALUES,*MULTIPLEVALUELIST,*EVAL1,*APPLY,*FUNCALL;
CELL *RPLACA,*RPLACD,*SYMBOLP,*CHARACTERP,*CHAREQUAL,*CHARLESS,*MACROEXPAND;
CELL *INTEGERP,*SETARRAY,*CHARACTER,*ARRAYCAR,*ARRAYCDR,*ARRAYP,*RANDOM;
CELL *SQRT,*EXPT,*EXP,*LOG,*SIN,*COS,*TAN,*ASIN,*ACOS,*ATAN;
CELL *EVENP,*FLOOR,*NOW,*LOCALTIME,*MAKETIME;
CELL *CHARCODE,*CODECHAR;
CELL *CATCHLISP,*THROWLISP;
CELL *UNWINDPROTECT;
CELL *ABORT;
CELL *ERRORLISP;
CELL *BREAKLISP,*CONTINUELISP;
CELL *SYMBOLNAME,*TINTERN,*TINTERNAL;
CELL *SYMBOLFUNCTION,*MACRO;
CELL *BLOCKLISP,*RETURNFROMLISP;
CELL *TINTERNWIDE;

CELL *ReturnValue,*GoSymbol;

char ReturnFlag;

char *ReadEvalData[READ_EVAL_NUM] = {
    "(setq pi 3.14159265358979)",

    "(defmacro prog (args &rest bd)"
	"`(block nil (prog** ,args ,@bd)) )",

    "(defmacro return (&rest x) `(return-from nil ,@x))",

    "(defun oddp (n) (cond ((evenp n) nil) (t t)))",

    "(defun append1 (x y) (cond ((atom x) y) (t (cons (car x) (append1 (cdr x) y)))))",

    "(defmacro loop1 (&rest b) (append1 (append1 (list 'prog '() 'label) b) (list '(go label))))",

    "(defmacro loop (&rest b) (append1 (append1 (list 'prog '() 'label) b) (list '(go label))))",

    "(defmacro dolist (&rest b)"
	"(append1"
	    "(list 'prog `(,(car (car b)))"
		"`(setq list ,(car (cdr (car b))))"
		" 'label "
                " `(if (null list) (setq ,(car (car b)) nil))"
		" `(if (null list) ,(list 'return (cond ((null (cdr (cdr (car b)))) nil) (t (car (cdr (cdr (car b))))))))"
                " `(setq ,(car (car b)) (car list))"
	    ")"
	    "(append1 (cdr b) (list '(setq list (cdr list)) '(go label))) ) )",

    "(defmacro dotimes (&rest b)"
	"`(prog (,(car (car b)))"
	    "(setq ,(car (car b)) 0)"
	    " label "
	    "(if (= ,(car (car b)) ,(car (cdr (car b))))"
                "(return ,(cond ((null (cdr (cdr (car b)))) nil) (t (car (cdr (cdr (car b))))))) )"
            ",@(cdr b)"
                "(setq ,(car (car b)) (1+ ,(car (car b))))"
            "(go label) ) )",

    "(defun load (fname)"
	"(let ((strm) (s))"
	    "(setq strm (open fname))"
	    "(if (null strm) nil"
	        "(loop1 "
		    "(if (null (setq s (read strm nil))) (progn (close strm) (return t)))"
		    "(eval s) ) ) ) )",

    "(defmacro with-open-file (a &rest b)"
	"`(let ((,(car a) (open ,@(cdr a))))"
	    "(unwind-protect"
		"(if (null ,(car a)) nil"
		    "(progn ,@b) )"
		"(if (not (null ,(car a)))"
		    "(progn (close ,(car a)) (print 'closed)) ) ) ) )",

    "(defun not (e) (cond ((null e) t) (t nil)))",

    "(defun zerop (x) (cond ((= x 0) t) (t nil)))",

    "(defun listp (x) (cond ((null x) t) ((not (atom x)) t) (t nil)))",

    "(defun > (x y) (cond ((minusp (- x y)) nil) ((zerop (- x y)) nil) (t t)))",

    "(defun >= (x y) (cond ((minusp (- x y)) nil) (t t)))",

    "(defun < (x y) (cond ((minusp (- x y)) t) (t nil)))",

    "(defun <= (x y) (cond ((minusp (- x y)) t) ((zerop (- x y)) t) (t nil)))",
/*
    "(defun equal (x y)"
	"(if (atom x) (eql x y)"
	    "(if (atom y) nil"
		"(if (equal (car x) (car y))"
		    "(equal (cdr x) (cdr y))"
		    "nil ) ) ) )",
*/

    "(defun equal (x y)"
	"(if (atom x)"
	    "(if (eql x y)"
		"t"
		"(if (and (stringp x) (stringp y))"
		    "(string-equal x y)"
		    "nil ) )"
	    "(if (atom y)"
		"nil"
		"(if (equal (car x) (car y))"
		    "(equal (cdr x) (cdr y))"
		    "nil ) ) ) )",

    "(defun subseq (x m &rest n)"
	"(let ((ans nil) (len) (k (if (atom n) nil (car n))))"
	    "(cond"
	        "((stringp x)"
	         "(cond"
		    "((null k) (setq len (length x))"
		     "(loop"
			"(if (= m len) (return (coerce ans 'string)))"
			"(setq ans (append ans (list (char x m))))"
			"(setq m (1+ m)) ) )"
		    "(t"
		     "(loop"
			"(if (= m k) (return (coerce ans 'string)))"
			"(setq ans (append ans (list (char x m))))"
			"(setq m (1+ m)) ) ) ) )"
	        "(t"
	         "(cond"
		    "((null k) (setq len (length x))"
		     "(loop"
			"(if (= m len) (return ans))"
			"(setq ans (append ans (list (nth m x))))"
			"(setq m (1+ m)) ) )"
		    "(t"
		     "(loop"
			"(if (= m k) (return ans))"
			"(setq ans (append ans (list (nth m x))))"
			"(setq m (1+ m)) ) ) ) ) ) ) )",

    "(defun nth (n lst)"
	"(let ((k 0))"
	    "(loop"
	        "(if (= k n) (return (car lst)))"
	        "(setq lst (cdr lst))"
	        "(setq k (1+ k)) ) ) )",

    "(defun pprint (x)"
        "(pprint2 x 0)"
        "(terpri)"
        " t )",

    "(defun pprint2 (x n)"
        "(cond"
            "((atom x) (spaces n) (prin1 x))"
            "((is-all-atom-1-list x) (spaces n) (prin1 x))"
            "(t"
             "(spaces n)"
             "(princ #\\()"
             "(pprint3 (car x) (+ n 1))"
             "(setq x (cdr x))"
             "(loop"
                 "(if (atom x) (return))"
                 "(terpri)"
                 "(pprint2 (car x) (+ n 1))"
                 "(setq x (cdr x)) )"
             "(princ #\\)) ) ) )",

    "(defun pprint3 (x n)"
        "(cond"
            "((atom x) (prin1 x))"
            "((is-all-atom-1-list x) (prin1 x))"
            "(t"
             "(princ #\\()"
             "(pprint3 (car x) (+ n 1))"
             "(setq x (cdr x))"
             "(loop"
                 "(if (atom x) (return))"
                 "(terpri)"
                 "(pprint2 (car x) (+ n 1))"
                 "(setq x (cdr x)) )"
         "(princ #\\)) ) ) )",

    "(defun is-all-atom (x)"
        "(loop"
            "(if (atom x) (return t))"
            "(if (not (atom (car x))) (return nil))"
            "(setq x (cdr x)) ) )",

    "(defun is-all-atom-1-list (x)"
        "(loop"
            "(if (atom x) (return t))"
            "(if (and (not (atom (car x))) (not (is-all-atom (car x)))) (return nil))"
            "(setq x (cdr x)) ) )",

    "(defmacro append (&rest l)"
        "(cond"
	    "((null l) nil)"
	    "((null (cdr l)) (car l))"
	    "(t (list 'append1 (car l)"
		    "(cons 'append (cdr l)) ) ) ) )",

    "(defun reverse (x)"
	"(cond"
	    "((stringp x) (coerce (reverse1 (coerce x 'list)) 'string)) "
	    "(t (reverse1 x)) ) )",

    "(defun reverse1 (x)"
	"(cond"
	    "((atom x) x)"
	    "(t (append (reverse (cdr x)) (cons (car x) nil))) ) )",

    "(defun multiple-value-born$ (arg value)"
        "(cond"
	    "((null arg) nil)"
	    "((null value) (cons (list 'setq (car arg) 'nil) (multiple-value-born$ (cdr arg) value)))"
	    "(t (cons (list 'setq (car arg) (list 'quote (car value))) (multiple-value-born$ (cdr arg) (cdr value)))) ) )",

    "(defmacro multiple-value-setq (a b) `(multiple-value-setq$ ,a ,(multiple-value-list (eval1 b))))",

    "(defmacro multiple-value-setq$ (a b) `(prog1 ,@(multiple-value-born$ a b)))",

    "(defmacro multiple-value-bind (a b &rest c) `(let ,(mapcar #'list a) (multiple-value-setq ,a ,b) ,@c))",

    "(defun mapcar (func a &rest b)"
        "(cond"
	    "((atom a) nil)"
	    "((and (not (atom b)) (atom (car b))) nil)"
	    "((atom b) (cons"
                         "(funcall func (car a))"
                         "(mapcar func (cdr a)) ) )"
            "(t (cons (apply func (cons (car a) (list (car (car b)))))"
		     "(mapcar func (cdr a) (cdr (car b))) ) ) ) )",

    "(defun member (item lst &rest a)"
        "(let ((test (if (and (not (null a))"
                             "(eq :test (car a))"
			     "(not (atom (cdr a))) )"
		        "(car (cdr a))"
		    "#'eql) ) )"
	    "(if (null lst) nil"
	        "(if (funcall test item (car lst)) lst"
		    "(member item (cdr lst) :test test)))))",

    "(defun first (x) (car x))",

    "(defun rest (x) (cdr x))",

    "(defun char= (x y) (char-equal x y))",

    "(defun char/= (x y) (not (char-equal x y)))",

    "(defun char> (x y) (char< y x))",

    "(defun char<= (x y) (or (char< x y) (char= x y)))",

    "(defun char>= (x y) (or (char> x y) (char= x y)))",

    "(defmacro lambda (&rest b) `(function (lambda ,@b)))",

//-----------------------------------------------------------------------------------------------------------------
    "(defun caaaar (x) (car (caaar x)))",

    "(defun caaadr (x) (car (caadr x)))",

    "(defun caaar (x) (car (caar x)))",

    "(defun caadar (x) (car (cadar x)))",

    "(defun caaddr (x) (car (caddr x)))",

    "(defun caadr (x) (car (cadr x)))",

    "(defun caar (x) (car (car x)))",

    "(defun cadaar (x) (car (cdaar x)))",

    "(defun cadadr (x) (car (cdadr x)))",

    "(defun cadar (x) (car (cdar x)))",

    "(defun caddar (x) (car (cddar x)))",

    "(defun cadddr (x) (car (cdddr x)))",

    "(defun caddr (x) (car (cddr x)))",

    "(defun cadr (x) (car (cdr x)))",

//----------------------------------------------
    "(defun cdaaar (x) (cdr (caaar x)))",

    "(defun cdaadr (x) (cdr (caadr x)))",

    "(defun cdaar (x) (cdr (caar x)))",

    "(defun cdadar (x) (cdr (cadar x)))",

    "(defun cdaddr (x) (cdr (caddr x)))",

    "(defun cdadr (x) (cdr (cadr x)))",

    "(defun cdar (x) (cdr (car x)))",

    "(defun cddaar (x) (cdr (cdaar x)))",

    "(defun cddadr (x) (cdr (cdadr x)))",

    "(defun cddar (x) (cdr (cdar x)))",

    "(defun cdddar (x) (cdr (cddar x)))",

    "(defun cddddr (x) (cdr (cdddr x)))",

    "(defun cdddr (x) (cdr (cddr x)))",

    "(defun cddr (x) (cdr (cdr x)))",

//----------------------------------------------
    "(defun second (x) (nth 1 x))",

    "(defun third (x) (nth 2 x))",

    "(defun fourth (x) (nth 3 x))",

    "(defun fifth (x) (nth 4 x))",

    "(defun sixth (x) (nth 5 x))",

    "(defun seventh (x) (nth 6 x))",

    "(defun eighth (x) (nth 7 x))",

    "(defun ninth (x) (nth 8 x))",

    "(defun tenth (x) (nth 9 x))",

//------------------------------------------------

//; }NSETF
"(defmacro setf (x? y?)"
    "(cond"
	"((symbolp x?) `(setq ,x? ,y?))"
	"((and (not (atom x?)) (eq (car x?) 'car) (not (atom (cdr x?))))"
	 "`(progn (rplaca ,(car (cdr x?)) ,y?) ,y?) )"
	"((and (not (atom x?)) (eq (car x?) 'cdr) (not (atom (cdr x?))))"
	 "`(progn (rplacd ,(car (cdr x?)) ,y?) ,y?) )"
	"((and (not (atom x?)) (eq (car x?) 'rest) (not (atom (cdr x?))))"
	 "`(progn (rplacd ,(car (cdr x?)) ,y?) ,y?) )"
	"((and (not (atom x?)) (eq (car x?) 'nth) (not (atom (cdr x?))) (not (atom (cdr (cdr x?)))))"
	 "`(progn (rplaca (nthcdr ,(car (cdr x?)) ,(car (cdr (cdr x?)))) ,y?) ,y?) )"
	"((and (not (atom x?)) (not (atom (cdr x?))) (find-nth-name (car x?)))"
	 "`(progn (rplaca (nthcdr ,(cdr (find-nth-name (car x?))) ,(car (cdr x?))) ,y?) ,y?) )"
	"((and (not (atom x?)) (not (atom (cdr x?))) (find-a-name (car x?)))"
	 "`(progn (rplaca ,(cons (cdr (find-a-name (car x?))) (cdr x?)) ,y?) ,y?) )"
	"((and (not (atom x?)) (not (atom (cdr x?))) (find-d-name (car x?)))"
	 "`(progn (rplacd ,(cons (cdr (find-d-name (car x?))) (cdr x?)) ,y?) ,y?) )"
	"((and (not (atom x?)) (eql (car x?) 'gethash) (not (atom (cdr x?))) (not (atom (cdr (cdr x?)))))"
	    "(list 'setf-hash (car (cdr x?)) (car (cdr (cdr x?))) y?)"
	")"
	"((and (consp x?) (eql (car x?) 'aref))"
	 "(list 'progn (list 'rplaca (cons 'pre-aref (cons y? (cdr x?))) y?) y?) )"
	"(t '(error \"G[: SETFɑ΂Ă̊֐͒`Ă܂\"))"
    ")"
")",

"(defun setf-hash (k i v)"
  "(let ((org-i i))"
    "(setq i (cdr (cdr org-i)))"
    "(let ((pair (apply #'assoc (list k i :test (car (cdr org-i))))))"
	"(cond"
	    "((null pair) (rplacd (last org-i) (list (cons k v))) v)"
	    "(t (rplacd pair v) v) ) ) ) )",

"(defun find-nth-name (name)"
    "(assoc name '((first . 0) (second . 1) (third . 2) (fourth . 3) (fifth . 4) (sixth . 5)"
		  "(seventh . 6) (eighth . 7) (ninth . 8) (tenth . 9) ) ) )",

//; ֐NTHCDR
"(defun nthcdr (n lst)"
    "(cond"
	"((> 0 n) nil)"
	"((atom lst) lst)"
	"((zerop n) lst)"
	"(t (nthcdr (1- n) (cdr lst))) ) )",

//; ֐ASSOC
"(defun assoc (x alist &rest r)"
  "(let ((test #'eq))"
    "(if (and (>= (length r) 2) (eq :test (car r))) (setq test (car (cdr r))))"
    "(cond"
	"((atom alist) nil)"
	"((apply test (list x (car (car alist)))) (car alist))"
	"(t (apply #'assoc (if (>= (length r) 2) (list x (cdr alist) (car r) (car (cdr r))) (list x (cdr alist))))) ) ) )",

"(defun find-a-name (name)"
    "(assoc name '("
"(caaaar . caaar) (caaadr . caadr) (caaar . caar) (caadar . cadar) (caaddr . caddr) (caadr . cadr) (caar . car)"
"(cadaar . cdaar) (cadadr . cdadr) (cadar . cdar) (caddar . cddar) (cadddr . cdddr) (caddr . cddr) (cadr . cdr)"
    ") ) )",

"(defun find-d-name (name)"
    "(assoc name '("
"(cdaaar . caaar) (cdaadr . caadr) (cdaar . caar) (cdadar . cadar) (cdaddr . caddr) (cdadr . cadr) (cdar . car)"
"(cddaar . cdaar) (cddadr . cdadr) (cddar . cdar) (cdddar . cddar) (cddddr . cdddr) (cdddr . cddr) (cddr . cdr)"
    ") ) )",

//;----- hash-table functions---------------------------------------------------------------------------------------

"(defun make-hash-table (&rest b)"
    "(cond"
	"((null b) (list 'hash-table 'eql))"
	"((eql (car b) :test) (list 'hash-table (car (cdr b))))"
	"(t (error \"G[: @G[\")) ) )",

"(defun gethash (key ins)"
    "(let ((cnd))"
	"(if (or (atom ins) (not (eql (car ins) 'hash-table)) (atom (cdr ins)) (not (atom (setq cnd (car (cdr ins))))) )"
	    "(error \"G[: nbVe[uł͂܂\")"
	    "(progn"
		"(setq ins (cdr (cdr ins)))"
		"(loop"
		    "(if (null ins) (return (values nil nil)))"
		    "(if (atom (car ins)) (error \"G[: nbVe[uĂ܂\"))"
		    "(if (funcall cnd key (car (car ins))) (return (values (cdr (car ins)) t)))"
		    "(setq ins (cdr ins))"
		")"
	    ")"
	")"
    ")"
")",

"(defun hash-table-count (ins)"
    "(if (or (atom ins) (not (eql (car ins) 'hash-table)) (atom (cdr ins)))"
	"(error \"G[: nbVe[uł͂܂\")"
	"(length (cdr (cdr ins)))"
    ")"
")",

"(defun remhash (key ins)"
    "(if (or (atom ins) (not (eql (car ins) 'hash-table)) (atom (cdr ins)))"
	"(error \"G[: nbVe[uł͂܂\")"
	"(let ((previns) (test (car (cdr ins))))"
	    "(setq previns (cdr ins))"
	    "(setq ins (cdr (cdr ins)))"
	    "(loop"
		"(if (null ins) (return nil))"
		"(if (funcall test key (car (car ins)))"
		    "(progn"
			"(rplacd previns (cdr ins))"
			"(return t)"
		    ")"
		")"
		"(setq previns ins)"
		"(setq ins (cdr ins))"
	    ")"
	")"
    ")"
")",

"(defun maphash (func hash)"
    "(setq hash (cdr (cdr hash)))"
    "(loop"
	"(if (null hash) (return nil))"
	"(funcall func (car (car hash)) (cdr (car hash)))"
	"(setq hash (cdr hash)) ) )",

//;--------------------------------------------------
//; ֐LAST
"(defun last (x)"
    "(cond"
	"((atom x) x)"
	"((= (length x) 1) x)"
	"(t (last (cdr x))) ) )",

//; }NCASE
"(defmacro case (key &rest body)"
    "`(cond"
	" ,@(make-case key body) ) )",

"(defun make-case (key body)"
    "(cond"
	"((null body) nil)"
	"((eq (car (car body)) 'otherwise) (cons `(t ,@(cdr (car  body))) (make-case key (cdr body))))"
	"((atom (car (car body))) (cons `((eql ,key ',(car (car body))) ,@(cdr (car body))) (make-case key (cdr body))))"
	"(t (cons `((member ,key ',(car (car body))) ,@(cdr (car body))) (make-case key (cdr body)))) ) )",

//-array------------------------------------------------------------------------------------------------------------

"(defun make-array (dim &rest b)"
    "(let ((contents nil) (element nil) (type nil) (arr nil))"
        "(setq contents (get-contents b))"
	"(setq element (get-element b))"
	"(setq type (get-type b))"
	"(if (eql type 'character)"
	    "(cond"
		"((characterp element) t)"
		"((eql element 'nothing?) (setq element #\\ ))"
		"(t nil) )"
	    "(progn (if (eql element 'nothing?) (setq element nil)) (setq type 'all)) )"
	"(cond"
	    "((and (eql type 'character) (not (characterp element))) (error \"G[: ̏vfKvƂ܂\"))"
	    "((and (integerp dim) (not (< dim 0)))"
	     "(setq arr (set-array (cons 1 (cons type (consn dim element)))))"
             "(set-contents arr (list dim) contents element) arr )"
	    "((accept-dimension-element dim)"
	     "(setq arr (set-array (cons (length dim) (cons type (make-array-base (reverse dim) element)))))"
	     "(set-contents arr dim contents element) arr )"
	    "(t (error \"G[: @G[\")) ) ) )",

"(defun set-contents (arr dim contents def)"
    "(let ((pos (init-pos (length dim))) (el nil))"
	"(loop"
	    "(if (eql (car pos) (car dim)) (return t))"
	    "(setq el (a-content contents pos def))"
	    "(eval `(setf (aref arr ,@pos) el))"
	    "(setq pos (reverse (inc-pos (reverse pos) (reverse dim)))) ) ) )",

"(defun a-content (contents pos def)"
    "(let ((k))"
	"(setq k (car pos))"
	"(cond"
	    "((atom contents) def)"
	    "((>= k (length contents)) def)"
	    "((atom (cdr pos)) (nth k contents))"
	    "(t (a-content (nth k contents) (cdr pos) def)) ) ) )",

"(defun inc-pos (pos dim)"
    "(let ((k))"
	"(setq k (1+ (car pos)))"
	"(cond"
	    "((eql k (car dim))"
	     "(cond"
		"((atom (cdr pos)) (list k))"
		"(t (cons 0 (inc-pos (cdr pos) (cdr dim)))) ) )"
	    "(t"
	     "(cons k (cdr pos)) ) ) ) )",

"(defun init-pos (len)"
    "(cond"
	"((zerop len) nil)"
	"(t (cons 0 (init-pos (1- len)))) ) )",

"(defun make-array-base (dim element)"
    "(cond"
	"((atom dim) element)"
	"(t (make-array-base (cdr dim) (consn (car dim) element))) ) )",

"(defun copy (s)"
    "(cond"
	"((consp s) (cons (copy (car s)) (copy (cdr s))))"
	"(t s) ) )",

"(defun consn (n element)"
    "(cond"
	"((zerop n) nil)"
	"(t (cons element (consn (1- n) (copy element)))) ) )",

"(defun get-contents (b)"
    "(let ((f (find :initial-contents b)))"
	"(cond"
	    "((null f) nil)"
	    "((atom (cdr f)) nil)"
	    "(t (car (cdr f))) ) ) )",

"(defun get-element (b)"
    "(let ((f (find :initial-element b)))"
	"(cond"
	    "((null f) 'nothing?)"
	    "((atom (cdr f)) 'nothing?)"
	    "(t (car (cdr f))) ) ) )",

"(defun get-type (b)"
    "(let ((f (find :element-type b)))"
	"(cond"
	    "((null f) nil)"
	    "((atom (cdr f)) nil)"
	    "((eql (car (cdr f)) 'character) 'character)"
	    "(t 'all) ) ) )",

"(defun find (v l)"
    "(cond"
	"((atom l) nil)"
	"((eql (car l) v) l)"
	"(t (find v (cdr l))) ) )",

"(defun accept-dimension-element (l)"
    "(cond"
	"((atom l) nil)"
	"(t (accept-dimension-element-base l)) ) )",

"(defun accept-dimension-element-base (l)"
    "(cond"
	"((atom l) t)"
	"((not (integerp (car l))) nil)"
	"((< (car l) 0) nil)"
	"(t (accept-dimension-element-base (cdr l))) ) )",

"(defun aref (a &rest b)"
    "(let ((dim (array-car a)))"
	"(cond"
	    "((not (eql dim (length b))) (error \"G[: ݒ肷ƂŃG[𐶂܂\"))"
	    "(t (get-array-pos (cdr (array-cdr a)) b)) ) ) )",

"(defun get-array-pos (a b)"
    "(cond"
	"((atom b) a)"
	"((not (integerp (car b))) (error \"G[: ݒ肷ƂŃG[𐶂܂\"))"
	"(t (get-array-pos (nth (car b) a) (cdr b))) ) )",

"(defun pre-aref (c a &rest b)"
    "(let ((dim (array-car a)) (type (car (array-cdr a))))"
	"(cond"
	    "((not (eql dim (length b))) (error \"G[: ݒ肷ƂŃG[𐶂܂\"))"
	    "((and (eql type 'character) (not (characterp c))) (error \"G[: ݒG[\"))"
	    "(t (pre-get-array-pos (cdr (array-cdr a)) b)) ) ) )",

"(defun pre-get-array-pos (a b)"
    "(cond"
	"((atom b) a)"
	"((not (integerp (car b))) (error \"G[: ݒ肷ƂŃG[𐶂܂\"))"
	"((eql (length b) 1) (nthcdr (car b) a))"
	"(t (pre-get-array-pos (nth (car b) a) (cdr b))) ) )",

//;------July 2017-----------------------------------------------------------------------------------
"(defun nconc (a &rest b)"
    "(loop"
	"(if (null b) (return a))"
	"(rplacd (last a) (car b))"
	"(setq b (cdr b)) ) )",

"(defun remove (a b &rest c)"
    "(let ((test) (count) (ans nil))"
     "(cond"
	"((setq test (member :test c))"
	 "(cond"
	   "((not (atom (cdr test)))"
	    "(setq test (cadr test)) )"
	   "(t (setq test #'eql)) ) )"
	"(t (setq test #'eql)) )"
     "(cond"
	"((setq count (member :count c))"
	 "(cond"
	   "((not (atom (cdr count)))"
	    "(setq count (cadr count)) )"
	   "(t (setq count nil)) ) )"
	"(t (setq count nil)) )"
     "(loop"
	"(if (eql 0 count) (return (append ans b)))"
	"(if (atom b) (return ans))"
	"(cond"
	    "((funcall test a (car b)) (if (null count) nil (setq count (1- count))))"
	    "(t (setq ans (append ans (list (car b))))) )"
	"(setq b (cdr b)) ) ) )",

"(defun delete (a b &rest c)"
    "(let ((test) (count) (k 0))"
     "(cond"
	"((setq test (member :test c))"
	 "(cond"
	   "((not (atom (cdr test)))"
	    "(setq test (cadr test)) )"
	   "(t (setq test #'eql)) ) )"
	"(t (setq test #'eql)) )"
     "(cond"
	"((setq count (member :count c))"
	 "(cond"
	   "((not (atom (cdr count)))"
	    "(setq count (cadr count)) )"
	   "(t (setq count nil)) ) )"
	"(t (setq count nil)) )"
     "(loop"
	"(if (eql 0 count) (return b))"
	"(if (atom (nthcdr k b)) (return b))"
	"(cond"
	    "((funcall test a (nth k b))"
	     "(if (= k 0)"
		 "(return (apply #'remove (cons a (cons b c))))"
		 "(progn (rplacd (nthcdr (1- k) b) (nthcdr (1+ k) b))) )"
	     "(if (null count) nil (setq count (1- count))) )"
	    "(t (setq k (1+ k))) ) ) ) )",

"(defun sort (lst test &rest c)"
    "(let ((key) (s) (l) (m) (len (length lst)) (v) (k 0) (kep lst))"
	"(cond"
	    "((setq key (member :key c))"
	     "(setq key (cadr key)) )"
	    "(t (setq key nil)) )"
	"(cond"
	    "((< len 2) lst)"
	    "(t"
	     "(loop"
		"(setq lst kep)"
		"(if (= k len) (return lst))"
	    	"(if (null key) (setq m (nth k lst)) (setq m (funcall key (nth k lst))))"
		"(setq s nil)"
		"(setq l nil)"
	    	"(loop"
		    "(if (null lst) (return))"
		    "(if (null key) (setq v (car lst)) (setq v (funcall key (car lst))))"
		    "(cond"
			"((funcall test v m) (setq s (cons (car lst) s)))"
			"(t (setq l (cons (car lst) l))) )"
		    "(setq lst (cdr lst)) )"
		"(if (or (null s) (null l))"
		    "(setq k (1+ k))"
		    "(return (replace-destroy kep (append (apply #'sort (cons (reverse s) (cons test c)))"
						     "(apply #'sort (cons (reverse l) (cons test c))) ) ) ) ) ) ) ) ) )",

"(defun replace-destroy (a b)"
    "(cond"
	"((and (null a) (null b)) a)"
	"(t"
	 "(rplaca a (car b))"
	 "(rplacd a (cdr b))"
	 "a ) ) )",

//; gensym.lisp

"(defparameter *gensym-counter* 1000)",

"(defun mod (x y)"
    "(let ((n (floor (/ x y))))"
	"(- x (* n y)) ) )",

"(defun int-to-string (i)"
    "(let ((ans nil) (j))"
	"(loop"
	    "(setq j (mod i 10))"
	    "(setq i (floor (/ i 10)))"
	    "(setq ans (cons (code-char (+ (char-code #\\0) j)) ans))"
	    "(if (zerop i) (return (coerce ans 'string))) ) ) )",

"(defun gensym (&rest b)"
    "(cond"
	"((null b) (make-symbol (coerce (append '(#\\G)"
					  "(coerce (int-to-string (setq *gensym-counter* (1+ *gensym-counter*))) 'list)"
				  ") 'string)) )"
	"((and (integerp (car b) (< 0 (car b)))) (make-symbol (coerce (append '(#\\G)"
					  "(coerce (int-to-string (car b)) 'list)"
				  ") 'string)) )"
	"((stringp (car b)) (make-symbol (coerce (append (coerce (car b) 'list)"
					     "(coerce (int-to-string (setq *gensym-counter* (1+ *gensym-counter*))) 'list)"
				  ") 'string)) )"
	"(t (error \"BAD USAGE GENSYM\"))"
    ") )",

"(defun make-symbol (str)"
    "(intern-wide str) )"

//;---------------------------------------------------------------------------------------------------------

};

CELL *cmemory::IncCellStack(CELL *c)
{
	if (curCellStack - CellStack >= CELL_STACK_SIZE - 1) {
		printf("G[: ZX^bNtłBI܂\n");
		exit(1);
		return NULL;
	}
	*curCellStack++ = c;
	return c;
}

int cmemory::IncLocStack(SYMVAL *loc,long len)
{
	long i;

    	if (curLocStack - LocStack >= LOC_STACK_SIZE - 1) {
	    	printf("G[: [JX^bNtłBI܂\n");
		exit(1);
		return -1;
	}
	*curLocStack++ = NULL;
//	loc++;
	for (i = 0;i < len;i++) {
    	    if (curLocStack - LocStack >= LOC_STACK_SIZE - 1) {
	    	printf("G[: [JX^bNtłBI܂\n");
		exit(1);
		return -1;
	    }
	    *curLocStack++ = loc;
	    loc++;
	}
	return 0;
}

void cmemory::IncStaticStack(SYMVAL *loc,long len)
{
	long i;

    	if (curStaticStack - StaticStack >= STATIC_STACK_SIZE - 1) {
	    	printf("G[: X^eBbNX^bNtłBI܂\n");
		exit(1);
	}
	*curStaticStack++ = NULL;
	for (i = 0;i < len;i++) {
    	    if (curStaticStack - StaticStack >= STATIC_STACK_SIZE - 1) {
	    	printf("G[: X^eBbNX^bNtłBI܂\n");
		exit(1);
	    }
	    *curStaticStack++ = loc;
	    loc++;
	}
}

void cmemory::PopCellStack(void)
{
    while (*(--curCellStack) != NULL);
}

void cmemory::PopLocStack(void)
{
    while (*(--curLocStack) != NULL);
}

void cfunction::SetOrgLoc(SYMVAL *orgloc,long orglen,SYMVAL *temploc,long start,long newlen)
{
    long i,j;

    for (i = newlen - 1;!(i < start);i--) {
	if (temploc[i].sym != NULL) {
	    for (j = 0;j < orglen;j++) {
		if (orgloc[j].sym != NULL) {
		    if (orgloc[j].sym == temploc[i].sym) {
			orgloc[j].attr = temploc[i].attr;
			orgloc[j].val = temploc[i].val;
			break;
		    }
		}
	    }
	}
    }
}

void cmemory::Disable(CELL *p)
{
	if (p->attr == 's') {
		p->gc = 'd';
		if (p->mtop != NULL) { (p->mtop)->gc = 'd'; if (((p->mtop)->cdr)->attr == 'l') Disable((p->mtop)->cdr); }
	}
	else if (p->attr == 'i') {
		p->gc = 'd';
		((AINT *)(p->car))->gc = 'd'; // mtop̏uDisableŃ[vȂ悤ɁB
		if (p->mtop != NULL) { (p->mtop)->gc = 'd'; if (((p->mtop)->cdr)->attr == 'l') Disable((p->mtop)->cdr); } //Disable(p->mtop);
	}
	else if (p->attr == 'n') {
		p->gc = 'd';
		((AFLOAT *)(p->car))->gc = 'd';
		if (p->mtop != NULL) { (p->mtop)->gc = 'd'; if (((p->mtop)->cdr)->attr == 'l') Disable((p->mtop)->cdr); } //Disable(p->mtop);
	}
	else if (p->attr == 'b') {
		p->gc = 'd';
		((AINT *)(p->car))->gc = 'd';
		((AINT *)(p->cdr))->gc = 'd';
		if (p->mtop != NULL) { (p->mtop)->gc = 'd'; if (((p->mtop)->cdr)->attr == 'l') Disable((p->mtop)->cdr); } //Disable(p->mtop);
	}
	else if (p->attr == 'm') {
		p->gc = 'd';
		if (p->mtop != NULL) { (p->mtop)->gc = 'd'; if (((p->mtop)->cdr)->attr == 'l') Disable((p->mtop)->cdr); } //Disable(p->mtop);
	}
	else if (p->attr == 'l' && p->gc != 'd') {
		p->gc = 'd';
		Disable(p->car);
		Disable(p->cdr);
		if (p->mtop != NULL) { (p->mtop)->gc = 'd'; if (((p->mtop)->cdr)->attr == 'l') Disable((p->mtop)->cdr); } //Disable(p->mtop);
	}
	else if (p->attr == 'a' && p->gc != 'd') {
		p->gc = 'd';
		Disable(p->car);
		Disable(p->cdr);
		if (p->mtop != NULL) { (p->mtop)->gc = 'd'; if (((p->mtop)->cdr)->attr == 'l') Disable((p->mtop)->cdr); } //Disable(p->mtop);
	}
}

void cmemory::gc(void)
{
	CELL *p;
	CELL **q;
	SYMVAL **r;
	AFLOAT *a;
	SYMVAL *b;
        long k = 0,k2 = 0,k3 = 0,i;
	char c;
	AINT *q2;

	p = cl;
	for (i = 0;i < CELL_SIZE;i++) {
		if (p->attr != 's') p->gc = 'e';
		p++;
	}
	a = nArea;
	for (i = 0;i < NUMBER_SIZE;i++) {
		a->gc = 'e';
	    a++;
	}
	q2 = iArea;
	for (i = 0;i < INT_SIZE;i++) {
		q2->gc = 'e';
		q2++;
	}
	q = CellStack;
	while (q != curCellStack) {
		if (*q != NULL) {
			Disable(*q);
		}
		q++;
	}
	r = LocStack;
//printf("len = %ld\n",curLocStack - LocStack);
	while (r != curLocStack) {
		if (*r != NULL && (*r)->sym != NULL) {
//printf("%x ",(*r)->sym);
			Disable((*r)->val);
		}
		r++;
	}
	r = StaticStack;
	while (r != curStaticStack) {
		if (*r != NULL && (*r)->sym != NULL) {
//printf("%x ",(*r)->sym);
			Disable((*r)->val);
		}
		r++;
	}
	b = sv;
	while (b != curSv) {
                Disable(b->val);
		b++;
	}
	b = fv;  // Add
	while (b != curFv) {
                Disable(b->val);
		b++;
	}
	p = cl;
	for (i = 0;i < CELL_SIZE;i++) {
		if (p->gc == 'd') k++;
		p++;
	}
	a = nArea;
	for (i = 0;i < NUMBER_SIZE;i++) {
		if (a->gc == 'd') k2++;
	    a++;
	}
	q2 = iArea;
	for (i = 0;i < INT_SIZE;i++) {
		if (q2->gc == 'd') k3++;
	    q2++;
	}
	if (GetToEndCell) c = '*';
	else c = ' ';
	FreeCellSize = CELL_SIZE - k;
	FreeNumberSize = NUMBER_SIZE - k2;
	FreeIntSize = INT_SIZE - k3;
	printf("gc: Z %ld _ %ld  %ld gp %c\n",k,k2,k3,c);
}

CELL *cmemory::getCell(void)
{
	CELL *p;
	long i;

	for (i = 0;i < CELL_SIZE - 1;i++) {
	    if (curCl->gc == 'e') {
		p = curCl++;
                if (curCl == cl + CELL_SIZE - 1) {
			curCl = cl;
			GetToEndCell = TRUE;
	        }
                p->gc = 'd';
	        FreeCellSize--;
                p->mtop = NULL;  // Add.
		p->clj = NULL;
		p->cljlen = 0L;

		// 2015/2/24 ȉ4sǉBftH[g̒lBꂪȂƋ̃ZvꂽK[x[WRNVɖ\B
		p->attr = 'l';
		p->attr2 = 'u';
		p->car = NILCell;
		p->cdr = NILCell;

	        return p;
	    }
	    curCl++;
            if (curCl == cl + CELL_SIZE - 1) {
			curCl = cl;
			GetToEndCell = TRUE;
	    }
	}
        printf("G[: Z̈悪tłBI܂B\n");
	exit(1);
	return NULL;
}

int ctextproc::isKanji(unsigned char *str)
{
#if LINUXCOMPILER
	return 0; /* for Linux */
#endif
	if (((unsigned char)0x81 <= *str && *str <= (unsigned char)0x9f) ||
		((unsigned char)0xe0 <= *str && *str <= (unsigned char)0xfc) ) {
		return 1;
	}
	return 0;
}

CELL *cmemory::getMojiCell(char c1,  // A܂̓VtgihrR[hPoCg
		  char c2 ) // VtgihrR[hQoCg
{
    CELL *p;

    if ((p = getCell()) == NULL) return NULL;
    p->attr = 'm';
    p->car = (CELL *)(long)c1;
    if (isKanji((unsigned char *)&c1)) {
	p->cdr = (CELL *)(long)c2;
    }
    else p->cdr = NULL;
    return p;
}

// 2̍ő񐔂Ƃ߂B
long cmathproc::gcd(long x,long y)
{
    long temp,oy;

    if (!(x > y)) { temp  = x; x = y; y = temp; }
    oy = y;
    while (y != 1) {
        if (x % y == 0 && oy % y == 0) return y;
        y--;
    }
    return 1;
}

void cmathproc::yakubun(long bunshi,long bunbo,long *newbunshi,long *newbunbo)
{
    BOOL plus;
    long Gcd;

    if (bunshi == 0L) {
	*newbunshi = 0L;
	*newbunbo = 1L;
	return;
    }
    if (bunshi < 0 && bunbo < 0) plus = TRUE;
    else if (bunshi < 0 && bunbo > 0) plus = FALSE;
    else if (bunshi > 0 && bunbo < 0) plus = FALSE;
    else plus = TRUE;
    if (bunshi < 0) bunshi *= -1L;
    if (bunbo < 0) bunbo *= -1L;
    while ((Gcd = gcd(bunshi,bunbo)) != 1) {
	bunshi /= Gcd;
        bunbo /= Gcd;
    }
    if (plus) { *newbunshi = bunshi; *newbunbo = bunbo; return; }
    *newbunshi = bunshi * (-1L);
    *newbunbo = bunbo;
}

void cmemory::InitCellAndNumber(void)
{
	CELL *p;
	long i;
	AFLOAT *q;
        AINT *q2;

	p = cl;
	for (i = 0;i < CELL_SIZE;i++) {
		p->gc = 'e';
		p++;
	}
	q = nArea;
	for (i = 0;i < NUMBER_SIZE;i++) {
		q->gc = 'e';
		q++;
	}
	q2 = iArea;
	for (i = 0;i < INT_SIZE;i++) {
		q2->gc = 'e';
		q2++;
	}
        for (i = 0;i < FILE_POINTER_LEN;i++) FilePointer[i] = NULL; /* June 2017 */
	throw_flag = FALSE;
	block_throw_flag = FALSE;
}

AFLOAT *cmemory::getNumber(LONGDOUBLE f)
{
	AFLOAT *p;
	long i;

	for (i = 0;i < NUMBER_SIZE - 1;i++) {
	    if (curN->gc == 'e') {
			p = curN++;
            if (curN == nArea + NUMBER_SIZE - 1) curN = nArea;
            p->gc = 'd';
			p->f = f;
			FreeNumberSize--;
			return p;
		}
		curN++;
        if (curN == nArea + NUMBER_SIZE - 1) curN = nArea;
	}
        printf("G[: l̈悪tłBI܂B\n");
	exit(1);
	return NULL; // dummy
}

AINT *cmemory::getInt(long f)
{
	AINT *p;
	long i;

	for (i = 0;i < INT_SIZE - 1;i++) {
	    if (curI->gc == 'e') {
			p = curI++;
            		if (curI == iArea + INT_SIZE - 1) curI = iArea;
            		p->gc = 'd';
			p->f = f;
			FreeIntSize--;
			return p;
	    }
	    curI++;
            if (curI == iArea + INT_SIZE - 1) curI = iArea;
	}
        printf("G[: ̈悪tłBI܂B\n");
	exit(1);
	return NULL; // dummy
}

char *cmemory::getString(char *str)
{
	int len;
	char *p;

	len = strlen(str);
	if (curStr + len + 2 - strArea >= STRING_SIZE) {
		printf("G[: XgÖ悪tłBI܂B\n");
		exit(1);
		*curStr = '\0';
		return NULL;
	}
	strcpy(curStr,str);
	p = curStr;
	curStr += len + 1;
	return p;
}

CELL *cmemory::CheckStringCell(char *u)
{
    CELL **p;

    p = strAreaAddr;
    while (p != curStrAddr) {
        if ((!strcmp((char *)((*p)->car),u)) && (*p)->attr2 != 'g') return *p;
        p++;
    }
    return NULL;
}

CELL *cmemory::getStringCell(char *u)
{
    char *e;
    CELL *p;

    if ((p = CheckStringCell(u)) != NULL) return p;
    e = getString(u); if (e == NULL) return NULL;
    p = getCell(); if (p == NULL) return NULL;
    p->attr = 's';
    if (u[0] == ':') p->attr2 = 'c';
    else p->attr2 = 'u';
    p->car = (CELL *)e;
    if (curStrAddr - strAreaAddr >= STRING_NUM - 1) { printf("G[: XgO̐܂\n"); return NULL; }
    *curStrAddr++ = p;
    return p;
}

CELL *cmemory::getStringCellWide(char *u)
{
    char *e;
    CELL *p;

//    if ((p = CheckStringCell(u)) != NULL) return p;
    e = getString(u); if (e == NULL) return NULL;
    p = getCell(); if (p == NULL) return NULL;
    p->attr = 's';
    p->attr2 = 'g';
    p->car = (CELL *)e;
    if (curStrAddr - strAreaAddr >= STRING_NUM - 1) { printf("G[: XgO̐܂\n"); return NULL; }
    *curStrAddr++ = p;
    return p;
}

BOOL cmemory::AllocMemory(void)
{
    cl = (CELL *)calloc(CELL_SIZE,sizeof(CELL));
    if (cl == NULL) { printf("G[: Z̈̊lɎs܂B\n"); return FALSE; }
    curCl = cl;
    FreeCellSize = CELL_SIZE;
    strArea = (char *)malloc(STRING_SIZE);
    if (strArea == NULL) { printf("G[: XgÖ̊lɎs܂"); return FALSE; }
    curStr = strArea;
    strAreaAddr = (CELL **)calloc(STRING_NUM,sizeof(CELL *));
    if (strAreaAddr == NULL) { printf("G[: XgOAhẌ̊lɎs܂"); return FALSE; }
    curStrAddr = strAreaAddr;
    nArea = (AFLOAT *)calloc(NUMBER_SIZE,sizeof(AFLOAT));
    if (nArea == NULL) { printf("G[: l̈̊lɎs܂\n"); return FALSE; }
    curN = nArea;
    FreeNumberSize = NUMBER_SIZE;
    iArea = (AINT *)calloc(INT_SIZE,sizeof(AINT));
    if (iArea == NULL) { printf("G[: ̈̊lɎs܂\n"); return FALSE; }
    curI = iArea;
    FreeIntSize = INT_SIZE;
    sv = (SYMVAL *)calloc(GLOBAL_SIZE,sizeof(SYMVAL));
    if (sv == NULL) { printf("G[: O[oϐ̈̊lɎs܂\n"); return FALSE; }
    curSv = sv;
    fv = (SYMVAL *)calloc(FUNCTION_SIZE,sizeof(SYMVAL));
    if (fv == NULL) { printf("G[: ֐̈̊lɎs܂\n"); return FALSE; }
    curFv = fv;
    CellStack = (CELL **)calloc(CELL_STACK_SIZE,sizeof(CELL *));
    if (CellStack == NULL) {
	printf("G[: ZX^bN̈̊lɎs܂\n"); return FALSE;
    }
    LocStack = (SYMVAL **)calloc(LOC_STACK_SIZE,sizeof(SYMVAL *));
    if (LocStack == NULL) {
	printf("G[: [JX^bN̈̊lɎs܂\n"); return FALSE;
    }
    curCellStack = CellStack; curLocStack = LocStack;
    StaticStack = (SYMVAL **)calloc(STATIC_STACK_SIZE,sizeof(SYMVAL *));
    if (StaticStack == NULL) {
	printf("G[: X^eBbNX^bN̈̊lɎs܂\n"); return FALSE;
    }
    curStaticStack = StaticStack;
    InitCellAndNumber();
    return TRUE;
}

char *ctextproc::SkipSpc(char *s)
{
	while (*s == ' ') s++;
	return s;
}

void ctextproc::NewString(char *s2,char *s1)
{
	char *s,*p;

	s = SkipSpc(s1);
	if (s[0] == '(') {
		p = s + 1;
		p = SkipSpc(p);
		strcpy(s2 + 1,p);
		s2[0] = '(';
		return;
	}
	strcpy(s2,s);
}

BOOL ctextproc::isListString(char *str)
{
	if (*str == '(') return TRUE;
	return FALSE;
}

char *ctextproc::SkipDoubleQuote(char *s)
{
    char prev = '"';

    while (*s != '\0') {
        if (*s == '"')
            if (prev != '\\') return s + 1;
        if (isKanji((unsigned char *)s)) { prev = '*'; s++; }
        else prev = *s;
        s++;
    }
    return NULL;
}

char *ctextproc::SkipDoubleQuote2(char *s)
{
    char prev = '"';

    while (*s != '\0') {
        if (*s == '"')
            if (prev != '\\') return s + 1;
        if (isKanji((unsigned char *)s)) {
	    if (*(s + 1) == '\0') return NULL;
	    else { prev = '*'; s++; }
	}
        else prev = *s;
        s++;
    }
    return NULL;
}

void ctextproc::getCarString(char *u,char *str)
{
	int i,ct;

	if (*str == '(') {
		ct = 0;
		for (i = 0;i < (int)strlen(str);i++) {
			if (str[i] == '(') ct++;
			else if (str[i] == ')') ct--;
			if (ct == 0) {
				strncpy(u,str,i + 1);
				u[i + 1] = '\0';
				return;
			}
			if (*(str + i) == '#' && *(str + i + 1) == '\\' && *(str + i + 2) != '\0') i += 2;
                        else if (*(str + i) == '"') {
			    i += SkipDoubleQuote(str + i + 1) - (str + i) - 1;
			}
			if (isKanji((unsigned char *)(str + i))) i++;
		}
		printf("G[: Eʂ܂\n");
		*u = '\0';
		return;
	}
	else {
		for (i = 0;i < (int)strlen(str);i++) {
			if (str[i] == ' ' || str[i] == '(' || str[i] == ')') {
				strncpy(u,str,i);
				u[i] = '\0';
				return;
			}
			if (*(str + i) == '#' && *(str + i + 1) == '\\' && *(str + i + 2) != '\0') i += 2;
                        else if (*(str + i) == '"') {
			    i += SkipDoubleQuote(str + i + 1) - (str + i) - 1;
			}
			if (isKanji((unsigned char *)(str + i))) i++;
		}
		strcpy(u,str);
	}
}

void ctextproc::getCdrString(char *s,char *str)
{
	int ct,i;

	if (str[0] == '(') {
		ct = 0;
		for (i = 0;i < (int)strlen(str);i++) {
			if (str[i] == '(') ct++;
			else if (str[i] == ')') ct--;
			if (ct == 0) {
				strcpy(s,str + i + 1);
				return;
			}
			if (*(str + i) == '#' && *(str + i + 1) == '\\' && *(str + i + 2) != '\0') i += 2;
                        else if (*(str + i) == '"') {
			    i += SkipDoubleQuote(str + i + 1) - (str + i) - 1;
			}
			if (isKanji((unsigned char *)(str + i))) i++;
		}
		printf("G[: Eʂ܂\n");
                strcpy(s,"");
		return;
	}
	else {
		for (i = 0;i < (int)strlen(str);i++) {
			if (str[i] == ' ') {
				strcpy(s,str + i + 1);
				return;
			}
			else if (str[i] == '(' || str[i] == ')') {
				strcpy(s,str + i);
				return;
			}
			if (*(str + i) == '#' && *(str + i + 1) == '\\' && *(str + i + 2) != '\0') i += 2;
                        else if (*(str + i) == '"') {
			    i += SkipDoubleQuote(str + i + 1) - (str + i) - 1;
			}
			if (isKanji((unsigned char *)(str + i))) i++;
		}
		strcpy(s,"");
		return;
	}
}

int ctextproc::AnaType(char *u)
{
    char *p;

    if (!strcmp(u,"1+") || !strcmp(u,"1-")) return 0;
    if (u[0] == '.' && isdigit(u[1])) return 1;  // _
    if (u[0] == '-' && u[1] == '.' && isdigit(u[2])) return 1;  // _
    if (isdigit(u[0])) {
        p = u + 1;
        while (*p != '\0') {
            if (*p == '.') {
                if (isdigit(*(p + 1))) return 1;
                else break;
            }
            if (!isdigit(*p)) break;
            p++;
        }
    }
    if (u[0] == '-' && isdigit(u[1])) {
        p = u + 2;
        while (*p != '\0') {
            if (*p == '.') {
                if (isdigit(*(p + 1))) return 1;
                else break;
            }
            if (!isdigit(*p)) break;
            p++;
        }
    }
    if (isdigit(u[0])) {
        p = u + 1;
        while (*p != '\0') {
            if (*p == 'E') {
                if (isdigit(*(p + 1)) || *(p + 1) == '+' || *(p + 1) == '-') return 1;
                else break;
            }
            if (!isdigit(*p)) break;
            p++;
        }
    }
    if (u[0] == '-' && isdigit(u[1])) {
        p = u + 2;
        while (*p != '\0') {
            if (*p == 'E') {
                if (isdigit(*(p + 1)) || *(p + 1) == '+' || *(p + 1) == '-') return 1;
                else break;
            }
            if (!isdigit(*p)) break;
            p++;
        }
    }
    if (isdigit(u[0])) {
        p = u + 1;
        while (*p != '\0') {
            if (*p == '/' && isdigit(*(p + 1))) return 3; // 
            if (!isdigit(*p)) break;
            p++;
        }
        return 2; // 
    }
    if (u[0] == '-' && isdigit(u[1])) {
        p = u + 2;
        while (*p != '\0') {
            if (*p == '/' && isdigit(*(p + 1))) return 3; // 
            if (!isdigit(*p)) break;
            p++;
        }
        return 2; // 
    }
    if (u[0] == '"') return 4; // 
    if (u[0] == '#' && u[1] == '\\' && u[2] != '\0') return 5; // 
    return 0;
}

LONGDOUBLE ctextproc::myatof(char *b)
{
    LONGDOUBLE v;

    sscanf(b,"%Lg",&v);
    return v;
}

void ctextproc::getBunshi(char *u,char *re)
{
    while (*u != '/') *re++ = *u++;
    *re = '\0';
}

void ctextproc::getBunbo(char *u,char *re)
{
    while (*u++ != '/');
    strcpy(re,u);
}

CELL *cmemory::readFromString(char *str)
{
	char *s,*u,*e,*s1,*carS,*cdrS;
	CELL *p;
	LONGDOUBLE f;
        long val,vs,vb,nvs,nvb;
	AFLOAT *k;
        AINT *k1,*k2;
	int buflen;
        char bs[20],bb[20];
	char c1,c2;

//printf("str = %s\n",str);
	buflen = strlen(str) + 2;
//	buflen = buflen * 2;
	s = (char *)malloc(buflen);
	u = (char *)malloc(buflen);
	s1 = (char *)malloc(buflen);
	carS = (char *)malloc(buflen);
	cdrS = (char *)malloc(buflen);
	NewString(s,str);
	if (!isListString(s)) {
		getCarString(u,s);
                switch (AnaType(u)) {
                    case 1:
			f = myatof(u);
			p = getCell(); if (p == NULL) goto Error;
			p->attr = 'n';
			k = getNumber(f); if (k == NULL) goto Error;
			p->car = (CELL *)k;
                        break;
                    case 2:
			sscanf(u,"%ld",&val);
integer:
			p = getCell(); if (p == NULL) goto Error;
			p->attr = 'i';
			k2 = getInt(val); if (k2 == NULL) goto Error;
			p->car = (CELL *)k2;
                        break;
		    case 3:
                        getBunshi(u,bs);
			sscanf(bs,"%ld",&vs);
			if (vs == 0) { val = 0; goto integer; }
                        getBunbo(u,bb);
			sscanf(bb,"%ld",&vb);
			if (vb == 0) { printf("G[: 0Ŋ܂B\n"); goto Error; }
			yakubun(vs,vb,&nvs,&nvb);
                        if (nvb == 1) { val = nvs; goto integer; }
			p = getCell(); if (p == NULL) goto Error;
			p->attr = 'b';
			k1 = getInt(nvs); if (k1 == NULL) goto Error;
			k2 = getInt(nvb); if (k2 == NULL) goto Error;
			p->car = (CELL *)k1;
			p->cdr = (CELL *)k2;
			break;
		    case 4:
			p = getMojiretsuCell(u);
			if (p == NULL) goto Error;
			break;
		    case 5:
			c1 = u[2];
			if (isKanji((unsigned char *)&c1)) c2 = u[3];
			else c2 = (char)NULL;
			p = getMojiCell(c1,c2);
			if (p == NULL) goto Error;
			break;
                    default:
			p = getStringCell(u); if (p == NULL) goto Error;
                        break;
                }
		free(cdrS);
		free(carS);
		free(s1);
		free(u);
		free(s);
		return p;
	}
	else if (s[0] == '(' && s[1] == ')') {
		free(cdrS);
		free(carS);
		free(s1);
		free(u);
		free(s);
	        return NILCell;
	}
        else if (s[0] == '(' && s[1] == '.' && s[2] == ' ') {  // Add.
            e = s;
            e += 3;
            e = SkipSpc(e);
            p = readFromString(e);
	    free(cdrS);
	    free(carS);
	    free(s1);
	    free(u);
	    free(s);
	    return p;
        }
	else {
		strcpy(s1,s + 1);
		e = SkipSpc(s1);
		getCarString(carS,e);
		getCdrString(cdrS,e);
		p = getCell(); if (p == NULL) goto Error;
		p->attr = 'l';
		p->car = readFromString(carS);
	        if (p->car == NULL) goto Error;
		e = SkipSpc(cdrS);
		strcpy(s1,"(");
		strcat(s1,e);
		p->cdr = readFromString(s1);
        	if (p->cdr == NULL) goto Error;
		free(cdrS);
		free(carS);
		free(s1);
		free(u);
		free(s);

		return p;
	}
  Error:
		free(cdrS);
		free(carS);
		free(s1);
		free(u);
		free(s);
		return NULL;
}

void ctextproc::slide(char *b,int len)
{
	char *q;

	q = b;
	while (*q != '\0') q++;
	while (q != b) {
		*(q + len) = *q;
		q--;
	}
	*(q + len) = *q;
}

char *ctextproc::taiou(char *b)
{
	int ct;

	b = SkipSpc(b);
	while (*b == '\'' || *b == '`' || *b == ',' || *b == '@') b++;
	if (*b != '(') {
		while (*b != '\0' && *b != ')' && *b != ' ' && *b != '(') {
                    if (*b == '"') b = SkipDoubleQuote(b + 1);
                    else if (*b == '#' && *(b + 1) == '\\' && *(b + 2) != '\0') {
                        if (isKanji((unsigned char *)(b + 2))) b = b + 4;
			else b = b + 3;
		    }
                    else {
                        if (isKanji((unsigned char *)b)) b++;
                        b++;
                    }
                }
		return b;
	}
	else {
		ct = 0;
		for (;;) {
			if (*b == '(') ct++;
			else if (*b == ')') {
				ct--;
				if (ct == 0) return b + 1;
			}
                        if (*b == '"') b = SkipDoubleQuote(b + 1);
                        else if (*b == '#' && *(b + 1) == '\\' && *(b + 2) != '\0') {
                            if (isKanji((unsigned char *)(b + 2))) b = b + 4;
			    else b = b + 3;
		        }
                        else {
                            if (isKanji((unsigned char *)b)) b++;
                            b++;
                        }
		}
	}
}

int ctextproc::MakeQuoteText(char *b,int len)
{
	int plen;
	char *q;

	plen = strlen(b);
	if (plen >= len) { printf("G[: r܂\n"); return 0; }
	while (*b != '\0') {
		if (*b == '"') {
		    if ((b = SkipDoubleQuote(b + 1)) == NULL) return 0;
                    else continue;
                }
		else if (*b == '#' && *(b + 1) == '\\') {
		    b = b + 2;
                    if (isKanji((unsigned char *)b)) b++;
                    if (*b != '\0') b++;
                    continue;
                }
		if (*b == '\'') {
			*b++ = '('; plen += 6;
                        if (plen >= len) { printf("G[: r܂\n"); return 0; }
			slide(b,6); *b++ = 'Q'; *b++ = 'U'; *b++ = 'O'; *b++ = 'T'; *b++ = 'E'; *b = ' ';
			q = taiou(b);
			plen++; if (plen >= len) { printf("G[: r܂\n"); return 0; }
			slide(q,1);
			*q = ')';
		}
		else {
                    if (isKanji((unsigned char *)b)) b++;
                    b++;
                }
	}
	return 1;
}

int ctextproc::MakeFunctionText(char *b,int len)
{
	int plen;
	char *q;

	plen = strlen(b);
	if (plen >= len) { printf("G[: r܂\n"); return 0; }
	while (*b != '\0') {
		if (*b == '"') {
		    if ((b = SkipDoubleQuote(b + 1)) == NULL) return 0;
                    else continue;
                }
		else if (*b == '#' && *(b + 1) == '\\') {
		    b = b + 2;
                    if (isKanji((unsigned char *)b)) b++;
                    if (*b != '\0') b++;
                    continue;
                }
		if (*b == '#' && *(b + 1) == '\'') {
			*b++ = '('; plen += 8;
                        if (plen >= len) { printf("G[: r܂\n"); return 0; }
			slide(b,8);
			*b++ = 'F'; *b++ = 'U'; *b++ = 'N'; *b++ = 'C';
			*b++ = 'T'; *b++ = 'I'; *b++ = 'O'; *b++ = 'N'; *b = ' ';
			q = taiou(b);
			plen++; if (plen >= len) { printf("G[: r܂\n"); return 0; }
			slide(q,1);
			*q = ')';
		}
		else {
                    if (isKanji((unsigned char *)b)) b++;
                    b++;
                }
	}
	return 1;
}

int ctextproc::MakeBackQuoteText(char *b,int len)
{
	int plen;
	char *q;

	plen = strlen(b);
	if (plen >= len) { printf("G[: r܂\n"); return 0; }
	while (*b != '\0') {
		if (*b == '"') {
		    if ((b = SkipDoubleQuote(b + 1)) == NULL) return 0;
                    else continue;
                }
		else if (*b == '#' && *(b + 1) == '\\') {
		    b = b + 2;
                    if (isKanji((unsigned char *)b)) b++;
                    if (*b != '\0') b++;
                    continue;
                }
		if (*b == '`') {
			*b++ = '('; plen += 6 + 4;
                        if (plen >= len) { printf("G[: r܂\n"); return 0; }
			slide(b,6 + 4);
                        *b++ = 'B'; *b++ = 'A'; *b++ = 'C'; *b++ = 'K';
                        *b++ = 'Q'; *b++ = 'U'; *b++ = 'O'; *b++ = 'T'; *b++ = 'E'; *b = ' ';
			q = taiou(b);
			plen++; if (plen >= len) { printf("G[: r܂\n"); return 0; }
			slide(q,1);
			*q = ')';
		}
		else {
                    if (isKanji((unsigned char *)b)) b++;
                    b++;
                }
	}
	return 1;
}

int ctextproc::MakeCommaText(char *b,int len)
{
	int plen;
	char *q;

	plen = strlen(b);
	if (plen >= len) { printf("G[: r܂\n"); return 0; }
	while (*b != '\0') {
		if (*b == '"') {
		    if ((b = SkipDoubleQuote(b + 1)) == NULL) return 0;
                    else continue;
                }
		else if (*b == '#' && *(b + 1) == '\\') {
		    b = b + 2;
                    if (isKanji((unsigned char *)b)) b++;
                    if (*b != '\0') b++;
                    continue;
                }
		if (*b == ',') {
			*b++ = ','; plen += 1;
                        if (plen >= len) { printf("G[: r܂\n"); return 0; }
			slide(b,1);
                        *b = ' ';
		}
		else {
                    if (isKanji((unsigned char *)b)) b++;
                    b++;
                }
	}
	return 1;
}

int ctextproc::MakeAstText(char *b,int len)
{
	int plen;
	char *q;

	plen = strlen(b);
	if (plen >= len) { printf("G[: r܂\n"); return 0; }
	while (*b != '\0') {
		if (*b == '"') {
		    if ((b = SkipDoubleQuote(b + 1)) == NULL) return 0;
                    else continue;
                }
		else if (*b == '#' && *(b + 1) == '\\') {
		    b = b + 2;
                    if (isKanji((unsigned char *)b)) b++;
                    if (*b != '\0') b++;
                    continue;
                }
		if (*b == '@') {
			*b++ = '@'; plen += 1;
                        if (plen >= len) { printf("G[: r܂\n"); return 0; }
			slide(b,1);
                        *b = ' ';
		}
		else {
                    if (isKanji((unsigned char *)b)) b++;
                    b++;
                }
	}
	return 1;
}

int ctextproc::CrTabOnly(char *s)
{
    while (*s != '\0') {
        if (*s == '\n' || *s == '\r' || *s == '\t' || *s == ' ' || *s == '\'' || *s == '`' || *s == ',' || *s == '@');
        else if (*s == '(' || *s == ')') return 1;
        else if (*s == '"') {
            if (SkipDoubleQuote(s + 1)) return 0;
            else return 1;
        }
        else return 0;
        if (isKanji((unsigned char *)s)) s++;
        s++;
    }
    return 1;
}

int ctextproc::CheckBrace(char *s)
{
    int cnt = 0;
    char *str;

    str = s;
    while (*s != '\0') {
        if (*s == '(') cnt++;
        else if (*s == ')') {
            cnt--;
            if (cnt < 0) return -1;
            else if (cnt == 0) return 1;
        }
        if (*s == '"') {
            if ((s = SkipDoubleQuote(s + 1)) == NULL) return 0;
        }
        else if (*s == '#' && *(s + 1) == '\\' && *(s + 2) != '\0') {
            if (isKanji((unsigned char *)(s + 2))) s = s + 4;
            else s = s + 3;
        }
        else {
            if (isKanji((unsigned char *)s)) s++;
            s++;
        }
    }
    if (CrTabOnly(str)) return 0;
    return 1;
}

int ctextproc::CheckBrace2(char *s)
{
    int cnt = 0;
    char kep = '+';

    while (*s == '\n' || *s == '\r' || *s == '\t' || *s == ' ' || *s == '\'' || *s == '`' || *s == ',' || *s == '@')
	s++;
    if (*s == ')') return -1;
    if (*s != '(') {
        while (*s != '\n' && *s != '\r' && *s != '\t' /*&& *s != ' '*/) {
	    if (!isKanji((unsigned char *)&kep) && *s == ' ') break;
	    if (*s == '\0') return 0;
            if (!isKanji((unsigned char *)&kep)) kep = *s;
	    else kep = '+';
	    s++;
	}
	return 1;
    }
    while (*s != '\0') {
        if (*s == '(') cnt++;
        else if (*s == ')') {
            cnt--;
            if (cnt < 0) return -1;
            else if (cnt == 0) return 1;
        }
        if (*s == '"') {
            if ((s = SkipDoubleQuote2(s + 1)) == NULL) return 0;
        }
        else if (*s == '#' && *(s + 1) == '\\' && *(s + 2) != '\0') {
            if (isKanji((unsigned char *)(s + 2)) && *(s + 3) != '\0') s = s + 4;
            else s = s + 3;
        }
        else {
            if (isKanji((unsigned char *)s) && *(s + 1) != '\0') s++;
            s++;
        }
    }
    return 0;
}

int ctextproc::GetOneS(void)
{
    char LineBuf[LINE_BUF_SIZE];
    int Size;
    int r;

    Size = 0;
    LineBuf[0] = '\0';
    for (;;) {
        fgets(LineBuf,LINE_BUF_SIZE,stdin);
        Size += strlen(LineBuf);
        if (Size >= BUFLEN) {
            printf("G[: S߂܂B\n");
            return 0;
        }
        strcat(lineStr,LineBuf);
        r = CheckBrace(lineStr);
        if (r == 1) return 1;
        else if (r == -1) {
            printf("G[: )܂B\n");
            return 0;
        }
    }
}

BOOL ctextproc::AllBlank(char *s)
{
    while (*s != '\0') {
        if (!(*s == ' ' || *s == '\t' || *s == '\r' || *s == '\n')) return FALSE;
        s++;
    }
    return TRUE;
}

void ctextproc::SkipComment(FILE *fp)
{
    int c;

    for (;;) {
	c = getc(fp);
	if (c == -1) break;
	if (c == 0x0a) break;
    }
}

int ctextproc::GetOneSStream(FILE *fp)
{
    char LineBuf[LINE_BUF_SIZE];
    int Size;
    int r;
    int c;
    BOOL knj = FALSE;
    char c0;
    BOOL in_string = FALSE;
    int prev1 = 0,prev2 = 0;

    Size = 0;
    LineBuf[0] = '\0';
    for (;;) {
	if ((c = getc(fp)) == -1) {
	    if (Size == 0) /*return 0;*/ return 2;
	    else if (AllBlank(LineBuf)) /*return 0;*/ return 2;
	    else return 1;
	}
	if (!knj && c == '"' && !in_string) in_string = TRUE;
	else if (!knj && c == '"' && in_string) in_string = FALSE;
	if (!knj && c == ';' && !in_string && !(prev1 == '#' && prev2 == '\\')) {
	    SkipComment(fp);
	    continue;
	}
	if (knj) { prev1 = prev2 = 0; }
	else {
	    prev1 = prev2;
	    prev2 = c;
	}
	c0 = (char)c;
	if (knj) knj = FALSE;
	else if (!knj && isKanji((unsigned char *)&c0)) knj = TRUE;
	sprintf(LineBuf,"%c",(char)c);
        Size += strlen(LineBuf);
        if (Size >= BUFLEN) {
            printf("G[: S߂܂B\n");
            return 0;
        }
        strcat(lineStr,LineBuf);
        r = CheckBrace2(lineStr);
        if (r == 1) return 1;
        else if (r == -1) {
            printf("G[: )܂B\n");
            return 0;
        }
    }
}

void ctextproc::SkipTerm(char *s)
{
    while (*s != '\0') {
        if (*s == '"') {
            s = SkipDoubleQuote(s + 1);
        }
        else if (*s == '#' && *(s + 1) == '\\' && *(s + 2) != '\0') {
	    s += 2;
            if (isKanji((unsigned char *)s)) s++;
	    s++;
	}
        else {
            if (*s == '\n') *s = ' ';
	    else if (*s == '\r') *s = ' ';
            else if (*s == '\t') *s = ' ';
            if (isKanji((unsigned char *)s)) s++;
            else *s = toupper(*s);
            s++;
        }
    }
}

void ctextproc::myftoa(char *b,LONGDOUBLE v)
{
/*	char buf[30];
	int i,len;*/
        char *p;

//	sprintf(b,"%Lg",v);
	sprintf(b,"%.15Lg",v);
	p = b;
        while (*p != '\0') {
            if (*p == '.') return;
            else if (*p == 'e') return;
            p++;
        }
        strcat(b,".0");
}

void cprint::printS(CELL *p)
{
	char buf[30];
	int len;
	BOOL ef;

	if (p->attr == 's') {
		printf("%s",(char *)(p->car));
	}
	else if (p->attr == 'n') {
		myftoa(buf,(((AFLOAT *)(p->car))->f));
		printf("%s",buf);
	}
	else if (p->attr == 'i') {
                printf("%ld",((AINT *)(p->car))->f);
	}
	else if (p->attr == 'b') {
                printf("%ld/%ld",((AINT *)(p->car))->f,((AINT *)(p->cdr))->f);
	}
	else if (p->attr == 'r') {
                printf("%s",(char *)(p->car));
	}
	else if (p->attr == 'm') {
		printf("#\\%c",(char)(long)(p->car));
		if (p->cdr != NULL) printf("%c",(char)(long)(p->cdr));
	}
	else if (p->attr == 'l') {
		printf("(");
		if (p->cdr == NIL) ef = TRUE;
		else ef = FALSE;
		printS2(p,ef);
		printf(")");
	}
	else if (p->attr == 'a' && (p->car)->attr == 'i' &&
		 ((AINT *)((p->car)->car))->f == 1 && (p->cdr)->car == CHARACTER) {
		PutString(p->cdr->cdr,stdout);
	}
	else if (p->attr == 'a') {
		printf("#");
		if ((p->car)->attr == 'i' && ((AINT *)((p->car)->car))->f == 1) ;
		else printS(p->car);
		printf("A");
		printS((p->cdr)->cdr);
		printf(" ");
	}
}

void cprint::printS0(CELL *p,BOOL endflag)
{
	char buf[30];
	int len;
	BOOL ef;

	if (p->attr == 's') {
		printf("%s",(char *)(p->car));
		if (!endflag) printf(" ");
	}
	else if (p->attr == 'n') {
		myftoa(buf,(((AFLOAT *)(p->car))->f));
		printf("%s",buf);
		if (!endflag) printf(" ");
	}
	else if (p->attr == 'i') {
                printf("%ld",((AINT *)(p->car))->f);
		if (!endflag) printf(" ");
	}
	else if (p->attr == 'b') {
                printf("%ld/%ld",((AINT *)(p->car))->f,((AINT *)(p->cdr))->f);
		if (!endflag) printf(" ");
	}
	else if (p->attr == 'r') {
                printf("%s",(char *)(p->car));
		if (!endflag) printf(" ");
	}
	else if (p->attr == 'm') {
		printf("#\\%c",(char)(long)(p->car));
		if (p->cdr != NULL) printf("%c",(char)(long)(p->cdr));
		if (!endflag) printf(" ");
	}
	else if (p->attr == 'l') {
		printf("(");
		if (p->cdr == NIL) ef = TRUE;
		else ef = FALSE;
		printS2(p,ef);
		printf(")");
		if (!endflag) printf(" ");
	}
	else if (p->attr == 'a' && (p->car)->attr == 'i' &&
		 ((AINT *)((p->car)->car))->f == 1 && (p->cdr)->car == CHARACTER) {
		PutString(p->cdr->cdr,stdout);
		if (!endflag) printf(" ");	// Added in Apr. 2016
	}
	else if (p->attr == 'a') {
		printf("#");
		if ((p->car)->attr == 'i' && ((AINT *)((p->car)->car))->f == 1) ;
		else printS(p->car);
		printf("A");
		printS((p->cdr)->cdr);
		if (!endflag) printf(" ");
	}
}

void cprint::printS2(CELL *p,BOOL endflag)
{
	printS0(p->car,endflag);
	printS3(p->cdr);
}

void cprint::printS3(CELL *p)
{
	char b[30];
	int len;
	BOOL ef;

	if (p->attr == 's' && p != NILCell) {
		printf(". ");
		printf("%s",(char *)(p->car));
	}
	else if (p->attr == 'n') {
		myftoa(b,((AFLOAT *)(p->car))->f);
                printf(". %s",b);
	}
	else if (p->attr == 'i') {
                printf(". %ld",((AINT *)(p->car))->f);
	}
	else if (p->attr == 'b') {
                printf(". %ld/%ld",((AINT *)(p->car))->f,((AINT *)(p->cdr))->f);
	}
	else if (p->attr == 'r') {
                printf(". %s",(char *)(p->car));
	}
	else if (p->attr == 'm') {
		printf(". #\\%c",(char)(long)(p->car));
		if (p->cdr != NULL) printf("%c",(char)(long)(p->cdr));
	}
	else if (p->attr == 'l') {
		if (p->cdr == NIL) ef = TRUE;
		else ef = FALSE;
		printS2(p,ef);
	}
	else if (p->attr == 'a' && (p->car)->attr == 'i' &&
		 ((AINT *)((p->car)->car))->f == 1 && (p->cdr)->car == CHARACTER) {
		fprintf(stdout,". ");
		PutString(p->cdr->cdr,stdout);
	}
	else if (p->attr == 'a') {
		printf(". ");
		printf("#");
		if ((p->car)->attr == 'i' && ((AINT *)((p->car)->car))->f == 1) ;
		else printS(p->car);
		printf("A");
		printS((p->cdr)->cdr);
		printf(" ");
	}
}

void cprint::printSStrm(CELL *p,FILE *fp)
{
	char buf[30];
	int len;
	BOOL ef;

	if (p->attr == 's') {
		fprintf(fp,"%s",(char *)(p->car));
	}
	else if (p->attr == 'n') {
		myftoa(buf,(((AFLOAT *)(p->car))->f));
		fprintf(fp,"%s",buf);
	}
	else if (p->attr == 'i') {
                fprintf(fp,"%ld",((AINT *)(p->car))->f);
	}
	else if (p->attr == 'b') {
                fprintf(fp,"%ld/%ld",((AINT *)(p->car))->f,((AINT *)(p->cdr))->f);
	}
	else if (p->attr == 'r') {
                fprintf(fp,"%s",(char *)(p->car));
	}
	else if (p->attr == 'm') {
		fprintf(fp,"#\\%c",(char)(long)(p->car));
		if (p->cdr != NULL) printf("%c",(char)(long)(p->cdr));
	}
	else if (p->attr == 'l') {
		fprintf(fp,"(");
		if (p->cdr == NIL) ef = TRUE;
		else ef = FALSE;
		printS2Strm(p,ef,fp);
		fprintf(fp,")");
	}
	else if (p->attr == 'a' && (p->car)->attr == 'i' &&
		 ((AINT *)((p->car)->car))->f == 1 && (p->cdr)->car == CHARACTER) {
		PutString(p->cdr->cdr,fp);
	}
	else if (p->attr == 'a') {
		puts("G[: z̓t@Cɏo͂ł܂");
	}
}

void cprint::printS0Strm(CELL *p,BOOL endflag,FILE *fp)
{
	char buf[30];
	int len;
	BOOL ef;

	if (p->attr == 's') {
		fprintf(fp,"%s",(char *)(p->car));
		if (!endflag) fprintf(fp," ");
	}
	else if (p->attr == 'n') {
		myftoa(buf,(((AFLOAT *)(p->car))->f));
		fprintf(fp,"%s",buf);
		if (!endflag) fprintf(fp," ");
	}
	else if (p->attr == 'i') {
                fprintf(fp,"%ld",((AINT *)(p->car))->f);
		if (!endflag) fprintf(fp," ");
	}
	else if (p->attr == 'b') {
                fprintf(fp,"%ld/%ld",((AINT *)(p->car))->f,((AINT *)(p->cdr))->f);
		if (!endflag) fprintf(fp," ");
	}
	else if (p->attr == 'r') {
                fprintf(fp,"%s",(char *)(p->car));
		if (!endflag) fprintf(fp," ");
	}
	else if (p->attr == 'm') {
		fprintf(fp,"#\\%c",(char)(long)(p->car));
		if (p->cdr != NULL) fprintf(fp,"%c",(char)(long)(p->cdr));
		if (!endflag) fprintf(fp," ");
	}
	else if (p->attr == 'l') {
		fprintf(fp,"(");
		if (p->cdr == NIL) ef = TRUE;
		else ef = FALSE;
		printS2Strm(p,ef,fp);
		fprintf(fp,")");
		if (!endflag) fprintf(fp," ");
	}
	else if (p->attr == 'a' && (p->car)->attr == 'i' &&
		 ((AINT *)((p->car)->car))->f == 1 && (p->cdr)->car == CHARACTER) {
		PutString(p->cdr->cdr,fp);
		if (!endflag) fprintf(fp," ");
	}
	else if (p->attr == 'a') {
		puts("G[: z̓t@Cɏo͂ł܂");
	}
}

void cprint::printS2Strm(CELL *p,BOOL endflag,FILE *fp)
{
	printS0Strm(p->car,endflag,fp);
	printS3Strm(p->cdr,fp);
}

void cprint::printS3Strm(CELL *p,FILE *fp)
{
	char b[30];
	int len;
	BOOL ef;

	if (p->attr == 's' && p != NILCell) {
		fprintf(fp,". ");
		fprintf(fp,"%s",(char *)(p->car));
	}
	else if (p->attr == 'n') {
		myftoa(b,((AFLOAT *)(p->car))->f);
                fprintf(fp,". %s",b);
	}
	else if (p->attr == 'i') {
                fprintf(fp,". %ld",((AINT *)(p->car))->f);
	}
	else if (p->attr == 'b') {
                fprintf(fp,". %ld/%ld",((AINT *)(p->car))->f,((AINT *)(p->cdr))->f);
	}
	else if (p->attr == 'r') {
                fprintf(fp,". %s",(char *)(p->car));
	}
	else if (p->attr == 'm') {
		fprintf(fp,". #\\%c",(char)(long)(p->car));
		if (p->cdr != NULL) fprintf(fp,"%c",(char)(long)(p->cdr));
	}
	else if (p->attr == 'l') {
		if (p->cdr == NIL) ef = TRUE;
		else ef = FALSE;
		printS2Strm(p,ef,fp);
	}
	else if (p->attr == 'a' && (p->car)->attr == 'i' &&
		 ((AINT *)((p->car)->car))->f == 1 && (p->cdr)->car == CHARACTER) {
		fprintf(fp,". ");
		PutString(p->cdr->cdr,fp);
	}
	else if (p->attr == 'a') {
		puts("G[: z̓t@Cɏo͂ł܂");
	}
}

void ctextproc::princMojiretsu(char *s,FILE *fp)
{
    int i,len;

    s++;
    len = strlen(s) - 1;
    for (i = 0;i < len;i++) {
	if (isKanji((unsigned char *)s)) { fprintf(fp,"%c",*s++); fprintf(fp,"%c",*s++); i++; }
	else if (*s == '\\' && *(s + 1) == '\\') { fprintf(fp,"\\"); s += 2; i++; }
	else if (*s == '\\' && *(s + 1) == '\"') { fprintf(fp,"%c",*(s + 1)); s += 2; i++; }
	else { fprintf(fp,"%c",*s++); }
    }
}

void cprint::princS(CELL *p,FILE *fp)
{
	char buf[30];
	int len;
	BOOL ef;

	if (p->attr == 's') {
		fprintf(fp,"%s",(char *)(p->car));
	}
	else if (p->attr == 'n') {
		myftoa(buf,(((AFLOAT *)(p->car))->f));
		fprintf(fp,"%s",buf);
	}
	else if (p->attr == 'i') {
                fprintf(fp,"%ld",((AINT *)(p->car))->f);
	}
	else if (p->attr == 'b') {
                fprintf(fp,"%ld/%ld",((AINT *)(p->car))->f,((AINT *)(p->cdr))->f);
	}
	else if (p->attr == 'r') {
                princMojiretsu((char *)(p->car),fp);
	}
	else if (p->attr == 'm') {
		fprintf(fp,"%c",(char)(long)(p->car));
		if (p->cdr != NULL) fprintf(fp,"%c",(char)(long)(p->cdr));
	}
	else if (p->attr == 'l') {
		fprintf(fp,"(");
		if (p->cdr == NIL) ef = TRUE;
		else ef = FALSE;
		princS2(p,ef,fp);
		fprintf(fp,")");
	}
	else if (p->attr == 'a' && (p->car)->attr == 'i' &&
		 ((AINT *)((p->car)->car))->f == 1 && (p->cdr)->car == CHARACTER) {
		PutStringC(p->cdr->cdr,fp);
	}
	else if (p->attr == 'a') {
		fprintf(fp,"#");
		if ((p->car)->attr == 'i' && ((AINT *)((p->car)->car))->f == 1) ;
		else princS(p->car,fp);
		fprintf(fp,"A");
		princS((p->cdr)->cdr,fp);
		fprintf(fp," ");
	}
}

void cprint::princS0(CELL *p,BOOL endflag,FILE *fp)
{
	char buf[30];
	int len;
	BOOL ef;

	if (p->attr == 's') {
		fprintf(fp,"%s",(char *)(p->car));
		if (!endflag) fprintf(fp," ");
	}
	else if (p->attr == 'n') {
		myftoa(buf,(((AFLOAT *)(p->car))->f));
		fprintf(fp,"%s",buf);
		if (!endflag) fprintf(fp," ");
	}
	else if (p->attr == 'i') {
                fprintf(fp,"%ld",((AINT *)(p->car))->f);
		if (!endflag) fprintf(fp," ");
	}
	else if (p->attr == 'b') {
                fprintf(fp,"%ld/%ld",((AINT *)(p->car))->f,((AINT *)(p->cdr))->f);
		if (!endflag) fprintf(fp," ");
	}
	else if (p->attr == 'r') {
                princMojiretsu((char *)(p->car),fp);
		if (!endflag) fprintf(fp," ");
	}
	else if (p->attr == 'm') {
		fprintf(fp,"%c",(char)(long)(p->car));
		if (p->cdr != NULL) fprintf(fp,"%c",(char)(long)(p->cdr));
		if (!endflag) fprintf(fp," ");
	}
	else if (p->attr == 'l') {
		fprintf(fp,"(");
		if (p->cdr == NIL) ef = TRUE;
		else ef = FALSE;
		princS2(p,ef,fp);
		fprintf(fp,")");
		if (!endflag) fprintf(fp," ");
	}
	else if (p->attr == 'a' && (p->car)->attr == 'i' &&
		 ((AINT *)((p->car)->car))->f == 1 && (p->cdr)->car == CHARACTER) {
		PutStringC(p->cdr->cdr,fp);
		if (!endflag) fprintf(fp," ");
	}
	else if (p->attr == 'a') {
		fprintf(fp,"#");
		if ((p->car)->attr == 'i' && ((AINT *)((p->car)->car))->f == 1) ;
		else princS(p->car,fp);
		fprintf(fp,"A");
		princS((p->cdr)->cdr,fp);
		if (!endflag) fprintf(fp," ");
	}
}

void cprint::princS2(CELL *p,BOOL endflag,FILE *fp)
{
	princS0(p->car,endflag,fp);
	princS3(p->cdr,fp);
}

void cprint::princS3(CELL *p,FILE *fp)
{
	char b[30];
	int len;
	BOOL ef;

	if (p->attr == 's' && p != NILCell) {
		fprintf(fp,". ");
		fprintf(fp,"%s",(char *)(p->car));
	}
	else if (p->attr == 'n') {
		myftoa(b,((AFLOAT *)(p->car))->f);
                fprintf(fp,". %s",b);
	}
	else if (p->attr == 'i') {
                fprintf(fp,". %ld",((AINT *)(p->car))->f);
	}
	else if (p->attr == 'b') {
                fprintf(fp,". %ld/%ld",((AINT *)(p->car))->f,((AINT *)(p->cdr))->f);
	}
	else if (p->attr == 'r') {
                fprintf(fp,". "); princMojiretsu((char *)(p->car),fp);
	}
	else if (p->attr == 'm') {
		fprintf(fp,". %c",(char)(long)(p->car));
		if (p->cdr != NULL) fprintf(fp,"%c",(char)(long)(p->cdr));
	}
	else if (p->attr == 'l') {
		if (p->cdr == NIL) ef = TRUE;
		else ef = FALSE;
		princS2(p,ef,fp);
	}
	else if (p->attr == 'a' && (p->car)->attr == 'i' &&
		 ((AINT *)((p->car)->car))->f == 1 && (p->cdr)->car == CHARACTER) {
		fprintf(fp,". ");
		PutStringC(p->cdr->cdr,fp);
	}
	else if (p->attr == 'a') {
		fprintf(fp,". ");
		fprintf(fp,"#");
		if ((p->car)->attr == 'i' && ((AINT *)((p->car)->car))->f == 1) ;
		else princS(p->car,fp);
		fprintf(fp,"A");
		princS((p->cdr)->cdr,fp);
		fprintf(fp," ");
	}
}

/*
void DumpString(void)
{
    long i = 1;
    char *p;

    p = strArea;
    while (p != curStr) {
        printf("%d. %s\n",i,p);
        while (*p != '\0') p++;
        p++;
        i++;
    }
}
*/

long cmemory::getSymVal2(CELL *s)
{
	SYMVAL *p;
	long i;

	p = sv;
	i = 0;
	while (p != curSv) {
		if (p->sym == s) return i;
		p++;
		i++;
	}
        if (curSv - sv >= GLOBAL_SIZE - 1) {
            printf("G[: O[oϐ̈悪sĂ܂BI܂B\n");
	    exit(1);
	    return -1;
	}
	curSv->sym = s;
	curSv++;
	return i;
}

long cmemory::getSymVal(CELL *s)
{
	SYMVAL *p;
	long i;

	p = sv;
	i = 0;
	while (p != curSv) {
		if (p->sym == s) return i;
		p++;
		i++;
	}
	return -1L;
}

CELL *cfunction::LastCell(CELL *p)
{
    if ((p->cdr)->attr != 'l') return p;
    return LastCell(p->cdr);
}

CELL *cfunction::Copy(CELL *p)
{
    CELL *r;

    if (p->attr != 'l') return p;
    r = getCell();
    if (r == NULL) return NULL;
    r->attr = 'l';
    r->car = Copy(p->car);
    r->cdr = Copy(p->cdr);
    return r;
}

CELL *cfunction::backquote2(CELL *p,SYMVAL *loc,long locn,long lock)
{
    CELL *r,*q,*e;

    if (p->attr == 'l' && p->car == COMMA && (p->cdr)->attr == 'l' && (p->cdr)->car == AST) {

	IncCellStack(NULL);		// Add in Apr.2016
	IncCellStack(p);		// Add in Apr.2016

	q = eval(((p->cdr)->cdr)->car,loc,locn,lock);

	PopCellStack();			// Add in Apr.2016

	if (q == NULL) return NULL;
        if (q == NILCell) return backquote2(((p->cdr)->cdr)->cdr,loc,locn,lock);
        if (q->attr != 'l') {
            printf("G[: XgłȂ̂͘Ał܂B\n");
	    errorlisp(loc,locn);
            return NULL;
	}
        q = Copy(q);
        if (q == NULL) return NULL;
        e = LastCell(q);

	IncCellStack(NULL);		// Add in Apr.2016
	IncCellStack(q);		// Add in Apr.2016
	IncCellStack(e);		// Add in Apr.2016

        e->cdr = backquote2(((p->cdr)->cdr)->cdr,loc,locn,lock);

	PopCellStack();			// Add in Apr.2016

        return q;
    }
    else if (p->attr == 'l' && p->car == COMMA) {
        if ((p->cdr)->attr != 'l') {
	    printf("G[: ,̌ɒl܂B\n");
	    errorlisp(loc,locn);
	    return NULL;
	}
        r = getCell();
        if (r == NULL) return NULL;
        r->attr = 'l';

	IncCellStack(NULL);		// Add in Apr.2016
	IncCellStack(p);		// Add in Apr.2016
	IncCellStack(r);		// Add in Apr.2016

        r->car = eval((p->cdr)->car,loc,locn,lock);
        if (r->car == NULL) { PopCellStack(); return NULL; }
        r->cdr = backquote2((p->cdr)->cdr,loc,locn,lock);

	PopCellStack();			// Add in Apr.2016

        return r;
    }
    else if (p->attr == 'l') {
        r = getCell();
        if (r == NULL) return NULL;
        r->attr = 'l';

	IncCellStack(NULL);		// Add in Apr.2016
	IncCellStack(p);		// Add in Apr.2016
	IncCellStack(r);		// Add in Apr.2016

        r->car = backquote2(p->car,loc,locn,lock);
        if (r->car == NULL) { PopCellStack(); return NULL; }
        r->cdr = backquote2(p->cdr,loc,locn,lock);

	PopCellStack();			// Add in Apr.2016

        if (r->cdr == NULL) return NULL;
        return r;
    }
    else {
        return p;
    }
}

CELL *cfunction::backquote(CELL *p,SYMVAL *loc,long locn,long lock)
{
    CELL *q,*r;
    char flag;

    if (p->attr == 'l' && p->car == COMMA && (p->cdr)->attr == 'l' && (p->cdr)->car == AST) {
        printf("G[: @Ⴂ܂B\n");
	errorlisp(loc,locn);
        return NULL;
    }
    if (p->attr == 'l' && p->car == COMMA && (p->cdr)->attr == 'l' && ((p->cdr)->cdr)->attr == 's') {
        q = eval((p->cdr)->car,loc,locn,lock);
        return q;
    }
    else if (p->attr == 'l') {
        return backquote2(p->car,loc,locn,lock);
    }
    else {
        printf("G[: @Ⴂ܂B\n");
	errorlisp(loc,locn);
        return NULL;
    }
}

SYMVAL *cmemory::getFuncval(CELL *s)
{
	SYMVAL *p;

	p = fv;
	while (p != curFv) {
		if (p->sym == s) return p;
		p++;
	}
	if (curFv - fv >= FUNCTION_SIZE - 1) {
		printf("G[: ֐̈悪tɂȂ܂BI܂B\n");
		exit(1);
		return NULL;
	}
	curFv++;
	return p;
}

SYMVAL *cmemory::getFuncval2(CELL *s)
{
	SYMVAL *p;

	p = fv;
	while (p != curFv) {
		if (p->sym == s) return p;
		p++;
	}
	return NULL;
}

long cfunction::tlength(CELL *p)
{
	long i;

	i = 0;
	for (;;) {
		if (p->attr != 'l') return i;
		i++;
		p = p->cdr;
	}
}

CELL *cfunction::MakeRest(CELL *v,SYMVAL *loc,long locn,long lock)
{
    CELL *p;

    if (v->attr != 'l') return v;
    if ((p = getCell()) == NULL) return NULL;;
    p->attr = 'l';

    IncCellStack(NULL);		// Add in Apr.2016
    IncCellStack(p);		// Add in Apr.2016
    IncCellStack(v);		// Add in Apr.2016

    p->car = eval(v->car,loc,locn,lock);
    if (p->car == NULL) { PopCellStack(); return NULL; }
    p->cdr = MakeRest(v->cdr,loc,locn,lock);

    PopCellStack();		// Add in Apr.2016

    if (p->cdr == NULL) return NULL;
    return p;
}

BOOL cfunction::initTemploc(SYMVAL *l,// temploc
                 long len, // ̒
                 CELL *s,  // 
                 CELL *v,  // 
                 SYMVAL *loc,
                 long locn,
		 long lock )
{
	CELL *p;
	long i;
        long k = -1;

	if (l == NULL || len == 0) return TRUE; // Add on 2014.2.25
//	if (len != tlength(v)) return FALSE;
	p = s;
	for (i = 0;i < len;i++) {
                if (p->car == AMPREST) {
                    k = i;
                    if (!(i + 1 < len)) return FALSE;
                    p = p->cdr;
                }
		if ((p->car)->attr != 's') return FALSE;
		l[i].sym = p->car;
		l[i].val = NILCell;
		p = p->cdr;
                if (k != -1) {
                    break;
                }
	}
	if (k != -1) l[len - 1].sym = NULL;    // Add.
        if (p->attr != 's') return FALSE;
//printf("here\n");
        if (k != -1) len--;

	IncCellStack(NULL);
	IncCellStack(v);

	for (i = 0;i < len;i++) {
		if (i != k && v->attr != 'l') { PopCellStack(); return FALSE; }

		//IncCellStack(NULL);		// Add in Apr.2016
		//IncCellStack(v);		// Add in Apr.2016

                if (i == k) {
                    l[i].val = MakeRest(v,loc,locn,lock);
                    if (l[i].val == NULL) { PopCellStack(); return FALSE; }
                }
		else if ((l[i].val = eval(v->car,loc,locn,lock)) == NULL) { PopCellStack(); return FALSE; }
//static long kazu = 0; if (kazu++ > 50000000) { printS(v->car); printf("\n"); }
		//PopCellStack();			// Add in Apr.2016
		IncCellStack(l[i].val);
//printS(v);
		v = v->cdr;
//printS(v);
	}
	PopCellStack();

        if (k == -1 && v->attr != 's') { return FALSE; }
//for (i = 0; i < len;i++) printf("%x ",l[i].sym);
	return TRUE;
}

CELL *cfunction::tprogn(CELL *s,SYMVAL *loc,long locn,long lock)
{
	CELL *p;

	p = NILCell;

	IncCellStack(NULL);		// Add in Apr.2016
	IncCellStack(s);		// Add in Apr.2016

	for (;;) {
		if (s->attr != 'l') { PopCellStack(); return p; }
		if ((p = eval(s->car,loc,locn,lock)) == NULL) { PopCellStack(); return NULL; }
		s = s->cdr;
	}
}

CELL *cfunction::tprog1(CELL *s,SYMVAL *loc,long locn,long lock)
{
	CELL *p,*ret;
	BOOL First = TRUE;

	ret = NILCell;

	IncCellStack(NULL);		// Add in Apr.2016
	IncCellStack(s);		// Add in Apr.2016
	IncCellStack(ret);		// Add in Apr.2016

	for (;;) {
		if (s->attr != 'l') { PopCellStack(); return ret; }
		if ((p = eval(s->car,loc,locn,lock)) == NULL) { PopCellStack(); return NULL; }
		if (First) {
		    ret = p;
		    IncCellStack(ret);
		    First = FALSE;
		}
		s = s->cdr;
	}
}

#if 0
long cmemory::getLocal(SYMVAL *loc,long locn,CELL *s,long lock)
{
	long i;
	if (lock == 0) lock++;
	for (i = lock - 1/*0*/;i < locn;i++) {
		if ((loc + i)->sym != NULL && s == (loc + i)->sym) return i;
        }
	return -1;
}
#else
long cmemory::getLocal(SYMVAL *loc,long locn,CELL *s,long lock)
{
	long i;

	for (i = 0;i < locn;i++) {
		if ((loc + i)->sym != NULL && s == (loc + i)->sym) return i;
        }
	return -1;
}
#endif

/*
void DumpSymval(SYMVAL *loc,long locn)
{
    long i;

    for (i = 0;i < locn;i++) {
//	printf("attr = %c\n",loc[i].attr);
	if (loc[i].sym != NULL) {
            printS(loc[i].sym); printf(" ");
            printS(loc[i].val); printf("\n");
	}
    }
}
*/

CELL *cfunction::Let(CELL *p,SYMVAL *loc,long locn,long lock)
{
    long len,i,j,newlen;
    SYMVAL *temploc;
    CELL *q,*r;

    if (p->attr != 'l') goto letserror;
    len = tlength(p->car);
    if (lock < 0) lock -= len;
    else if (lock > 0) lock += len;
    newlen = len + locn;
    if (len != 0) temploc = (SYMVAL *)calloc(newlen,sizeof(SYMVAL));
    else return tprogn(p->cdr,loc,locn,lock);
    if (temploc == NULL) {
	printf("G[: Ǐϐ̊mۂɎs܂B\n");
	errorlisp(loc,locn);
	return NULL;
    }
    j = 0;
    for (i = len;i < newlen;i++) {
        temploc[i].attr = loc[j].attr;
        temploc[i].sym = loc[j].sym;
        temploc[i].val = loc[j++].val;
    }
    q = p->car;

    IncCellStack(NULL);
    IncCellStack(q);

    for (i = 0;i < len;i++) {
        r = q->car;
        if (r->attr != 'l') { PopCellStack(); goto letserror2; }
        if ((r->car)->attr != 's') { PopCellStack(); goto letserror2; }
        temploc[i].sym = r->car;
        if ((r->cdr)->attr != 'l') temploc[i].val = NILCell;
        else {
            temploc[i].val = eval((r->cdr)->car,loc,locn,0);
            if (temploc[i].val == NULL) { free(temploc); PopCellStack(); return NULL; }
        }
	IncCellStack(temploc[i].val);
        q = q->cdr;
    }

    PopCellStack();

    IncLocStack(temploc,newlen);
//DumpSymval(temploc,newlen);
    q = tprogn(p->cdr,temploc,newlen,lock);
    PopLocStack();
    SetOrgLoc(loc,locn,temploc,len,newlen);
/* Deleted in Apr.2016
    if (q != NULL) IncCellStack(q);
*/
    if (q != NULL && q->cljlen != 0) {
	IncStaticStack(temploc,newlen);
	return q;  // Added on 2014.3.1
    }

    free(temploc);
    return q;

letserror2:
    free(temploc);
letserror:
    printf("G[: @Ⴂ܂B\n");
    errorlisp(loc,locn);
    return NULL;
}

BOOL cmemory::initTemplocForMacro(SYMVAL *l,// temploc
                 long len, // ̒
                 CELL *s,  // 
                 CELL *v ) // 
{
	CELL *p;
	long i;
        long k = -1;

	p = s;
	for (i = 0;i < len;i++) {
                if (p->car == AMPREST) {
                    k = i;
                    if (!(i + 1 < len)) return FALSE;
                    p = p->cdr;
                }
                else if (p->car == AMPBODY) {
                    k = i;
                    if (!(i + 1 < len)) return FALSE;
                    p = p->cdr;
                }
		if ((p->car)->attr != 's') return FALSE;
		l[i].sym = p->car;
		p = p->cdr;
                if (k != -1) {
                    break;
                }
	}
        if (p->attr != 's') return FALSE;
        if (k != -1) len--;
	for (i = 0;i < len;i++) {
		if (i != k && v->attr != 'l') return FALSE;
                if (i == k) {
                    l[i].val = v;
//                    if (l[i].val == NULL) return FALSE;
                }
		else {
		    l[i].val = v->car;
                }
		v = v->cdr;
	}
        if (k == -1 && v->attr != 's') return FALSE;
        if (k != -1) l[i].sym = NULL;
	return TRUE;
}

#if 0
CELL *cfunction::MacroExec(CELL *p,SYMVAL *m,SYMVAL *loc,long locn)
{
    long len,i,j,newlen,lock;
    SYMVAL *temploc;
    CELL *q;

    lock = len = tlength((m->val)->car);
    lock *= -1;
    lock--;
    newlen = len + locn;
    if (newlen != 0) {
	temploc = (SYMVAL *)calloc(newlen,sizeof(SYMVAL));
	if (temploc == NULL) {
	    printf("G[: [Jϐ̈̊mۂɎs܂\n");
	    return NULL;
	}
    }
    else temploc = NULL;
    if (!initTemplocForMacro(temploc,len,(m->val)->car,p)) {
	printf("G[: ̐Ⴂ܂\n");
	free(temploc);
	return NULL;
    }
    j = 0;
    for (i = len;i < newlen;i++) {
        temploc[i].attr = loc[j].attr;
        temploc[i].sym = loc[j].sym;
        temploc[i].val = loc[j++].val;
    }
    if (((m->val)->cdr)->attr != 'l') {
        free(temploc);
        return NILCell;
    }
    IncLocStack(temploc,newlen);
//DumpSymval(temploc,newlen);
    if ((q = eval(((m->val)->cdr)->car,temploc,newlen,lock)) == NULL) return NULL;
    PopLocStack();
    free(temploc);
    return eval(q,loc,locn,0);
}

CELL *cfunction::MacroExec2(CELL *p,SYMVAL *m,SYMVAL *loc,long locn,long lock)
{
    long len,i,j,newlen;
    SYMVAL *temploc;
    CELL *q;

    len = tlength((m->val)->car);
    newlen = len + locn;
    if (newlen != 0) {
	temploc = (SYMVAL *)calloc(newlen,sizeof(SYMVAL));
	if (temploc == NULL) {
	    printf("G[: [Jϐ̈̊mۂɎs܂\n");
	    return NULL;
	}
    }
    else temploc = NULL;
    if (!initTemplocForMacro(temploc,len,(m->val)->car,p)) {
	printf("G[: ̐Ⴂ܂\n");
	free(temploc);
	return NULL;
    }
    j = 0;
    for (i = len;i < newlen;i++) {
        temploc[i].attr = loc[j].attr;
        temploc[i].sym = loc[j].sym;
        temploc[i].val = loc[j++].val;
    }
    if (((m->val)->cdr)->attr != 'l') {
        free(temploc);
        return NILCell;
    }
    IncLocStack(temploc,newlen);
    if ((q = eval(((m->val)->cdr)->car,temploc,newlen,lock)) == NULL) return NULL;
    PopLocStack();
    free(temploc);
    return q;
}
#else
CELL *cfunction::MacroExec(CELL *p,SYMVAL *m,SYMVAL *loc,long locn)
{
    long len,i,j,newlen,lock;
    SYMVAL *temploc;
    CELL *q;

    lock = len = tlength((m->val)->car);
    lock *= -1;
    lock--;
//    newlen = len + locn;
    newlen = len;	// 22 June 2017
    if (newlen != 0) {
	temploc = (SYMVAL *)calloc(newlen,sizeof(SYMVAL));
	if (temploc == NULL) {
	    printf("G[: [Jϐ̈̊mۂɎs܂\n");
	    errorlisp(loc,locn);
	    return NULL;
	}
    }
    else temploc = NULL;
    if (!initTemplocForMacro(temploc,len,(m->val)->car,p)) {
	printf("G[: ̐Ⴂ܂\n");
	free(temploc);
	errorlisp(loc,locn);
	return NULL;
    }
    j = 0;
    for (i = len;i < newlen;i++) {
        temploc[i].attr = loc[j].attr;
        temploc[i].sym = loc[j].sym;
        temploc[i].val = loc[j++].val;
    }
    if (((m->val)->cdr)->attr != 'l') {
        free(temploc);
        return NILCell;
    }
    IncLocStack(temploc,newlen);
//DumpSymval(temploc,newlen);
    if ((q = eval(((m->val)->cdr)->car,temploc,newlen,lock)) == NULL) return NULL;
    PopLocStack();
    free(temploc);
    return eval(q,loc,locn,0);
}

CELL *cfunction::MacroExec2(CELL *p,SYMVAL *m,SYMVAL *loc,long locn,long lock)
{
    long len,i,j,newlen;
    SYMVAL *temploc;
    CELL *q;

    len = tlength((m->val)->car);
//    newlen = len + locn;
    newlen = len;	// 22 june 2017
    if (newlen != 0) {
	temploc = (SYMVAL *)calloc(newlen,sizeof(SYMVAL));
	if (temploc == NULL) {
	    printf("G[: [Jϐ̈̊mۂɎs܂\n");
	    errorlisp(loc,locn);
	    return NULL;
	}
    }
    else temploc = NULL;
    if (!initTemplocForMacro(temploc,len,(m->val)->car,p)) {
	printf("G[: ̐Ⴂ܂\n");
	free(temploc);
	errorlisp(loc,locn);
	return NULL;
    }
    j = 0;
    for (i = len;i < newlen;i++) {
        temploc[i].attr = loc[j].attr;
        temploc[i].sym = loc[j].sym;
        temploc[i].val = loc[j++].val;
    }
    if (((m->val)->cdr)->attr != 'l') {
        free(temploc);
        return NILCell;
    }
    IncLocStack(temploc,newlen);
    if ((q = eval(((m->val)->cdr)->car,temploc,newlen,lock)) == NULL) return NULL;
    PopLocStack();
    free(temploc);
    return q;
}
#endif

CELL *cfunction::LetAst(CELL *p,SYMVAL *loc,long locn,long lock)
{
    CELL *q,*r;
    long len,i,j;
    SYMVAL *temploc;

    if (p->attr != 'l') goto letastserror;
    q = p->car;
    if (q->attr != 'l') goto letastserror;
    len = tlength(q);

    if (lock < 0) lock -= len;
    else if (lock > 0) lock += len;

    if (len == 0) return tprogn(p->cdr,loc,locn,lock);
    temploc = (SYMVAL *)calloc(len + locn,sizeof(SYMVAL));
    if (temploc == NULL) {
	printf("G[: Ǐϐ̊mۂɎs܂B\n");
	errorlisp(loc,locn);
	return NULL;
    }
    j = 0;
    for (i = len;i < len + locn;i++) {
        temploc[i].attr = loc[j].attr;
        temploc[i].sym = loc[j].sym;
	temploc[i].val = loc[j++].val;
    }
    for (i = 0;i < len;i++)
        temploc[i].sym = NULL;
    IncCellStack(NULL);
    IncCellStack(q);
    for (i = 0;i < len;i++) {
        r = q->car;
        if (r->attr != 'l') { PopCellStack(); goto letastserror2; }
        if ((r->car)->attr != 's') { PopCellStack(); goto letastserror2; }
        temploc[i].sym = r->car;
        if ((r->cdr)->attr != 'l') temploc[i].val = NILCell;
        else {
            temploc[i].val = eval((r->cdr)->car,temploc,len + locn,lock);
            if (temploc[i].val == NULL) { PopCellStack(); free(temploc); return NULL; }
        }
	IncCellStack(temploc[i].val);
        q = q->cdr;
    }
    PopCellStack();
    IncLocStack(temploc,len + locn);
//DumpSymval(temploc,len + locn);
    q = tprogn(p->cdr,temploc,len + locn,lock);
/* Deleted in Apr.2016
    if (q != NULL) IncCellStack(q);
*/
    PopLocStack();
    SetOrgLoc(loc,locn,temploc,len,len + locn);

    if (q != NULL && q->cljlen != 0) {
	IncStaticStack(temploc,len + locn);
	return q;  // Added on 2014.3.1
    }

    free(temploc);
    return q;

letastserror2:
    free(temploc);
letastserror:
    printf("G[: @Ⴂ܂B\n");
    errorlisp(loc,locn);
    return NULL;
}

int cfunction::progBody2(CELL *p,SYMVAL *loc,long locn,long lock)
{
    CELL *q,*r;

    r = NILCell;
    for (;;) {
        if (p->attr != 'l') break;
        q = p->car;
        if (q->attr == 's') {
            p = p->cdr;
            continue;
        }
        ReturnFlag = 'u';
//printS(q);

	IncCellStack(NULL);	// Added in Apr.2016
	IncCellStack(p);	// Added in Apr.2016

        if (eval(q,loc,locn,lock) == NULL) return 2;

	PopCellStack();		// Added in Apr.2016

        if (ReturnFlag != 'u') {
            ReturnFlag = 'u';
            return 3;
        }
        if (GoSymbol != NULL) return 0;
        p = p->cdr;
    }
    ReturnValue = r;
    return 1;
}

CELL *cfunction::tmember(CELL *key,CELL *ob)
{
    while (ob->attr == 'l') {
        if (key == ob->car) return ob;
        ob = ob->cdr;
    }
    return NILCell;
}

CELL *cfunction::progBody(CELL *p,SYMVAL *loc,long locn,long lock)
{
    CELL *q,*r;
    int ret;

    GoSymbol = NULL;
    for (;;) {
        if (GoSymbol == NULL) q = p;
        else {
            q = tmember(GoSymbol,p);
            if (q == NILCell) {
		printf("G[: x܂B\n");
		errorlisp(loc,locn);
                return NULL;
            }
            GoSymbol = NULL;
        }

	IncCellStack(NULL);	// Added in Apr.2016
	IncCellStack(p);	// Added in Apr.2016

        ret = progBody2(q,loc,locn,lock);

	PopCellStack();		// Added in Apr.2016

        if (ret == 1 || ret == 3) break;
        else if (ret == 2) return NULL;
    }
    return ReturnValue;
}

CELL *cfunction::prog(CELL *p,SYMVAL *loc,long locn,long lock)
{
    CELL *q;
    long len,i,j;
    SYMVAL *temploc;

    if (p->attr != 'l') goto progserror;
    q = p->car;
    len = tlength(q);
    if (len == 0) return progBody(p->cdr,loc,locn,lock);
    temploc = (SYMVAL *)calloc(len + locn,sizeof(SYMVAL));
    if (temploc == NULL) {
	printf("G[: Ǐϐ̊mۂɎs܂B\n");
	errorlisp(loc,locn);
	return NULL;
    }
    j = 0;
    for (i = len;i < len + locn;i++)
        temploc[i] = loc[j++];
    for (i = 0;i < len;i++) {
        if ((q->car)->attr != 's') goto progserror2;
        temploc[i].sym = q->car;
        temploc[i].val = NILCell;
        q = q->cdr;
    }
    IncLocStack(temploc,len + locn);

    if (lock < 0) lock -= len;
    else if (lock > 0) lock += len;

    q = progBody(p->cdr,temploc,len + locn,lock);
    SetOrgLoc(loc,locn,temploc,len,len + locn);
    PopLocStack();
    free(temploc);
    return q;

progserror2:
    free(temploc);
progserror:
    printf("G[: @Ⴂ܂B\n");
    errorlisp(loc,locn);
    return NULL;
}

CELL *cfunction::tand(CELL *p,SYMVAL *loc,long locn,long lock)
{
	CELL *q;

	for (;;) {
		if (p->attr != 'l') { q->mtop = NULL; return q; }

		IncCellStack(NULL);	// Added in Apr.2016
		IncCellStack(p);	// Added in Apr.2016

		if ((q = eval(p->car,loc,locn,lock)) == NULL) return NULL;

		PopCellStack();		// Added in Apr.2016

		if (q == NILCell) return NILCell;
		p = p->cdr;
	}
}

CELL *cfunction::tor(CELL *p,SYMVAL *loc,long locn,long lock)
{
	CELL *q;

	for (;;) {
		if (p->attr != 'l') return NILCell;

		IncCellStack(NULL);
		IncCellStack(p);

		if ((q = eval(p->car,loc,locn,lock)) == NULL) { PopCellStack(); return NULL; }

		PopCellStack();

		if (q != NILCell) { q->mtop = NULL; return q; }
		p = p->cdr;
	}
}

void cmathproc::WaWithTsubun(long a,long b,long c,long d,long *e,long *f)
{
    long bunshi,bunbo;

    bunbo = b * d;
    bunshi = a * d + b * c;
    yakubun(bunshi,bunbo,e,f);
}

CELL *cfunction::Wa(CELL *x,CELL *y)
{
    AINT *a,*a2;
    CELL *p;
    AFLOAT *b;
    long bunshi,bunbo;

    if (x->attr == 'i' && y->attr == 'i') {
	a = getInt(((AINT *)(x->car))->f + ((AINT *)(y->car))->f);
	if (a == NULL) return NULL;
	p = getCell();
	if (p == NULL) return NULL;
	p->attr = 'i';
	p->car = (CELL *)a;
	return p;
    }
    if (x->attr == 'n' && y->attr == 'n') {
	b = getNumber(((AFLOAT *)(x->car))->f + ((AFLOAT *)(y->car))->f);
	if (b == NULL) return NULL;
	p = getCell();
	if (p == NULL) return NULL;
	p->attr = 'n';
	p->car = (CELL *)b;
	return p;
    }
    if (x->attr == 'i' && y->attr == 'n') {
	b = getNumber((LONGDOUBLE)(((AINT *)(x->car))->f) + ((AFLOAT *)(y->car))->f);
	if (b == NULL) return NULL;
	p = getCell();
	if (p == NULL) return NULL;
	p->attr = 'n';
	p->car = (CELL *)b;
	return p;
    }
    if (x->attr == 'n' && y->attr == 'i') {
	b = getNumber(((AFLOAT *)(x->car))->f + (LONGDOUBLE)(((AINT *)(y->car))->f));
	if (b == NULL) return NULL;
	p = getCell();
	if (p == NULL) return NULL;
	p->attr = 'n';
	p->car = (CELL *)b;
	return p;
    }
    if (x->attr == 'b' && y->attr == 'b') {
	WaWithTsubun(((AINT *)(x->car))->f,((AINT *)(x->cdr))->f,((AINT *)(y->car))->f,((AINT *)(y->cdr))->f,
		     &bunshi,&bunbo );
	if (bunbo == 1L) {
	    a = getInt(bunshi);
	    if (a == NULL) return NULL;
	    p = getCell();
	    if (p == NULL) return NULL;
	    p->attr = 'i';
	    p->car = (CELL *)a;
	    return p;
	}
	a = getInt(bunshi);
	if (a == NULL) return NULL;
	a2 = getInt(bunbo);
	if (a2 == NULL) return NULL;
	p = getCell();
	if (p == NULL) return NULL;
	p->attr = 'b';
	p->car = (CELL *)a;
	p->cdr = (CELL *)a2;
	return p;
    }
    if (x->attr == 'b' && y->attr == 'n') {
	b = getNumber((LONGDOUBLE)(((AINT *)(x->car))->f) / (LONGDOUBLE)(((AINT *)(x->cdr))->f) +
		      ((AFLOAT *)(y->car))->f );
	if (b == NULL) return NULL;
	p = getCell();
	if (p == NULL) return NULL;
	p->attr = 'n';
	p->car = (CELL *)b;
	return p;
    }
    if (x->attr == 'n' && y->attr == 'b') {
	b = getNumber((LONGDOUBLE)(((AINT *)(y->car))->f) / (LONGDOUBLE)(((AINT *)(y->cdr))->f) +
		      ((AFLOAT *)(x->car))->f );
	if (b == NULL) return NULL;
	p = getCell();
	if (p == NULL) return NULL;
	p->attr = 'n';
	p->car = (CELL *)b;
	return p;
    }
    if (x->attr == 'b' && y->attr == 'i') {
	WaWithTsubun(((AINT *)(x->car))->f,((AINT *)(x->cdr))->f,((AINT *)(y->car))->f,1L,&bunshi,&bunbo);
	if (bunbo == 1L) {
	    a = getInt(bunshi);
	    if (a == NULL) return NULL;
	    p = getCell();
	    if (p == NULL) return NULL;
	    p->attr = 'i';
	    p->car = (CELL *)a;
	    return p;
	}
	a = getInt(bunshi);
	if (a == NULL) return NULL;
	a2 = getInt(bunbo);
	if (a2 == NULL) return NULL;
	p = getCell();
	if (p == NULL) return NULL;
	p->attr = 'b';
	p->car = (CELL *)a;
	p->cdr = (CELL *)a2;
	return p;
    }
    if (x->attr == 'i' && y->attr == 'b') {
	WaWithTsubun(((AINT *)(y->car))->f,((AINT *)(y->cdr))->f,((AINT *)(x->car))->f,1L,&bunshi,&bunbo);
	if (bunbo == 1L) {
	    a = getInt(bunshi);
	    if (a == NULL) return NULL;
	    p = getCell();
	    if (p == NULL) return NULL;
	    p->attr = 'i';
	    p->car = (CELL *)a;
	    return p;
	}
	a = getInt(bunshi);
	if (a == NULL) return NULL;
	a2 = getInt(bunbo);
	if (a2 == NULL) return NULL;
	p = getCell();
	if (p == NULL) return NULL;
	p->attr = 'b';
	p->car = (CELL *)a;
	p->cdr = (CELL *)a2;
	return p;
    }
    printf("G[: ͐lł͂܂B\n");
    return NULL;
}

CELL *cfunction::Sa(CELL *x,CELL *y)
{
    AINT *a,*a2;
    CELL *p;
    AFLOAT *b;
    long bunshi,bunbo;

    if (x->attr == 'i' && y->attr == 'i') {
	a = getInt(((AINT *)(x->car))->f - ((AINT *)(y->car))->f);
	if (a == NULL) return NULL;
	p = getCell();
	if (p == NULL) return NULL;
	p->attr = 'i';
	p->car = (CELL *)a;
	return p;
    }
    if (x->attr == 'n' && y->attr == 'n') {
	b = getNumber(((AFLOAT *)(x->car))->f - ((AFLOAT *)(y->car))->f);
	if (b == NULL) return NULL;
	p = getCell();
	if (p == NULL) return NULL;
	p->attr = 'n';
	p->car = (CELL *)b;
	return p;
    }
    if (x->attr == 'i' && y->attr == 'n') {
	b = getNumber((LONGDOUBLE)(((AINT *)(x->car))->f) - ((AFLOAT *)(y->car))->f);
	if (b == NULL) return NULL;
	p = getCell();
	if (p == NULL) return NULL;
	p->attr = 'n';
	p->car = (CELL *)b;
	return p;
    }
    if (x->attr == 'n' && y->attr == 'i') {
	b = getNumber(((AFLOAT *)(x->car))->f - (LONGDOUBLE)(((AINT *)(y->car))->f));
	if (b == NULL) return NULL;
	p = getCell();
	if (p == NULL) return NULL;
	p->attr = 'n';
	p->car = (CELL *)b;
	return p;
    }
    if (x->attr == 'b' && y->attr == 'b') {
	WaWithTsubun(((AINT *)(x->car))->f,((AINT *)(x->cdr))->f,((AINT *)(y->car))->f * (-1L),((AINT *)(y->cdr))->f,
		     &bunshi,&bunbo );
	if (bunbo == 1L) {
	    a = getInt(bunshi);
	    if (a == NULL) return NULL;
	    p = getCell();
	    if (p == NULL) return NULL;
	    p->attr = 'i';
	    p->car = (CELL *)a;
	    return p;
	}
	a = getInt(bunshi);
	if (a == NULL) return NULL;
	a2 = getInt(bunbo);
	if (a2 == NULL) return NULL;
	p = getCell();
	if (p == NULL) return NULL;
	p->attr = 'b';
	p->car = (CELL *)a;
	p->cdr = (CELL *)a2;
	return p;
    }
    if (x->attr == 'b' && y->attr == 'n') {
	b = getNumber((LONGDOUBLE)(((AINT *)(x->car))->f) / (LONGDOUBLE)(((AINT *)(x->cdr))->f) +
		      ((AFLOAT *)(y->car))->f * (-1L) );
	if (b == NULL) return NULL;
	p = getCell();
	if (p == NULL) return NULL;
	p->attr = 'n';
	p->car = (CELL *)b;
	return p;
    }
    if (x->attr == 'n' && y->attr == 'b') {
	b = getNumber((LONGDOUBLE)(((AINT *)(y->car))->f * (-1L)) / (LONGDOUBLE)(((AINT *)(y->cdr))->f) +
		      ((AFLOAT *)(x->car))->f );
	if (b == NULL) return NULL;
	p = getCell();
	if (p == NULL) return NULL;
	p->attr = 'n';
	p->car = (CELL *)b;
	return p;
    }
    if (x->attr == 'b' && y->attr == 'i') {
	WaWithTsubun(((AINT *)(x->car))->f,((AINT *)(x->cdr))->f,((AINT *)(y->car))->f * (-1L),1L,&bunshi,&bunbo);
	if (bunbo == 1L) {
	    a = getInt(bunshi);
	    if (a == NULL) return NULL;
	    p = getCell();
	    if (p == NULL) return NULL;
	    p->attr = 'i';
	    p->car = (CELL *)a;
	    return p;
	}
	a = getInt(bunshi);
	if (a == NULL) return NULL;
	a2 = getInt(bunbo);
	if (a2 == NULL) return NULL;
	p = getCell();
	if (p == NULL) return NULL;
	p->attr = 'b';
	p->car = (CELL *)a;
	p->cdr = (CELL *)a2;
	return p;
    }
    if (x->attr == 'i' && y->attr == 'b') {
	WaWithTsubun(((AINT *)(y->car))->f * (-1L),((AINT *)(y->cdr))->f,((AINT *)(x->car))->f,1L,&bunshi,&bunbo);
	if (bunbo == 1L) {
	    a = getInt(bunshi);
	    if (a == NULL) return NULL;
	    p = getCell();
	    if (p == NULL) return NULL;
	    p->attr = 'i';
	    p->car = (CELL *)a;
	    return p;
	}
	a = getInt(bunshi);
	if (a == NULL) return NULL;
	a2 = getInt(bunbo);
	if (a2 == NULL) return NULL;
	p = getCell();
	if (p == NULL) return NULL;
	p->attr = 'b';
	p->car = (CELL *)a;
	p->cdr = (CELL *)a2;
	return p;
    }
    printf("G[: ͐lł͂܂B\n");
    return NULL;
}

CELL *cfunction::Seki(CELL *x,CELL *y)
{
    AINT *a,*a2;
    CELL *p;
    AFLOAT *b;
    long bunshi,bunbo,oldbunshi,oldbunbo;

    if (x->attr == 'i' && y->attr == 'i') {
	a = getInt(((AINT *)(x->car))->f * ((AINT *)(y->car))->f);
	if (a == NULL) return NULL;
	p = getCell();
	if (p == NULL) return NULL;
	p->attr = 'i';
	p->car = (CELL *)a;
	return p;
    }
    if (x->attr == 'n' && y->attr == 'n') {
	b = getNumber(((AFLOAT *)(x->car))->f * ((AFLOAT *)(y->car))->f);
	if (b == NULL) return NULL;
	p = getCell();
	if (p == NULL) return NULL;
	p->attr = 'n';
	p->car = (CELL *)b;
	return p;
    }
    if (x->attr == 'i' && y->attr == 'n') {
	if (((AINT *)(x->car))->f == 0L) {
	    a = getInt(0L);
	    p = getCell();
	    p->attr = 'i';
	    p->car = (CELL *)a;
	    return p;
	}
	b = getNumber((LONGDOUBLE)(((AINT *)(x->car))->f) * ((AFLOAT *)(y->car))->f);
	if (b == NULL) return NULL;
	p = getCell();
	if (p == NULL) return NULL;
	p->attr = 'n';
	p->car = (CELL *)b;
	return p;
    }
    if (x->attr == 'n' && y->attr == 'i') {
	if (((AINT *)(y->car))->f == 0L) {
	    a = getInt(0L);
	    p = getCell();
	    p->attr = 'i';
	    p->car = (CELL *)a;
	    return p;
	}
	b = getNumber(((AFLOAT *)(x->car))->f * (LONGDOUBLE)(((AINT *)(y->car))->f));
	if (b == NULL) return NULL;
	p = getCell();
	if (p == NULL) return NULL;
	p->attr = 'n';
	p->car = (CELL *)b;
	return p;
    }
    if (x->attr == 'b' && y->attr == 'b') {
        oldbunshi = ((AINT *)(x->car))->f * ((AINT *)(y->car))->f;
        oldbunbo = ((AINT *)(x->cdr))->f * ((AINT *)(y->cdr))->f;
        yakubun(oldbunshi,oldbunbo,&bunshi,&bunbo);
	if (bunbo == 1L) {
	    a = getInt(bunshi);
	    if (a == NULL) return NULL;
	    p = getCell();
	    if (p == NULL) return NULL;
	    p->attr = 'i';
	    p->car = (CELL *)a;
	    return p;
	}
	a = getInt(bunshi);
	if (a == NULL) return NULL;
	a2 = getInt(bunbo);
	if (a2 == NULL) return NULL;
	p = getCell();
	if (p == NULL) return NULL;
	p->attr = 'b';
	p->car = (CELL *)a;
	p->cdr = (CELL *)a2;
	return p;
    }
    if (x->attr == 'b' && y->attr == 'n') {
	b = getNumber((LONGDOUBLE)(((AINT *)(x->car))->f) / (LONGDOUBLE)(((AINT *)(x->cdr))->f) *
		      ((AFLOAT *)(y->car))->f );
	if (b == NULL) return NULL;
	p = getCell();
	if (p == NULL) return NULL;
	p->attr = 'n';
	p->car = (CELL *)b;
	return p;
    }
    if (x->attr == 'n' && y->attr == 'b') {
	b = getNumber((LONGDOUBLE)(((AINT *)(y->car))->f) / (LONGDOUBLE)(((AINT *)(y->cdr))->f) *
		      ((AFLOAT *)(x->car))->f );
	if (b == NULL) return NULL;
	p = getCell();
	if (p == NULL) return NULL;
	p->attr = 'n';
	p->car = (CELL *)b;
	return p;
    }
    if (x->attr == 'b' && y->attr == 'i') {
	if (((AINT *)(y->car))->f == 0L) {
	    a = getInt(0L);
	    p = getCell();
	    p->attr = 'i';
	    p->car = (CELL *)a;
	    return p;
	}
        oldbunshi = ((AINT *)(x->car))->f * ((AINT *)(y->car))->f;
        oldbunbo = ((AINT *)(x->cdr))->f;

        yakubun(oldbunshi,oldbunbo,&bunshi,&bunbo);
	if (bunbo == 1L) {
	    a = getInt(bunshi);
	    if (a == NULL) return NULL;
	    p = getCell();
	    if (p == NULL) return NULL;
	    p->attr = 'i';
	    p->car = (CELL *)a;
	    return p;
	}

	a = getInt(bunshi);
	if (a == NULL) return NULL;
	a2 = getInt(bunbo);
	if (a2 == NULL) return NULL;
	p = getCell();
	if (p == NULL) return NULL;
	p->attr = 'b';
	p->car = (CELL *)a;
	p->cdr = (CELL *)a2;
	return p;
    }
    if (x->attr == 'i' && y->attr == 'b') {
	if (((AINT *)(x->car))->f == 0L) {
	    a = getInt(0L);
	    p = getCell();
	    p->attr = 'i';
	    p->car = (CELL *)a;
	    return p;
	}
        oldbunshi = ((AINT *)(y->car))->f * ((AINT *)(x->car))->f;
        oldbunbo = ((AINT *)(y->cdr))->f;

        yakubun(oldbunshi,oldbunbo,&bunshi,&bunbo);
	if (bunbo == 1L) {
	    a = getInt(bunshi);
	    if (a == NULL) return NULL;
	    p = getCell();
	    if (p == NULL) return NULL;
	    p->attr = 'i';
	    p->car = (CELL *)a;
	    return p;
	}

	a = getInt(bunshi);
	if (a == NULL) return NULL;
	a2 = getInt(bunbo);
	if (a2 == NULL) return NULL;
	p = getCell();
	if (p == NULL) return NULL;
	p->attr = 'b';
	p->car = (CELL *)a;
	p->cdr = (CELL *)a2;
	return p;
    }
    printf("G[: ͐lł͂܂B\n");
    return NULL;
}

CELL *cfunction::Sho(CELL *x,CELL *y)
{
    AINT *a,*a2;
    CELL *p;
    AFLOAT *b;
    long bunshi,bunbo,oldbunshi,oldbunbo;

    if (x->attr == 'i' && y->attr == 'i') {
	if (((AINT *)(y->car))->f == 0L) { printf("G[: 0ŏZ܂B\n"); return NULL; }
	oldbunshi = ((AINT *)(x->car))->f;
	if (oldbunshi == 0L) {
	    p = getCell();
	    p->attr = 'i';
	    p->car = (CELL *)getInt(0L);
	    return p;
	}
	oldbunbo = ((AINT *)(y->car))->f;
	yakubun(oldbunshi,oldbunbo,&bunshi,&bunbo);
	if (bunbo == 1L) {
	    p = getCell();
	    p->attr = 'i';
	    p->car = (CELL *)getInt(bunshi);
	    return p;
	}
	a = getInt(bunshi);
	a2 = getInt(bunbo);
	p = getCell();
	if (p == NULL) return NULL;
	p->attr = 'b';
	p->car = (CELL *)a;
	p->cdr = (CELL *)a2;
	return p;
    }
    if (x->attr == 'n' && y->attr == 'n') {
	if (((AFLOAT *)(y->car))->f == 0.0) { printf("G[: 0ŏZ܂B\n"); return NULL; }
	b = getNumber(((AFLOAT *)(x->car))->f / ((AFLOAT *)(y->car))->f);
	if (b == NULL) return NULL;
	p = getCell();
	if (p == NULL) return NULL;
	p->attr = 'n';
	p->car = (CELL *)b;
	return p;
    }
    if (x->attr == 'i' && y->attr == 'n') {
	if (((AFLOAT *)(y->car))->f == 0.0) { printf("G[: 0ŏZ܂B\n"); return NULL; }
	if (((AINT *)(x->car))->f == 0L) {
	    a = getInt(0L);
	    p = getCell();
	    p->attr = 'i';
	    p->car = (CELL *)a;
	    return p;
	}
	b = getNumber((LONGDOUBLE)(((AINT *)(x->car))->f) / ((AFLOAT *)(y->car))->f);
	if (b == NULL) return NULL;
	p = getCell();
	if (p == NULL) return NULL;
	p->attr = 'n';
	p->car = (CELL *)b;
	return p;
    }
    if (x->attr == 'n' && y->attr == 'i') {
	if (((AINT *)(y->car))->f == 0L) { printf("G[: 0ŏZ܂B\n"); return NULL; }
	b = getNumber(((AFLOAT *)(x->car))->f / (LONGDOUBLE)(((AINT *)(y->car))->f));
	if (b == NULL) return NULL;
	p = getCell();
	if (p == NULL) return NULL;
	p->attr = 'n';
	p->car = (CELL *)b;
	return p;
    }
    if (x->attr == 'b' && y->attr == 'b') {
        oldbunshi = ((AINT *)(x->car))->f * ((AINT *)(y->cdr))->f;
        oldbunbo = ((AINT *)(x->cdr))->f * ((AINT *)(y->car))->f;
        yakubun(oldbunshi,oldbunbo,&bunshi,&bunbo);
	if (bunbo == 1L) {
	    a = getInt(bunshi);
	    if (a == NULL) return NULL;
	    p = getCell();
	    if (p == NULL) return NULL;
	    p->attr = 'i';
	    p->car = (CELL *)a;
	    return p;
	}
	a = getInt(bunshi);
	if (a == NULL) return NULL;
	a2 = getInt(bunbo);
	if (a2 == NULL) return NULL;
	p = getCell();
	if (p == NULL) return NULL;
	p->attr = 'b';
	p->car = (CELL *)a;
	p->cdr = (CELL *)a2;
	return p;
    }
    if (x->attr == 'b' && y->attr == 'n') {
	if (((AFLOAT *)(y->car))->f == 0.0) { printf("G[: 0ŏZ܂B\n"); return NULL; }
	b = getNumber((LONGDOUBLE)(((AINT *)(x->car))->f) / (LONGDOUBLE)(((AINT *)(x->cdr))->f) /
		      ((AFLOAT *)(y->car))->f );
	if (b == NULL) return NULL;
	p = getCell();
	if (p == NULL) return NULL;
	p->attr = 'n';
	p->car = (CELL *)b;
	return p;
    }
    if (x->attr == 'n' && y->attr == 'b') {
	b = getNumber((LONGDOUBLE)(((AINT *)(y->cdr))->f) / (LONGDOUBLE)(((AINT *)(y->car))->f) *
		      ((AFLOAT *)(x->car))->f );
	if (b == NULL) return NULL;
	p = getCell();
	if (p == NULL) return NULL;
	p->attr = 'n';
	p->car = (CELL *)b;
	return p;
    }
    if (x->attr == 'b' && y->attr == 'i') {
	if (((AINT *)(y->car))->f == 0L) { printf("G[: 0ŏZ܂B\n"); return NULL; }
        oldbunshi = ((AINT *)(x->car))->f;
        oldbunbo = ((AINT *)(x->cdr))->f * ((AINT *)(y->car))->f;

        yakubun(oldbunshi,oldbunbo,&bunshi,&bunbo);
	if (bunbo == 1L) {
	    a = getInt(bunshi);
	    if (a == NULL) return NULL;
	    p = getCell();
	    if (p == NULL) return NULL;
	    p->attr = 'i';
	    p->car = (CELL *)a;
	    return p;
	}

	a = getInt(bunshi);
	if (a == NULL) return NULL;
	a2 = getInt(bunbo);
	if (a2 == NULL) return NULL;
	p = getCell();
	if (p == NULL) return NULL;
	p->attr = 'b';
	p->car = (CELL *)a;
	p->cdr = (CELL *)a2;
	return p;
    }
    if (x->attr == 'i' && y->attr == 'b') {
	if (((AINT *)(x->car))->f == 0L) {
	    a = getInt(0L);
	    p = getCell();
	    p->attr = 'i';
	    p->car = (CELL *)a;
	    return p;
	}
        oldbunshi = ((AINT *)(y->cdr))->f * ((AINT *)(x->car))->f;
        oldbunbo = ((AINT *)(y->car))->f;

        yakubun(oldbunshi,oldbunbo,&bunshi,&bunbo);
	if (bunbo == 1L) {
	    a = getInt(bunshi);
	    if (a == NULL) return NULL;
	    p = getCell();
	    if (p == NULL) return NULL;
	    p->attr = 'i';
	    p->car = (CELL *)a;
	    return p;
	}
	a = getInt(bunshi);
	if (a == NULL) return NULL;
	a2 = getInt(bunbo);
	if (a2 == NULL) return NULL;
	p = getCell();
	if (p == NULL) return NULL;
	p->attr = 'b';
	p->car = (CELL *)a;
	p->cdr = (CELL *)a2;
	return p;
    }
    printf("G[: ͐lł͂܂B\n");
    return NULL;
}

CELL *cfunction::plus(CELL *s,SYMVAL *loc,long locn,long lock)
{
	CELL *k;
	AFLOAT *f;
	CELL *p;
	AINT *a;

        a = getInt(0L);
	if (a == NULL) return NULL;
	k = getCell();
	if (k == NULL) return NULL;
	k->attr = 'i';
	k->car = (CELL *)a;

	for (;;) {
		if (s->attr != 'l') break;

		IncCellStack(NULL);	// Added in Apr.2016
		IncCellStack(k);	// Added in Apr.2016
		IncCellStack(s);

		if ((p = eval(s->car,loc,locn,lock)) == NULL) { PopCellStack(); return NULL; }

		PopCellStack();		// Added in Apr.2016

		k = Wa(k,p);
		if (k == NULL) {
		    errorlisp(loc,locn);
		    return NULL;
		}
		s = s->cdr;
	}
	return k;
}

CELL *cfunction::OnePlus(CELL *s,SYMVAL *loc,long locn,long lock)
{
    long k;
    AINT *a,*a2;
    CELL *b;
    LONGDOUBLE c;
    AFLOAT *d;
    long bunshi,bunbo;

    s = eval(s,loc,locn,lock);
    if (s == NULL) return NULL;
    if (s->attr == 'i') {
        k = ((AINT *)(s->car))->f + 1;
        a = getInt(k);
        if (a == NULL) return NULL;
        b = getCell();
	if (b == NULL) return NULL;
        b->attr = 'i';
        b->car = (CELL *)a;
        return b;
    }
    if (s->attr == 'n') {
        c = ((AFLOAT *)(s->car))->f + 1.0;
        d = getNumber(c);
        if (d == NULL) return NULL;
        b = getCell();
	if (b == NULL) return NULL;
        b->attr = 'n';
        b->car = (CELL *)d;
        return b;
    }
    if (s->attr == 'b') {
	WaWithTsubun(1,1,((AINT *)(s->car))->f,((AINT *)(s->cdr))->f,&bunshi,&bunbo);
	if (bunbo == 1) {
            a = getInt(bunshi);
            if (a == NULL) return NULL;
	    b = getCell();
 	    if (b == NULL) return NULL;
	    b->attr = 'i';
	    b->car = (CELL *)a;
	    return b;
	}
        a = getInt(bunshi);
        if (a == NULL) return NULL;
        a2 = getInt(bunbo);
        if (a2 == NULL) return NULL;
        b = getCell();
	if (b == NULL) return NULL;
	b->attr = 'b';
	b->car = (CELL *)a;
	b->cdr = (CELL *)a2;
	return b;
    }
    else {
        printf("G[: Ώۂlł͂܂B\n");
	errorlisp(loc,locn);
        return NULL;
    }
}

CELL *cfunction::OneMinus(CELL *s,SYMVAL *loc,long locn,long lock)
{
    long k;
    AINT *a,*a2;
    CELL *b;
    LONGDOUBLE c;
    AFLOAT *d;
    long bunshi,bunbo;

    s = eval(s,loc,locn,lock);
    if (s == NULL) return NULL;
    if (s->attr == 'i') {
        k = ((AINT *)(s->car))->f - 1;
        a = getInt(k);
        if (a == NULL) return NULL;
        b = getCell();
	if (b == NULL) return NULL;
        b->attr = 'i';
        b->car = (CELL *)a;
        return b;
    }
    if (s->attr == 'n') {
        c = ((AFLOAT *)(s->car))->f - 1.0;
        d = getNumber(c);
        if (d == NULL) return NULL;
        b = getCell();
	if (b == NULL) return NULL;
        b->attr = 'n';
        b->car = (CELL *)d;
        return b;
    }
    if (s->attr == 'b') {
	WaWithTsubun(-1,1,((AINT *)(s->car))->f,((AINT *)(s->cdr))->f,&bunshi,&bunbo);
	if (bunbo == 1) {
            a = getInt(bunshi);
            if (a == NULL) return NULL;
	    b = getCell();
 	    if (b == NULL) return NULL;
	    b->attr = 'i';
	    b->car = (CELL *)a;
	    return b;
	}
        a = getInt(bunshi);
        if (a == NULL) return NULL;
        a2 = getInt(bunbo);
        if (a2 == NULL) return NULL;
        b = getCell();
	if (b == NULL) return NULL;
	b->attr = 'b';
	b->car = (CELL *)a;
	b->cdr = (CELL *)a2;
	return b;
    }
    else {
        printf("G[: Ώۂlł͂܂B\n");
	errorlisp(loc,locn);
        return NULL;
    }
}

CELL *cfunction::condition(CELL *s,SYMVAL *loc,long locn,long lock)
{
	CELL *p,*u;

	for (;;) {
		if (s->attr != 'l') return NILCell;
		u = s->car;
		if (u->attr != 'l') return NILCell;

		IncCellStack(NULL);
		IncCellStack(u);
		IncCellStack(s);

		if ((p = eval(u->car,loc,locn,lock)) == NULL) { PopCellStack(); return NULL; }

		PopCellStack();

		if (p != NILCell) return tprogn(u->cdr,loc,locn,lock);
		s = s->cdr;
	}
}

CELL *cfunction::tlist(CELL *p,SYMVAL *loc,long locn,long lock)
{
	CELL *s,*q,*r,*x;

	if (p->attr != 'l') return NILCell;
	if ((s = getCell()) == NULL) return NULL;
	q = s;
	for (;;) {

		IncCellStack(NULL);	// Added in Apr.2016
		IncCellStack(q);	// Added in Apr.2016

		if ((x = eval(p->car,loc,locn,lock)) == NULL) return NULL;

		PopCellStack();		// Added in Apr.2016

		q->attr = 'l';
		q->car = x;
		p = p->cdr;
		if (p->attr != 'l') { q->cdr = NILCell; return s; }
		if ((r = getCell()) == NULL) return NULL;
		q->cdr = r;
		q = r;
	}
}

CELL *cfunction::tlist2(CELL *p,SYMVAL *loc,long locn,CELL **u,long lock)
{
	CELL *s,*q,*r,*x;
        BOOL First = TRUE;

	if (p->attr != 'l') return NILCell;
	if ((s = getCell()) == NULL) return NULL;
	q = s;
	for (;;) {

		IncCellStack(NULL);
		IncCellStack(q);
		IncCellStack(p);
		if (!First) IncCellStack(*u);

		if ((x = eval(p->car,loc,locn,lock)) == NULL) { PopCellStack(); return NULL; }

		PopCellStack();

                if (First) {
		    *u = x;
		    First = FALSE;
		}
		q->attr = 'l';
		q->car = x;
		p = p->cdr;
		if (p->attr != 'l') { q->cdr = NILCell; return s; }
		if ((r = getCell()) == NULL) return NULL;
		q->cdr = r;
		q = r;
	}
}

CELL *cfunction::values(CELL *p,SYMVAL *loc,long locn,long lock)
{
    CELL *q,*r;

    q = getCell();

    IncCellStack(NULL);
    IncCellStack(q);

    q->mtop = tlist2(p,loc,locn,&r,lock);

    PopCellStack();

    if (q->mtop == NULL) return NULL;
    q->attr = r->attr;
    q->attr2 = r->attr2;
    q->car = r->car;
    q->cdr = r->cdr;
    return q;
}

CELL *cfunction::IfClause(CELL *p,SYMVAL *loc,long locn,long lock)
{
    CELL *q,*r;

    if (p->attr != 'l') goto serror;
    q = p->car;

    IncCellStack(NULL);	// Added in Apr.2016
    IncCellStack(p);	// Added in Apr.2016

    r = eval(q,loc,locn,lock);

    PopCellStack();		// Added in Apr.2016

    if (r == NULL) return NULL;
    p = p->cdr;
    if (r != NILCell) {
        if (p->attr != 'l') goto serror;
        return eval(p->car,loc,locn,lock);
    }
    p = p->cdr;
    if (p->attr != 'l') return NILCell;
    return eval(p->car,loc,locn,lock);

serror:
    printf("G[: @Ⴂ܂B\n");
    errorlisp(loc,locn);
    return NULL;
}

FILE *cfunction::getStream(CELL *p)
{
    int a;

    a = (int)(((AINT *)(p->car))->f);
    if (a < 1 || FILE_POINTER_LEN < a) return NULL;
    return FilePointer[a - 1];
}

FILE *cfunction::getStream2(CELL *p)
{
    int a;
    FILE *fp;

    a = (int)(((AINT *)(p->car))->f);
    if (a < 1 || FILE_POINTER_LEN < a) return NULL;
    fp = FilePointer[a - 1];
    FilePointer[a - 1] = NULL;
    return fp;
}

CELL *cfunction::tclose(CELL *p,SYMVAL *loc,long locn,long lock)
{
    FILE *fp;

    p = eval(p,loc,locn,lock);
    if (p == NULL) return NULL;
    if (p->attr != 'i') goto serror; //return NULL;
    if (p->attr2 != 's') goto serror; //return NULL;
    fp = getStream2(p);
    if (fp == NULL) goto serror; //return NULL;
    fclose(fp);
    return TCell;

serror:
    puts("G[: t@CN[Ył܂");
    errorlisp(loc,locn);
    return NULL;
}

CELL *cfunction::tread(CELL *p,SYMVAL *loc,long locn,long lock)
{
    CELL *top;
    CELL *q;
    BOOL errmode;
    CELL *ret;
    FILE *fp;
    BOOL owari;
    int v;

    if (p->attr != 'l') {
        lineStr[0] = '\0';
        if (GetOneS()) {
            SkipTerm(lineStr);
            if (!MakeFunctionText(lineStr,BUFLEN - 1)) return NULL;
            if (!MakeQuoteText(lineStr,BUFLEN - 1)) return NULL;
            if (!MakeBackQuoteText(lineStr,BUFLEN - 1)) return NULL;
            if (!MakeCommaText(lineStr,BUFLEN - 1)) return NULL;
            if (!MakeAstText(lineStr,BUFLEN - 1)) return NULL;
            top = readFromString(lineStr);
            if (top == NULL) return NULL;
	    return top;
        }
	errorlisp(loc,locn);
        return NULL;
    }
    else {

	IncCellStack(NULL);
	IncCellStack(p);

        q = eval(p->car,loc,locn,lock);

	PopCellStack();

        if (q == NULL) return NULL;
	if (q->attr2 != 's') {
	    puts("G[: Xg[ł͂܂");
	    errorlisp(loc,locn);
	    return NULL;
	}
	fp = getStream(q);
	if (fp == NULL) {
	    puts("G[: ̃Xg[͖ł");
	    errorlisp(loc,locn);
	    return NULL;
	}
	p = p->cdr;
	if (p->attr != 'l') {
	    errmode = TRUE;
	}
        else {

	    IncCellStack(NULL);
	    IncCellStack(p);

	    q = eval(p->car,loc,locn,lock);

	    PopCellStack();

	    if (q == NULL) return NULL;
	    if (q == NILCell) errmode = FALSE;
	    else errmode = TRUE;
	    p = p->cdr;
            if (p->attr != 'l') ret = NILCell;
            else {
                ret = eval(p->car,loc,locn,lock);
                if (ret == NULL) return NULL;
	    }
	}
        lineStr[0] = '\0';
        if ((v = GetOneSStream(fp)) && v != 2) {
	    if (CheckBrace(lineStr) != 1) {
		printf("G[: ʂ̐܂B\n");
		errorlisp(loc,locn);
		return NULL;
	    } // Add
            SkipTerm(lineStr);
            if (!MakeFunctionText(lineStr,BUFLEN - 1)) return NULL;
            if (!MakeQuoteText(lineStr,BUFLEN - 1)) return NULL;
            if (!MakeBackQuoteText(lineStr,BUFLEN - 1)) return NULL;
            if (!MakeCommaText(lineStr,BUFLEN - 1)) return NULL;
            if (!MakeAstText(lineStr,BUFLEN - 1)) return NULL;
            top = readFromString(lineStr);
            if (top == NULL) return NULL;
	    return top;
        }
	if (errmode) {
	    printf("G[: ǂݍݒɃG[𐶂܂B\n");
	    errorlisp(loc,locn);
	    return NULL;
	}
	if (v != 2) {
	    errorlisp(loc,locn);
	    return NULL;
	}
        return ret;
    }
}

CELL *cfunction::SetFilePointer(FILE *fp)
{
    int i;
    AINT *k;
    CELL *p;

    for (i = 0;i < FILE_POINTER_LEN;i++)
        if (FilePointer[i] == NULL) {
	    if ((p = getCell()) == NULL) return NILCell;
            p->attr = 'i';
	    p->attr2 = 's'; // It is stream.
	    k = getInt((long)(i + 1));
	    p->car = (CELL *)k;
            FilePointer[i] = fp;
	    return p;
        }
    return NILCell;
}

BOOL cfunction::CheckFunc(CELL *p) // p:(test (x) (print x))
{
    CELL *q;

    if (p->attr != 'l') return FALSE;
    if ((p->car)->attr != 's') return FALSE;
    if ((p->car)->attr2 == 'c') return FALSE;
    p = p->cdr;         // p: ((x) (print x))
    if (p->attr != 'l') return FALSE;
    if (!(p->car == NILCell || (p->car)->attr == 'l')) return FALSE;
    p = p->car;         // p:(x)
    while (p->attr == 'l') {
        q = p->car;
        if (q->attr != 's') return FALSE;
        if (q->attr2 == 'c') return FALSE;
        p = p->cdr;
    }
    return TRUE;
}

CELL *cfunction::minus(CELL *s,SYMVAL *loc,long locn,long lock)
{
	CELL *k;
	CELL *p;

	if (s->attr != 'l') {
		printf("G[: ܂\n");
		errorlisp(loc,locn);
		return NULL;
	}

	IncCellStack(NULL);
	IncCellStack(s);

	if ((p = eval(s->car,loc,locn,lock)) == NULL) { PopCellStack(); return NULL; }

	PopCellStack();

	if (p->attr != 'n' && p->attr != 'i' && p->attr != 'b') {
		printf("G[: lł͂܂\n");
		errorlisp(loc,locn);
		return NULL;
	}
	k = p;
	s = s->cdr;
	for (;;) {
		if (s->attr != 'l') break;

		IncCellStack(NULL);	// Added in Apr.2016
		IncCellStack(k);	// Added in Apr.2016
		IncCellStack(s);

		if ((p = eval(s->car,loc,locn,lock)) == NULL) { PopCellStack(); return NULL; }

		PopCellStack();		// Added in Apr.2016

		if ((k = Sa(k,p)) == NULL) {
		    errorlisp(loc,locn);
		    return NULL;
		}
		s = s->cdr;
	}
	return k;
}

CELL *cfunction::times(CELL *s,SYMVAL *loc,long locn,long lock)
{
	CELL *k;
	AINT *a;
        AFLOAT *f;
	CELL *p;

	k = getCell();
	k->attr = 'i';
	a = getInt(1L);
	k->car = (CELL *)a;
	for (;;) {
		if (s->attr != 'l') break;

		IncCellStack(NULL);
		IncCellStack(k);
		IncCellStack(s);

		if ((p = eval(s->car,loc,locn,lock)) == NULL) { PopCellStack(); return NULL; }

		PopCellStack();

		if ((k = Seki(k,p)) == NULL) {
		    errorlisp(loc,locn);
		    return NULL;
		}
		s = s->cdr;
	}
	return k;
}

CELL *cfunction::divide(CELL *s,SYMVAL *loc,long locn,long lock)
{
	CELL *k;
	AFLOAT *f;
	CELL *p;

	if (s->attr != 'l') {
		printf("G[: ܂\n");
		errorlisp(loc,locn);
		return NULL;
	}

	IncCellStack(NULL);
	IncCellStack(s);

	if ((p = eval(s->car,loc,locn,lock)) == NULL) { PopCellStack(); return NULL; }

	PopCellStack();

	if (p->attr != 'n' && p->attr != 'i' && p->attr != 'b') {
		printf("G[: lł͂܂\n");
		errorlisp(loc,locn);
		return NULL;
	}
	k = p;
	s = s->cdr;
	for (;;) {
		if (s->attr != 'l') break;

		IncCellStack(NULL);
		IncCellStack(s);
		IncCellStack(k);

		if ((p = eval(s->car,loc,locn,lock)) == NULL) { PopCellStack(); return NULL; }

		PopCellStack();

		k = Sho(k,p);
		if (k == NULL) {
		    errorlisp(loc,locn);
		    return NULL;
		}
		s = s->cdr;
	}
	return k;
}

CELL *cfunction::minusp(CELL *s)
{
	if (s == NULL) return NULL;
	switch (s->attr) {
	    case 'i':
		if (((AINT *)(s->car))->f < 0L) return TCell;
		else return NILCell;
	    case 'b':
		if (((AINT *)(s->car))->f < 0L) return TCell;
		else return NILCell;
	    case 'n':
		if (((AFLOAT *)(s->car))->f < 0.0) return TCell;
		else return NILCell;
	    default:
		printf("G[: lł͂܂\n");
		//errorlisp(loc,locn);
		return NULL;
	}
}

CELL *cfunction::jlength(CELL *p)
{
    CELL *q;
    AINT *k;
    long i,len,cnt;
    char *ch;

    if (p->attr == 'l' || p == NILCell) {
  jlengthlist:
	q = getCell();
	if (q == NULL) return NULL;
	k = getInt(tlength(p));
	if (k == NULL) return NULL;
	q->attr = 'i';
	q->car = (CELL *)k;
	return q;
    }
    if (p->attr == 'a') {
	if ((p->car)->attr != 'i') { puts("G[: z񂪉Ă܂"); return NULL; }
	if (((AINT *)((p->car)->car))->f != 1) { puts("G[: ꎟzł͂܂"); return NULL; }
	if ((p->cdr)->attr != 'l') { puts("G[: z񂪉Ă܂"); return NULL; }
	p = p->cdr->cdr;
	goto jlengthlist;
    }
    if (p->attr == 'r') {
	len = strlen((char *)(p->car)) - 2;
	cnt = 0;
	ch = (char *)(p->car) + 1;
	for (i = 0;i < len;i++) {
	    if (*ch == '\\' && *(ch + 1) == '\"') {
		ch += 2;
		i++;
		cnt++;
	    }
	    else if (*ch == '\\' && *(ch + 1) == '\\') {
		ch += 2;
		i++;
		cnt++;
	    }
	    else if (isKanji((unsigned char *)ch)) {
		ch += 2;
		i++;
		cnt++;
	    }
	    else { ch++; cnt++; }
	}
	q = getCell();
	if (q == NULL) return NULL;
	k = getInt(cnt);
	if (k == NULL) return NULL;
	q->attr = 'i';
	q->car = (CELL *)k;
	return q;
    }
    printf("G[: Ώۂ̓XgłȂĂ͂Ȃ܂B\n");
    return NULL;
}

long cmemory::getRealMojiretsuLen(CELL *p)
{
    long k = 0;
    CELL *q;
    char ch;

    while (p->attr == 'l') {
	q = p->car;
	if (q->attr != 'm') return -1L;
	ch = (char)(long)(q->car);
	if (isKanji((unsigned char *)&ch)) k += 2;
	else if (ch == '\\') k += 2;
	else if (ch == '"') k += 2;
	else k++;
	p = p->cdr;
    }
    return k;
}

CELL *cmemory::ReadLineStrm(FILE *fp)
{
    char *p;
    CELL *q,*r,*r2;
    BOOL flag = TRUE;

    if (fgets(lineStr + 1,BUFLEN - 1,fp) == NULL) {
	strcpy(lineStr,"\"\"");
        q = getMojiretsuCell(lineStr);
        r = getCell();
        r2 = getCell();
        r->attr = 'l';
        r->car = q;
        r->cdr = r2;
        r2->attr = 'l';
        r2->car = TCell;
        r2->cdr = NILCell;
        q->mtop = r;
        return q;
    }
    p = lineStr;
    *p++ = '"';
    while (*p != '\0') {
	if (*p == '\n') { *p = '\0'; flag = FALSE; break; }
	else if (*p == '\r') { *p = '\0'; flag = FALSE; break; }
	else if (*p == '\\') { p++; slide(p,1); *p++ = '\\'; }
	else if (*p == '"') { *p++ = '\\'; slide(p,1); *p++ = '"'; }
	else if (isKanji((unsigned char *)p)) { p += 2; }
	else {
	    p++;
	}
    }
    strcat(lineStr,"\"");
    q = getMojiretsuCell(lineStr);
    r = getCell();
    r2 = getCell();
    r->attr = 'l';
    r->car = q;
    r->cdr = r2;
    r2->attr = 'l';
    if (flag) r2->car = TCell;
    else r2->car = NILCell;
    r2->cdr = NILCell;
    q->mtop = r;
    return q;
}

CELL *cfunction::MakeQuoteEach(CELL *p)
{
    CELL *q,*r,*s;

    if (p->attr != 'l') return NILCell;
    q = getCell();
    q->attr = 'l';
    q->car = QUOTE;
    r = getCell();
    r->attr = 'l';
    q->cdr = r;
    r->car = p->car;
    r->cdr = NILCell;
    s = getCell();
    s->attr = 'l';
    s->car = q;
    s->cdr = MakeQuoteEach(p->cdr);
    return s;
}

CELL *cfunction::apply(CELL *x,CELL *y,SYMVAL *loc,long locn,long lock)
{
	CELL *tp,*z,*p,*r,*q,*d,*e;
	long len,i,j,len2;
	SYMVAL *temploc,*lc;

	IncCellStack(NULL);
	IncCellStack(y);

	if ((x = eval(x->car,loc,locn,lock)) == NULL) { PopCellStack(); return NULL; }

	PopCellStack();

	if (y->attr != 'l') y = NILCell;
	else {

		IncCellStack(NULL);
		IncCellStack(x);

		if ((y = eval(y->car,loc,locn,lock)) == NULL) { PopCellStack(); return NULL; }

		PopCellStack();

	}
        tp = MakeQuoteEach(y);
	if (x->attr == 's') {
		if ((r = getCell()) == NULL) return NULL;
		r->attr = 'l';
		r->car = x;
		r->cdr = tp;
		return eval(r,loc,locn,lock);
	}
	else if (x->attr == 'l' && (x->car)->attr == 's' && x->car == LAMBDA) {
		if ((x->cdr)->attr != 'l') {
		    printf("G[: @G[\n");
		    errorlisp(loc,locn);
		    return NULL;
		}
		len = tlength((x->cdr)->car);
		len2 = x->cljlen;
		if (len + /*locn +*/ len2 != 0L) {
		    temploc = (SYMVAL *)calloc(len + /*locn +*/ len2,sizeof(SYMVAL));
		    if (temploc == NULL) {
			printf("G[: [Jϐ̈̊mۂɎs܂\n");
			errorlisp(loc,locn);
			return NULL;
		    }
		}
		else temploc = NULL;

		IncCellStack(NULL);		// Add in Apr.2016
		IncCellStack(x);		// Add in Apr.2016

		if (!initTemploc(temploc,len,(x->cdr)->car,tp,loc,locn,lock)) {
			printf("G[: ܂\n");
			free(temploc);
			PopCellStack();
			errorlisp(loc,locn);
			return NULL;
		}

		PopCellStack();			// Add in Apr.2016

		locn = 0L; // Added on 2014.3.1

		for (i = len,j = 0;i < len + locn;i++,j++) {
			temploc[i].attr = loc[j].attr;
			temploc[i].sym = loc[j].sym;
			temploc[i].val = loc[j].val;
		}

		lc = x->clj;
		for (i = len + locn,j = 0;i < len + locn + len2;i++,j++) {
			temploc[i].attr = lc[j].attr;
			temploc[i].sym = lc[j].sym;
			temploc[i].val = lc[j].val;
		}

/* Deleted in Apr.2016
		IncCellStack(NULL);
*/
		if (IncLocStack(temploc,len + locn + len2) == -1) {
		    free(temploc);
		    return NULL;
		}

		IncCellStack(NULL);		// Add in Apr.2016
		IncCellStack(x);		// Add in Apr.2016

		if (lock < 0) lock -= (len + len2);
		else if (lock > 0) lock += (len + len2);

		p = tprogn((x->cdr)->cdr,temploc,len + locn + len2,lock);

		PopCellStack();			// Add in Apr.2016

		PopLocStack();
/* Deleted in Apr.2016
		PopCellStack();
		if (p != NULL) IncCellStack(p);
*/
		SetOrgLoc(x->clj,x->cljlen,temploc,len + locn,len + locn + len2);
		/*SetOrgLoc(loc,locn,temploc,len,len + locn); Deleted on 2014.3.1 */
		free(temploc);
		return p;
	}
	printf("G[: APPLY̌ĂяoԈĂ܂\n");
	errorlisp(loc,locn);
	return NULL;
}

CELL *cfunction::funcall(CELL *x,CELL *y,SYMVAL *loc,long locn,long lock)
{
	CELL *tp,*z,*p,*r,*q,*d,*e;
	long len,i,j,len2;
	SYMVAL *temploc,*lc;

	IncCellStack(NULL);
	IncCellStack(y);

	if ((x = eval(x->car,loc,locn,lock)) == NULL) { PopCellStack(); return NULL; }

	PopCellStack();

	if (x->attr == 's') {
		if ((r = getCell()) == NULL) return NULL;
		r->attr = 'l';
		r->car = x;
		r->cdr = y;
		return eval(r,loc,locn,lock);
	}
	else if (x->attr == 'l' && (x->car)->attr == 's' && x->car == LAMBDA) {
		if ((x->cdr)->attr != 'l') {
		    printf("G[: @G[\n");
		    errorlisp(loc,locn);
		    return NULL;
		}
		len = tlength((x->cdr)->car);
		len2 = x->cljlen;
		if (len + /*locn +*/ len2 != 0L) {
		    temploc = (SYMVAL *)calloc(len + /*locn +*/ len2,sizeof(SYMVAL));
		    if (temploc == NULL) {
			printf("G[: [Jϐ̈̊mۂɎs܂\n");
			errorlisp(loc,locn);
			return NULL;
		    }
		}
		else temploc = NULL;

		IncCellStack(NULL);		// Add in Apr.2016
		IncCellStack(x);		// Add in Apr.2016

		if (!initTemploc(temploc,len,(x->cdr)->car,y,loc,locn,lock)) {
			printf("G[: ܂\n");
			free(temploc);
			PopCellStack();
			errorlisp(loc,locn);
			return NULL;
		}

		PopCellStack();			// Add in Apr.2016

		locn = 0L;  // Added on 2014.3.1

		for (i = len,j = 0;i < len + locn;i++,j++) {
			temploc[i].attr = loc[j].attr;
			temploc[i].sym = loc[j].sym;
			temploc[i].val = loc[j].val;
		}

		lc = x->clj;
		for (i = len + locn,j = 0;i < len + locn + len2;i++,j++) {
			temploc[i].attr = lc[j].attr;
			temploc[i].sym = lc[j].sym;
			temploc[i].val = lc[j].val;
		}

/* Deleted in Apr.2016
		IncCellStack(NULL);
*/
		if (IncLocStack(temploc,len + locn + len2) == -1) {
		    free(temploc);
		    return NULL;
		}

		IncCellStack(NULL);		// Add in Apr.2016
		IncCellStack(x);		// Add in Apr.2016

		if (lock < 0) lock -= (len + len2);
		else if (lock > 0) lock += (len + len2);

		p = tprogn((x->cdr)->cdr,temploc,len + locn + len2,lock);

		PopCellStack();			// Add in Apr.2016

		PopLocStack();
/* Deleted in Apr.2016
		PopCellStack();
		if (p != NULL) IncCellStack(p);
*/
		SetOrgLoc(x->clj,x->cljlen,temploc,len + locn,len + locn + len2);
		/*SetOrgLoc(loc,locn,temploc,len,len + locn); Deleted on 2014.3.1 */
		free(temploc);
		return p;
	}
	printf("G[: FUNCALĽĂяoԈĂ܂\n");
	errorlisp(loc,locn);
	return NULL;
}

CELL *cfunction::rplaca(CELL *x,CELL *y,SYMVAL *loc,long locn)
{
    if (x->attr != 'l') {
	printf("G[: RXł͂܂\n");
	errorlisp(loc,locn);
	return NULL;
    }
    x->car = y;
    return x;
}

CELL *cfunction::rplacd(CELL *x,CELL *y,SYMVAL *loc,long locn)
{
    if (x->attr != 'l') {
	printf("G[: RXł͂܂\n");
	errorlisp(loc,locn);
	return NULL;
    }
    x->cdr = y;
    return x;
}

CELL *cfunction::function(CELL *p,SYMVAL *loc,long locn,long lock)
{
    CELL *q1,*q2;

    if (p->attr == 's') return p;
    if (p->attr == 'l' && p->car == LAMBDA && (p->cdr)->attr == 'l' &&
        ((p->cdr)->car == NILCell || ((p->cdr)->car)->attr == 'l') ) {
        p->clj = loc;
	p->cljlen = locn;
	return p;
    }
    printf("G[: FUNCTION̎gԈĂ܂\n");
    locn = lock; // Dummy
    errorlisp(loc,locn);
    return NULL;
}

CELL *cfunction::evalu(CELL *top,SYMVAL *loc,long locn,BOOL *multi,long lock)
{
	CELL *p,*x,*y,*z;
	long n2;
	SYMVAL *r,*temploc;
	long len,i;
	long mi,ni;
	char ch,ch2;
	char *xc,*yc;
	FILE *fp;
	if (FreeCellSize < CELL_SIZE_MIN || FreeNumberSize < NUMBER_SIZE_MIN ||
	    FreeIntSize < INT_SIZE_MIN ) {
            gc();
	}
//printS(top);
//printf("\n");
	*multi = FALSE;
	if (top->attr == 'l') {
		if ((top->car)->attr != 's') {
			if ((top->car)->attr == 'l' && (top->car)->car == LAMBDA) {
				*multi = TRUE;
				x = getCell();
				y = getCell();
				z = getCell();
				p = getCell();
				z->attr = 'l';
				y->attr = 'l';
				x->attr = 'l';
				p->attr = 'l';
				p->car = FUNCALL;
				p->cdr = x;
				x->cdr = top->cdr;
				x->car = y;
				y->car = FUNCTION;
				y->cdr = z;
				z->car = top->car;
				z->cdr = NILCell;
				return eval(p,loc,locn,lock);
			}
			goto serror;
		}
		else {
                        if (top->car == QUOTE) {
				if ((top->cdr)->attr != 'l') goto serror;
				return (top->cdr)->car;
			}
			else if (top->car == CAR) {
				if ((top->cdr)->attr != 'l') goto serror;
				if ((top = eval((top->cdr)->car,loc,locn,lock)) == NULL) return NULL;
				if (top->attr != 'l') goto serror;
				return top->car;
			}
			else if (top->car == CDR) {
				if ((top->cdr)->attr != 'l') goto serror;
				if ((top = eval((top->cdr)->car,loc,locn,lock)) == NULL) return NULL;
				if (top->attr != 'l') goto serror;
				return top->cdr;
			}
			else if (top->car == CONS) {
				if ((top->cdr)->attr != 'l') goto serror;
				if (((top->cdr)->cdr)->attr != 'l') goto serror;
				if ((p = getCell()) == NULL) return NULL;
				p->attr = 'l';

				IncCellStack(NULL);
				IncCellStack(top);
				IncCellStack(p);

				if ((p->car = eval((top->cdr)->car,loc,locn,lock)) == NULL) { PopCellStack(); return NULL; }
				if ((p->cdr = eval(((top->cdr)->cdr)->car,loc,locn,lock)) == NULL) { PopCellStack(); return NULL; }

				PopCellStack();

				return p;
			}
			else if (top->car == BACKQUOTE) {
//                                if ((top->cdr)->attr != 'l') goto serror;
                                return backquote(top->cdr,loc,locn,lock);
			}
			else if (top->car == PRINT) {
				if ((top->cdr)->attr != 'l') goto serror;

				IncCellStack(NULL);
				IncCellStack(top);

				if ((p = eval((top->cdr)->car,loc,locn,lock)) == NULL) { PopCellStack(); return NULL; }

				PopCellStack();

				if (((top->cdr)->cdr)->attr == 'l') {

				    IncCellStack(NULL);
				    IncCellStack(p);

				    if ((y = eval(((top->cdr)->cdr)->car,loc,locn,lock)) == NULL) { PopCellStack(); return NULL; }

				    PopCellStack();

				    if (y->attr2 != 's') goto serror;
				    if ((fp = getStream(y)) == NULL) goto serror;
				    fprintf(fp,"\r\n");
				    printSStrm(p,fp);
				    fprintf(fp," ");
				}
				else {
                    		    printf("\n");
                                    printS(p);
				    printf(" ");
				}
				return p;
			}
			else if (top->car == PRIN1) {
				if ((top->cdr)->attr != 'l') goto serror;

				IncCellStack(NULL);
				IncCellStack(top);

				if ((p = eval((top->cdr)->car,loc,locn,lock)) == NULL) { PopCellStack(); return NULL; }

				PopCellStack();

				if (((top->cdr)->cdr)->attr == 'l') {

				    IncCellStack(NULL);
				    IncCellStack(p);

				    if ((y = eval(((top->cdr)->cdr)->car,loc,locn,lock)) == NULL) { PopCellStack(); return NULL; }

				    PopCellStack();

				    if (y->attr2 != 's') goto serror;
				    if ((fp = getStream(y)) == NULL) goto serror;
				    printSStrm(p,fp);
				}
				else {
                                    printS(p);
				}
				return p;
			}
			else if (top->car == PRINC) {
				if ((top->cdr)->attr != 'l') goto serror;

				IncCellStack(NULL);
				IncCellStack(top);

				if ((p = eval((top->cdr)->car,loc,locn,lock)) == NULL) { PopCellStack(); return NULL; }

				PopCellStack();

				if (((top->cdr)->cdr)->attr == 'l') {

				    IncCellStack(NULL);
				    IncCellStack(p);

				    if ((y = eval(((top->cdr)->cdr)->car,loc,locn,lock)) == NULL) { PopCellStack(); return NULL; }

				    PopCellStack();

				    if (y->attr2 != 's') goto serror;
				    if ((fp = getStream(y)) == NULL) goto serror;
				    princS(p,fp);
				}
				else {
                                    princS(p,stdout);
				}
				return p;
			}
			else if (top->car == SET) {
				if ((top->cdr)->attr != 'l') goto serror;
				if (((top->cdr)->cdr)->attr != 'l') goto serror;

				IncCellStack(NULL);
				IncCellStack(top);

				if ((x = eval((top->cdr)->car,loc,locn,lock)) == NULL) { PopCellStack(); return NULL; }

				PopCellStack();

				IncCellStack(NULL);
				IncCellStack(x);

				if ((y = eval(((top->cdr)->cdr)->car,loc,locn,lock)) == NULL) { PopCellStack(); return NULL; }

				PopCellStack();

				if (x->attr != 's') goto serror;
				if (x->attr2 == 'c') goto serror;
				if ((n2 = getSymVal2(x)) == -1) return NULL;
				sv[n2].sym = x;
				sv[n2].val = y;
				return y;
			}
			else if (top->car == DEFPARAMETER) {
				if ((top->cdr)->attr != 'l') goto serror;
				if (((top->cdr)->cdr)->attr != 'l') goto serror;
				x = (top->cdr)->car;

				IncCellStack(NULL);
				IncCellStack(x);

				if ((y = eval(((top->cdr)->cdr)->car,loc,locn,lock)) == NULL) { PopCellStack(); return NULL; }

				PopCellStack();

				if (x->attr != 's') goto serror;
				if (x->attr2 == 'c') goto serror;
				if ((n2 = getSymVal2(x)) == -1) return NULL;
				y->mtop = NULL;
				sv[n2].sym = x;
				sv[n2].val = y;
				return x;
			}
			else if (top->car == DEFVAR) {
				if ((top->cdr)->attr != 'l') goto serror;
				if (((top->cdr)->cdr)->attr != 'l') goto serror;
				x = (top->cdr)->car;

				IncCellStack(NULL);
				IncCellStack(x);

				if ((y = eval(((top->cdr)->cdr)->car,loc,locn,lock)) == NULL) { PopCellStack(); return NULL; }

				PopCellStack();

				if (x->attr != 's') goto serror;
				if (x->attr2 == 'c') goto serror;
				if (getSymVal(x) != -1) return x;
				if ((n2 = getSymVal2(x)) == -1) return NULL;
				y->mtop = NULL;
				sv[n2].sym = x;
				sv[n2].val = y;
				return x;
			}
			else if (top->car == SETQ) {
				if ((top->cdr)->attr != 'l') goto serror;
				if (((top->cdr)->cdr)->attr != 'l') goto serror;
	setq:
				x = (top->cdr)->car;
				if (x->attr2 == 'c') goto serror;

				IncCellStack(NULL);	// Added in Apr.2016
				IncCellStack(x);	// Added in Apr.2016

				if ((y = eval(((top->cdr)->cdr)->car,loc,locn,lock)) == NULL) return NULL;

				PopCellStack();		// Added in Apr.2016

				if (x->attr != 's') goto serror;

				if (lock < 0) lock = 0;
//printf("lock=%ld ",lock);
				if ((n2 = getLocal(loc,locn,x,lock)) != -1) {
					loc[n2].val = y;
//printS(y);
					return y;
				}
				else {
					if ((n2 = getSymVal2(x)) == -1) return NULL;
					sv[n2].sym = x;
					sv[n2].val = y;
					return y;
				}
			}
			else if (top->car == AND) {
				return tand(top->cdr,loc,locn,lock);
			}
			else if (top->car == OR) {
				return tor(top->cdr,loc,locn,lock);
			}
			else if (top->car == ONEPLUS) {
				if ((top->cdr)->attr != 'l') goto serror;
				return OnePlus((top->cdr)->car,loc,locn,lock);
			}
			else if (top->car == ONEMINUS) {
				if ((top->cdr)->attr != 'l') goto serror;
				return OneMinus((top->cdr)->car,loc,locn,lock);
			}
			else if (top->car == ATOM) {
				if ((top->cdr)->attr != 'l') goto serror;
		                if ((p = eval((top->cdr)->car,loc,locn,lock)) == NULL) return NULL;
				if (p->attr != 'l') return TCell;
				else return NILCell;
			}
			else if (top->car == CONSP) {
				if ((top->cdr)->attr != 'l') goto serror;
		                if ((p = eval((top->cdr)->car,loc,locn,lock)) == NULL) return NULL;
				if (p->attr == 'l') return TCell;
				else return NILCell;
			}
			else if (top->car == NUMBERP) {
				if ((top->cdr)->attr != 'l') goto serror;
                		if ((p = eval((top->cdr)->car,loc,locn,lock)) == NULL) return NULL;
				if (p->attr == 'i' || p->attr == 'n' || p->attr == 'b' ) return TCell;
				else return NILCell;
			}
			else if (top->car == INTEGERP) {
				if ((top->cdr)->attr != 'l') goto serror;
                		if ((p = eval((top->cdr)->car,loc,locn,lock)) == NULL) return NULL;
				if (p->attr == 'i') return TCell;
				else return NILCell;
			}
			else if (top->car == EVAL) {
				if ((top->cdr)->attr != 'l') goto serror;

				IncCellStack(NULL);
				IncCellStack(top);

                		if ((p = eval((top->cdr)->car,loc,locn,lock)) == NULL) { PopCellStack(); return NULL; }

				PopCellStack();

				if (lock < 0) lock *= -1;
//printf("p="); printS(p);
				p = eval(p,loc,locn,lock);
//printf("p2="); printS(p);
				return p;
			}
			else if (top->car == TIMES) {
				p = times(top->cdr,loc,locn,lock);
				return p;
			}
			else if (top->car == PLUS) {
				p = plus(top->cdr,loc,locn,lock);
				return p;
			}
			else if (top->car == MINUS) {
				p = minus(top->cdr,loc,locn,lock);
				return p;
			}
			else if (top->car == DIVIDE) {
				p = divide(top->cdr,loc,locn,lock);
				return p;
			}
			else if (top->car == NUMBEREQUAL) {
				if ((top->cdr)->attr != 'l') goto serror;
				if (((top->cdr)->cdr)->attr != 'l') goto serror;

				IncCellStack(NULL);
				IncCellStack(top);

				if ((x = eval((top->cdr)->car,loc,locn,lock)) == NULL) { PopCellStack(); return NULL; }

				PopCellStack();

				IncCellStack(NULL);
				IncCellStack(x);

				if ((y = eval(((top->cdr)->cdr)->car,loc,locn,lock)) == NULL) { PopCellStack(); return NULL; }

				PopCellStack();

				if (x->attr == 'n' && y->attr == 'n') {
				    if (((AFLOAT *)(x->car))->f == ((AFLOAT *)(y->car))->f) return TCell;
				    return NILCell;
                                }
				if (x->attr == 'i' && y->attr == 'i') {
				    if (((AINT *)(x->car))->f == ((AINT *)(y->car))->f) return TCell;
				    return NILCell;
                                }
				if (x->attr == 'i' && y->attr == 'n') {
				    if (((LONGDOUBLE)(((AINT *)(x->car))->f)) ==
                                        ((AFLOAT *)(y->car))->f ) return TCell;
				    return NILCell;
                                }
				if (x->attr == 'n' && y->attr == 'i') {
				    if (((AFLOAT *)(x->car))->f ==
                                        ((LONGDOUBLE)(((AINT *)(y->car))->f)) ) return TCell;
				    return NILCell;
                                }
				if (x->attr == 'b' && y->attr == 'b') {
				    if (((AINT *)(x->car))->f !=
					((AINT *)(y->car))->f ) return NILCell;
				    if (((AINT *)(x->cdr))->f !=
					((AINT *)(y->cdr))->f ) return NILCell;
				    return TCell;
				}
				if (x->attr == 'b' && y->attr == 'n') {
				    mi = ((AINT *)(x->car))->f;
				    ni = ((AINT *)(x->cdr))->f;
				    if ((LONGDOUBLE)mi / (LONGDOUBLE)ni == ((AFLOAT *)(y->car))->f) return TCell;
				    return NILCell;
                                }
				if (x->attr == 'n' && y->attr == 'b') {
				    mi = ((AINT *)(y->car))->f;
				    ni = ((AINT *)(y->cdr))->f;
				    if ((LONGDOUBLE)mi / (LONGDOUBLE)ni == ((AFLOAT *)(x->car))->f) return TCell;
				    return NILCell;
                                }
				if (x->attr == 'b' && y->attr == 'i') return NILCell;  // 2015/3/26
				if (x->attr == 'i' && y->attr == 'b') return NILCell;  // 2015/3/26
                                goto serror;
			}
			else if (top->car == EQ) {
				if ((top->cdr)->attr != 'l') goto serror;
				if (((top->cdr)->cdr)->attr != 'l') goto serror;

				IncCellStack(NULL);
				IncCellStack(top);

				if ((x = eval((top->cdr)->car,loc,locn,lock)) == NULL) { PopCellStack(); return NULL; }

				PopCellStack();

				IncCellStack(NULL);
				IncCellStack(x);

				if ((y = eval(((top->cdr)->cdr)->car,loc,locn,lock)) == NULL) { PopCellStack(); return NULL; }

				PopCellStack();

				if (x->attr == 's') {
					if (y->attr != 's') return NILCell;
					if (x == y) return TCell;
					return NILCell;
				}
				else if (x->attr == 'i') {
					if (y->attr != 'i') return NILCell;
					if (((AINT *)(x->car))->f == ((AINT *)(y->car))->f) return TCell;
					return NILCell;
				}
				else if (x->attr == 'l' || x->attr == 'a') {
					if (x == y) return TCell;
					return NILCell;
				}
				return NILCell;
			}
			else if (top->car == EQL) {
			    if ((top->cdr)->attr != 'l') goto serror;
			    x = (top->cdr)->car;
			    if (((top->cdr)->cdr)->attr != 'l') goto serror;
			    y = ((top->cdr)->cdr)->car;

			    IncCellStack(NULL);
			    IncCellStack(y);

			    x = eval(x,loc,locn,lock);

			    PopCellStack();

			    if (x == NULL) return NULL;

			    IncCellStack(NULL);
			    IncCellStack(x);

			    y = eval(y,loc,locn,lock);

			    PopCellStack();

			    if (y == NULL) return NULL;
			    if (x == y) return TCell;
			    if (x->attr == 'i' && y->attr == 'i') {
				if (((AINT *)(x->car))->f == ((AINT *)(y->car))->f) return TCell;
				else return NILCell;
			    }
			    if (x->attr == 'n' && y->attr == 'n') {
				if (((AFLOAT *)(x->car))->f == ((AFLOAT *)(y->car))->f) return TCell;
				else return NILCell;
			    }
			    if (x->attr == 'b' && y->attr == 'b') {
				if (((AINT *)(x->car))->f == ((AINT *)(y->car))->f
				    && ((AINT *)(x->cdr))->f == ((AINT *)(y->cdr))->f )
				    return TCell;
				else return NILCell;
			    }
			    if (x->attr == 'm' && y->attr == 'm') {
				if (x->cdr == NULL && y->cdr == NULL) {
				    if ((unsigned char)(long)(x->car) == (unsigned char)(long)(y->car)) return TCell;
				    else return NILCell;
				}
				if (x->cdr == NULL || y->cdr == NULL) return NILCell;
				if ((unsigned char)(long)(x->car) == (unsigned char)(long)(y->car) &&
				    (unsigned char)(long)(x->cdr) == (unsigned char)(long)(y->cdr) ) return TCell;
				return NILCell;
			    }
			    return NILCell;
			}
			else if (top->car == NULLP) {
				if ((top->cdr)->attr != 'l') goto serror;
				if ((p = eval((top->cdr)->car,loc,locn,lock)) == NULL) return NULL;
				if (p == NILCell) return TCell;
				return NILCell;
			}
			else if (top->car == PROGN) {
				*multi = TRUE;
				p = tprogn(top->cdr,loc,locn,lock);
				return p;
			}
			else if (top->car == PROG1) {
				*multi = TRUE;
				p = tprog1(top->cdr,loc,locn,lock);
				return p;
			}
			else if (top->car == MINUSP) {
				if ((top->cdr)->attr != 'l') goto serror;
				return minusp(eval((top->cdr)->car,loc,locn,lock));
			}
			else if (top->car == COND) {
				*multi = TRUE;
				p = condition(top->cdr,loc,locn,lock);
				return p;
			}
			else if (top->car == IF) {
				*multi = TRUE;
				p = IfClause(top->cdr,loc,locn,lock);
				return p;
			}
			else if (top->car == DEFUN) {
#if 0
				if (!CheckFunc(top->cdr)) goto serror;
				if ((top->cdr)->attr != 'l') goto serror;
				if (((top->cdr)->cdr)->attr != 'l') goto serror;
				if ((r = getFuncval((top->cdr)->car)) == NULL) return NULL;
				if ( ( r->sym = getStringCell( (char *)(((top->cdr)->car)->car) ) ) == NULL)
                                    return NULL;
				r->val = (top->cdr)->cdr;
				r->attr = 'f';
				return r->sym;
#else
				CELL *body,*args,*cbody,*newbody,*blk,*name,*cmd,*tree;
				if (!CheckFunc(top->cdr)) goto serror;
				if ((top->cdr)->attr != 'l') goto serror;
				body = (top->cdr)->cdr;
				cmd = (top->cdr)->car;
				if (((top->cdr)->cdr)->attr != 'l') goto serror;
				args = ((top->cdr)->cdr)->car;
				if ((r = getFuncval(cmd)) == NULL) return NULL;
				if ( ( r->sym = getStringCell( (char *)(cmd->car) ) ) == NULL)
                                    return NULL;
				args = body->car;
				cbody = body->cdr;
				newbody = getCell(); newbody->attr = 'l';
				newbody->car = args;
				blk = getCell(); blk->attr = 'l';
				tree = getCell(); tree->attr = 'l';
				tree->car = blk;
				tree->cdr = NILCell;
				newbody->cdr = tree;
				blk->car = BLOCKLISP;
				name = getCell(); name->attr = 'l';
				blk->cdr = name;
				name->car = cmd;
				name->cdr = cbody;
				r->val = newbody;
				r->attr = 'f';
				return r->sym;
#endif
			}
			else if (top->car == DEFMACRO) {
#if 0
				if (!CheckFunc(top->cdr)) goto serror;
				if ((top->cdr)->attr != 'l') goto serror;
				if (((top->cdr)->cdr)->attr != 'l') goto serror;
				if ((r = getFuncval((top->cdr)->car)) == NULL) return NULL;
				if ( ( r->sym = getStringCell( (char *)(((top->cdr)->car)->car) ) ) == NULL)
                                    return NULL;
				r->val = (top->cdr)->cdr;
				r->attr = 'm';
				return r->sym;
#else
				CELL *body,*args,*cbody,*newbody,*blk,*name,*cmd,*tree;
				if (!CheckFunc(top->cdr)) goto serror;
				if ((top->cdr)->attr != 'l') goto serror;
				body = (top->cdr)->cdr;
				cmd = (top->cdr)->car;
				if (((top->cdr)->cdr)->attr != 'l') goto serror;
				args = ((top->cdr)->cdr)->car;
				if ((r = getFuncval(cmd)) == NULL) return NULL;
				if ( ( r->sym = getStringCell( (char *)(cmd->car) ) ) == NULL)
                                    return NULL;
				args = body->car;
				cbody = body->cdr;
				newbody = getCell(); newbody->attr = 'l';
				newbody->car = args;
				blk = getCell(); blk->attr = 'l';
				tree = getCell(); tree->attr = 'l';
				tree->car = blk;
				tree->cdr = NILCell;
				newbody->cdr = tree;
				blk->car = BLOCKLISP;
				name = getCell(); name->attr = 'l';
				blk->cdr = name;
				name->car = cmd;
				name->cdr = cbody;
				r->val = newbody;
				r->attr = 'm';
				return r->sym;
#endif
			}
			else if (top->car == LET) { *multi = TRUE; p = Let(top->cdr,loc,locn,lock); return p; }
			else if (top->car == LETAST) { *multi = TRUE; p = LetAst(top->cdr,loc,locn,lock); return p; }
                        else if (top->car == PROG) {
				*multi = TRUE; p = prog(top->cdr,loc,locn,lock); return p;
			}
			else if (top->car == GO) {
                            if ((top->cdr)->attr != 'l') goto serror;
			    if (((top->cdr)->car)->attr != 's') goto serror;
                            GoSymbol = (top->cdr)->car;
                            return TCell;
                        }
/*			else if (top->car == RETURN) {
                            if ((top->cdr)->attr != 'l') {
			        ReturnValue = NILCell;
                                ReturnFlag = 'a';
                                return NILCell;
                            }
                            x = eval((top->cdr)->car,loc,locn,lock);
                            if (x == NULL) return NULL;
			    ReturnValue = x;
                            ReturnFlag = 'a';
			    *multi = TRUE;
                            return x;
                        }*/
			else if (top->car == APPLY) {
				*multi = TRUE;
				if ((top->cdr)->attr != 'l') goto serror;
				return apply(top->cdr,(top->cdr)->cdr,loc,locn,lock);
			}
			else if (top->car == FUNCALL) {
				*multi = TRUE;
				if ((top->cdr)->attr != 'l') goto serror;
				return funcall(top->cdr,(top->cdr)->cdr,loc,locn,lock);
			}
			else if (top->car == LIST) {
				p = tlist(top->cdr,loc,locn,lock);
				return p;
			}
			else if (top->car == TOPEN) {
				p = topen(top->cdr,loc,locn,lock);
				return p;
			}
			else if (top->car == TCLOSE) {
				if ((top->cdr)->attr != 'l') goto serror;
				return tclose((top->cdr)->car,loc,locn,lock);
			}
			else if (top->car == TREAD) {
				return tread(top->cdr,loc,locn,lock);
			}
			else if (top->car == CHAR) {
				p = tchar(top->cdr,loc,locn,lock);
				return p;
			}
			else if (top->car == STRINGP) {
				if ((top->cdr)->attr != 'l') goto serror;
				x = eval((top->cdr)->car,loc,locn,lock);
				if (x == NULL) return NULL;
				if (x->attr == 'a' && (x->car)->attr == 'i' && ((AINT *)((x->car)->car))->f == 1L &&
				    (x->cdr)->car == CHARACTER )
				    return TCell;
				return NILCell;
			}
			else if (top->car == LENGTH) {
				if ((top->cdr)->attr != 'l') goto serror;
				x = eval((top->cdr)->car,loc,locn,lock);
				if (x == NULL) return NULL;
				return jlength(x);
			}
			else if (top->car == COERCE) {
				return coerce(top->cdr,loc,locn,lock);
			}
			else if (top->car == FUNCTION) {
				if ((top->cdr)->attr != 'l') goto serror;
				return function((top->cdr)->car,loc,locn,lock);
			}
			else if (top->car == SPACES) {
				if ((top->cdr)->attr != 'l') goto serror;

				IncCellStack(NULL);
				IncCellStack(top);

				x = eval((top->cdr)->car,loc,locn,lock);

				PopCellStack();

				if (x == NULL) return NULL;
				if (x->attr != 'i') goto serror;
				len = ((AINT *)(x->car))->f;
				if (len < 0) goto serror;
				if (((top->cdr)->cdr)->attr != 'l') {
				    for (i = 0;i < len;i++) printf(" ");
                                }
				else {
				    if ((y = eval(((top->cdr)->cdr)->car,loc,locn,lock)) == NULL) return NULL;
				    if (y->attr2 != 's') goto serror;
				    if ((fp = getStream(y)) == NULL) goto serror;
				    for (i = 0;i < len;i++) fprintf(fp," ");
				}
				return TCell;
			}
			else if (top->car == TERPRI) {
				if ((top->cdr)->attr != 'l') {
				    fp = stdout;
                                }
				else {
				    if ((y = eval((top->cdr)->car,loc,locn,lock)) == NULL) return NULL;
				    if (y->attr2 != 's') goto serror;
				    if ((fp = getStream(y)) == NULL) goto serror;
				}
				if (fp != stdout) fprintf(fp,"\r");
				fprintf(fp,"\n");
				return TCell;
			}
			else if (top->car == STRINGEQUAL) {
				if ((top->cdr)->attr != 'l') goto serror;

				IncCellStack(NULL);
				IncCellStack(top);

				x = eval((top->cdr)->car,loc,locn,lock);

				PopCellStack();

				if (x == NULL) return NULL;
//				if (x->attr != 'r') goto serror;
				if (((top->cdr)->cdr)->attr != 'l') goto serror;

				IncCellStack(NULL);
				IncCellStack(x);

				y = eval(((top->cdr)->cdr)->car,loc,locn,lock);

				PopCellStack();

				if (y == NULL) return NULL;
				return StringEqual(x,y);
			}
			else if (top->car == CHAREQUAL) {
				if ((top->cdr)->attr != 'l') goto serror;

				IncCellStack(NULL);
				IncCellStack(top);

				x = eval((top->cdr)->car,loc,locn,lock);

				PopCellStack();

				if (x == NULL) return NULL;
				if (x->attr != 'm') goto serror;
				if (((top->cdr)->cdr)->attr != 'l') goto serror;

				IncCellStack(NULL);
				IncCellStack(x);

				y = eval(((top->cdr)->cdr)->car,loc,locn,lock);

				PopCellStack();

				if (y == NULL) return NULL;
				if (y->attr != 'm') goto serror;
				if (x->cdr == NULL && y->cdr == NULL) {
				    if ((unsigned char)(long)(x->car) == (unsigned char)(long)(y->car)) return TCell;
				    else return NILCell;
				}
				if (x->cdr == NULL || y->cdr == NULL) return NILCell;
				if ((unsigned char)(long)(x->car) == (unsigned char)(long)(y->car) &&
				    (unsigned char)(long)(x->cdr) == (unsigned char)(long)(y->cdr) ) return TCell;
				return NILCell;
			}
			else if (top->car == CHARLESS) {
				if ((top->cdr)->attr != 'l') goto serror;

				IncCellStack(NULL);
				IncCellStack(top);

				x = eval((top->cdr)->car,loc,locn,lock);

				PopCellStack();

				if (x == NULL) return NULL;
				if (x->attr != 'm') goto serror;
				if (((top->cdr)->cdr)->attr != 'l') goto serror;

				IncCellStack(NULL);
				IncCellStack(x);

				y = eval(((top->cdr)->cdr)->car,loc,locn,lock);

				PopCellStack();

				if (y == NULL) return NULL;
				if (y->attr != 'm') goto serror;
				if (x->cdr == NULL && y->cdr == NULL) {
				    if ((unsigned char)(long)(x->car) < (unsigned char)(long)(y->car)) return TCell;
				    else return NILCell;
				}
				if (x->cdr == NULL) return TCell;
				if (y->cdr == NULL) return NILCell;
				if ((unsigned char)(long)(x->car) < (unsigned char)(long)(y->car)) return TCell;
				if ((unsigned char)(long)(x->car) != (unsigned char)(long)(y->car)) return NILCell;
				if ((unsigned char)(long)(x->cdr) < (unsigned char)(long)(y->cdr)) return TCell;
				return NILCell;
			}
			else if (top->car == GC) {
				gc();
				return TCell;
			}
			else if (top->car == MULTIPLEVALUELIST) {
				if ((top->cdr)->attr != 'l') goto serror;
				if ((x = eval((top->cdr)->car,loc,locn,lock)) == NULL) return NULL;
				if (x->mtop == NULL) {
					p =getCell();
					p->attr = 'l';
					p->car = x;
					p->cdr = NILCell;
					return p;
				}
				else {
					return x->mtop;
				}
			}
			else if (top->car == RPLACA) {
				if ((top->cdr)->attr != 'l') goto serror;
				if (((top->cdr)->cdr)->attr != 'l') goto serror;

				IncCellStack(NULL);
				IncCellStack(top);

				x = eval((top->cdr)->car,loc,locn,lock);

				PopCellStack();

				if (x == NULL) return NULL;

				IncCellStack(NULL);
				IncCellStack(x);

				y = eval(((top->cdr)->cdr)->car,loc,locn,lock);

				PopCellStack();

				if (y == NULL) return NULL;
				return rplaca(x,y,loc,locn);
			}
			else if (top->car == RPLACD) {
				if ((top->cdr)->attr != 'l') goto serror;
				if (((top->cdr)->cdr)->attr != 'l') goto serror;

				IncCellStack(NULL);
				IncCellStack(top);

				x = eval((top->cdr)->car,loc,locn,lock);

				PopCellStack();

				if (x == NULL) return NULL;

				IncCellStack(NULL);
				IncCellStack(x);

				y = eval(((top->cdr)->cdr)->car,loc,locn,lock);

				PopCellStack();

				if (y == NULL) return NULL;
				return rplacd(x,y,loc,locn);
			}
			else if (top->car == SYMBOLP) {
				if ((top->cdr)->attr != 'l') goto serror;
				x = eval((top->cdr)->car,loc,locn,lock);
				if (x == NULL) return NULL;
				if (x->attr == 's') return TCell;
				return NILCell;
			}
			else if (top->car == CHARACTERP) {
				if ((top->cdr)->attr != 'l') goto serror;
				x = eval((top->cdr)->car,loc,locn,lock);
				if (x == NULL) return NULL;
				if (x->attr == 'm') return TCell;
				return NILCell;
			}
			else if (top->car == BYE || top->car == QUIT || top->car == EXIT) {
				exit(0);
			}
			else if (top->car == MACROEXPAND) {
			    if ((top->cdr)->attr != 'l') goto serror;
			    top = eval((top->cdr)->car,loc,locn,lock);
			    if (top == NULL) return NULL;
			    r = getFuncval2(top->car);
			    if (r == NULL || r->attr != 'm') {
				printf("G[: `̃}Nł\n");
				return NULL;
			    }
			    return MacroExec2(top->cdr,r,loc,locn,lock);
			}

			else if (top->car == SETARRAY) {
				return SetArray(top,loc,locn,lock);
			}
		/*	else if (top->car == AREF) {
				return aref(top,loc,locn);
			}*/
		/*	else if (top->car == PREAREF) {
				return prearef(top,loc,locn);
			}*/
			else if (top->car == ARRAYCAR) {
			    if ((top->cdr)->attr != 'l') goto serror;
			    top = eval((top->cdr)->car,loc,locn,lock);
			    if (top == NULL) return NULL;
			    return ArrayCar(top);
			}
			else if (top->car == ARRAYCDR) {
			    if ((top->cdr)->attr != 'l') goto serror;
			    top = eval((top->cdr)->car,loc,locn,lock);
			    if (top == NULL) return NULL;
			    return ArrayCdr(top);
			}
			else if (top->car == ARRAYP) {
				if ((top->cdr)->attr != 'l') goto serror;
				x = eval((top->cdr)->car,loc,locn,lock);
				if (x == NULL) return NULL;
				if (x->attr == 'a') return TCell;
				return NILCell;
			}
			else if (top->car == RANDOM) {
				if ((top->cdr)->attr != 'l') goto serror;
				return Random((top->cdr)->car,loc,locn,lock);
			}
			else if (top->car == SQRT) {
				if ((top->cdr)->attr != 'l') goto serror;
				return jSqrt((top->cdr)->car,loc,locn,lock);
			}
			else if (top->car == EXPT) {
				if ((top->cdr)->attr != 'l') goto serror;
				if (((top->cdr)->cdr)->attr != 'l') goto serror;
				return jExpt((top->cdr)->car,((top->cdr)->cdr)->car,loc,locn,lock);
			}
			else if (top->car == EXP) {
				if ((top->cdr)->attr != 'l') goto serror;
				return jExp((top->cdr)->car,loc,locn,lock);
			}
			else if (top->car == LOG) {
				if ((top->cdr)->attr != 'l') goto serror;
				return jLog((top->cdr)->car,loc,locn,lock);
			}
			else if (top->car == SIN) {
				if ((top->cdr)->attr != 'l') goto serror;
				return jSin((top->cdr)->car,loc,locn,lock);
			}
			else if (top->car == COS) {
				if ((top->cdr)->attr != 'l') goto serror;
				return jCos((top->cdr)->car,loc,locn,lock);
			}
			else if (top->car == TAN) {
				if ((top->cdr)->attr != 'l') goto serror;
				return jTan((top->cdr)->car,loc,locn,lock);
			}
			else if (top->car == ASIN) {
				if ((top->cdr)->attr != 'l') goto serror;
				return jAsin((top->cdr)->car,loc,locn,lock);
			}
			else if (top->car == ACOS) {
				if ((top->cdr)->attr != 'l') goto serror;
				return jAcos((top->cdr)->car,loc,locn,lock);
			}
			else if (top->car == ATAN) {
				if ((top->cdr)->attr != 'l') goto serror;
				return jAtan((top->cdr)->car,loc,locn,lock);
			}
			else if (top->car == EVENP) {
				if ((top->cdr)->attr != 'l') goto serror;
				return jEvenp((top->cdr)->car,loc,locn,lock);
			}
			else if (top->car == FLOOR) {
				if ((top->cdr)->attr != 'l') goto serror;
				p = tfloor((top->cdr)->car,loc,locn,lock);
				if (p == NULL) return NULL;
				*multi = TRUE;
				return p;
			}
			else if (top->car == NOW) {
				return now();
			}
			else if (top->car == LOCALTIME) {
				if ((top->cdr)->attr != 'l') goto serror;
				return LocalTime((top->cdr)->car,loc,locn,lock);
			}
			else if (top->car == MAKETIME) {
				if ((top->cdr)->attr != 'l') goto serror;
				return MakeTime((top->cdr)->car,loc,locn,lock);
			}
			else if (top->car == CHARCODE) {
				if ((top->cdr)->attr != 'l') goto serror;
				return char_to_code(top->cdr,loc,locn,lock);
			}
			else if (top->car == CODECHAR) {
				if ((top->cdr)->attr != 'l') goto serror;
				return code_to_char(top->cdr,loc,locn,lock);
			}
			else if (top->car == CATCHLISP) {
				//if ((top->cdr)->attr != 'l') goto serror;
				return catch_lisp(top->cdr,loc,locn);
			}
			else if (top->car == THROWLISP) {
				//if ((top->cdr)->attr != 'l') goto serror;
				return throw_lisp(top->cdr,loc,locn);
			}
			else if (top->car == BLOCKLISP) {
				//if ((top->cdr)->attr != 'l') goto serror;
				return block_lisp(top->cdr,loc,locn);
			}
			else if (top->car == RETURNFROMLISP) {
				//if ((top->cdr)->attr != 'l') goto serror;
				return return_from_lisp(top->cdr,loc,locn);
			}
			else if (top->car == UNWINDPROTECT) {
				//if ((top->cdr)->attr != 'l') goto serror;
				return unwind_protect(top->cdr,loc,locn);
			}
			else if (top->car == ERRORLISP) {
				if ((top->cdr)->attr != 'l') goto serror;
				p = eval((top->cdr)->car,loc,locn,0);
				if (p == NULL) return NULL;
				princS(p,stdout);
				printf("\n");
				errorlisp(loc,locn);
				return NULL;
			}
			else if (top->car == BREAKLISP) {
				return breaklisp(loc,locn);
			}
			else if (top->car == SYMBOLNAME) {
				return symbolname(top->cdr,loc,locn);
			}
			else if (top->car == TINTERN) {
				p = t_intern(top->cdr,loc,locn);
				if (p == NULL) return NULL;
				*multi = TRUE;
				return p;
			}
			else if (top->car == TINTERNWIDE) {
				p = t_intern_wide(top->cdr,loc,locn);
				if (p == NULL) return NULL;
				*multi = TRUE;
				return p;
			}
			else if (top->car == SYMBOLFUNCTION) {
				return symbol_function(top->cdr,loc,locn);
			}
			else {
				*multi = TRUE;
				r = getFuncval2(top->car);
				if (r == NULL) {
					printf("G[: `̊֐܂̓}Nł");
					printf("--"); printS(top->car); printf("\n");
					errorlisp(loc,locn);
					return NULL;
				}
				if (r->attr == 'm') {
				    return MacroExec(top->cdr,r,loc,locn);
				}
				len = tlength((r->val)->car);
                                if (len != 0) {
				    temploc = (SYMVAL *)calloc(len,sizeof(SYMVAL));
				    if (temploc == NULL) {
					printf("G[: [Jϐ̈̊mۂɎs܂\n");
					errorlisp(loc,locn);
					return NULL;
				    }
                                }
                                else temploc = NULL;
				if (!initTemploc(temploc,len,(r->val)->car,top->cdr,loc,locn,lock)) {
					printf("G[: ̐Ⴂ܂\n");
					free(temploc);
					errorlisp(loc,locn);
					return NULL;
				}

/* Deleted in Apr.2016
				IncCellStack(NULL);
*/
				IncLocStack(temploc,len);
				p = tprogn((r->val)->cdr,temploc,len,0);
				PopLocStack();
/* Deleted in Apr.2016
				PopCellStack();
				if (p != NULL) IncCellStack(p);
*/
				free(temploc);
				block_throw_value = NULL;	// 24 Aug 2017
				return p;
			}
		}
	}
	else if (top->attr == 'n' || top->attr == 'i' || top->attr == 'b' ||
                 top->attr == 'r' || top->attr == 'm' || top->attr == 'a' ) { *multi = TRUE; return top; }
        else if (top->attr2 == 'c') return top;
	else if ((n2 = getLocal(loc,locn,top,(lock < 0 ? 0 : lock))) != -1) {
		*multi = TRUE;
		return loc[n2].val;
	}
	else {
		*multi = TRUE;
		n2 = getSymVal(top);
		if (n2 == -1) {
			printf("G[: ̕ϐɂ͑Ă܂\n");
			errorlisp(loc,locn);
			return NULL;
		}
                return (sv + n2)->val;
	}
serror:
	printf("G[: @G[\n");
	errorlisp(loc,locn);
	return NULL;
}

CELL *cfunction::eval(CELL *top,SYMVAL *loc,long locn,long lock)
{
	CELL *p;
	FILE *fp;
	BOOL multi;

	if (top->car == VALUES) {
	    return values(top->cdr,loc,locn,lock);
	}
	else if (top->car == READLINE) {
	    if ((top->cdr)->attr != 'l') return ReadLine();
	    p = eval((top->cdr)->car,loc,locn,lock);
	    if (p->attr2 != 's') {
		puts("G[: Xg[ł͂܂");
		errorlisp(loc,locn);
		return NULL;
	    }
	    fp = getStream(p);
	    if (fp == NULL) {
		puts("G[: ȃXg[ł");
		errorlisp(loc,locn);
		return NULL;
	    }
            return ReadLineStrm(fp);
	}
	else if (top->car == EVAL1) {
	    if ((top->cdr)->attr != 'l') goto serror;
            if ((p = eval((top->cdr)->car,loc,locn,lock)) == NULL) return NULL;

	    if (lock < 0) lock *= -1;

	    p = eval(p,loc,locn,lock);
	    return p;
	}
	p = evalu(top,loc,locn,&multi,lock);
	if (p != NULL && !multi) p->mtop = NULL;
	return p;
serror:
	printf("G[: @Ⴂ܂B\n");
	errorlisp(loc,locn);
	return NULL;
}

void cfunction::MultiPrintS(CELL *mtop)
{
    while (mtop->attr == 'l') {
	printS(mtop->car);
	if ((mtop->cdr)->attr == 'l') printf(" ;\n");
	else printf("\n");
	mtop = mtop->cdr;
    }
}

void cfunction::errorlisp(SYMVAL *loc,long locn)
{
    CELL *top;
    int i = 1;
//    static int debug_level = 0;
    int k;

    puts("fobO[hɓ܂B");
    puts("ABORTƓ͂Δ܂B");
    debug_level++;
    for (;;) {
	for (k = 0;k < debug_level;k++) printf("*");
        printf("[%d]",i++);
        lineStr[0] = '\0';
        if (GetOneS()) {
            SkipTerm(lineStr);
            if (!MakeFunctionText(lineStr,BUFLEN - 1)) break;
            if (!MakeQuoteText(lineStr,BUFLEN - 1)) break;
            if (!MakeBackQuoteText(lineStr,BUFLEN - 1)) break;
            if (!MakeCommaText(lineStr,BUFLEN - 1)) break;
            if (!MakeAstText(lineStr,BUFLEN - 1)) break;
            top = readFromString(lineStr);
            if (top == NULL) {
		puts("sȃG[AABORT܂\n");
		break;
	    }
	    if (top == ABORT) break;

	    IncCellStack(NULL);
	    IncCellStack(top);

            top = eval(top,loc,locn,0);

	    PopCellStack();

            if (top != NULL) {
                if (top->mtop == NULL) { /*cfunprintS(top);*/printS(top); printf("\n"); }
                else MultiPrintS(top->mtop);
	    }
	    else if (throw_flag) {
		puts("G[: THROW܂󂯎̃^O܂\n");
		throw_flag = FALSE;
	    }
	    else if (block_throw_flag) {
		puts("G[: RETURN-FROM̎󂯎悪܂\n");
		block_throw_flag = FALSE;
	    }
        }
	else {
	    errorlisp(loc,locn);
	}
    }
    debug_level--;
}

CELL *cfunction::breaklisp(SYMVAL *loc,long locn)
{
    CELL *top;
    int i = 1;
//    static int debug_level = 0;
    int k;

    puts("f܂B");
    puts("CONTINUEƓ͂Αs܂B");
    puts("ABORTƓ͂Δ܂B");
    debug_level++;
    for (;;) {
	for (k = 0;k < debug_level;k++) printf("*");
        printf("[%d]",i++);
        lineStr[0] = '\0';
        if (GetOneS()) {
            SkipTerm(lineStr);
            if (!MakeFunctionText(lineStr,BUFLEN - 1)) break;
            if (!MakeQuoteText(lineStr,BUFLEN - 1)) break;
            if (!MakeBackQuoteText(lineStr,BUFLEN - 1)) break;
            if (!MakeCommaText(lineStr,BUFLEN - 1)) break;
            if (!MakeAstText(lineStr,BUFLEN - 1)) break;
            top = readFromString(lineStr);
            if (top == NULL) {
		puts("sȃG[AABORT܂\n");
		break;
	    }
	    if (top == ABORT) break;

	    if (top == CONTINUELISP) {
		debug_level--;
		return NILCell;
	    }

	    IncCellStack(NULL);
	    IncCellStack(top);

            top = eval(top,loc,locn,0);

	    PopCellStack();

            if (top != NULL) {
                if (top->mtop == NULL) { /*cfunprintS(top);*/printS(top); printf("\n"); }
                else MultiPrintS(top->mtop);
	    }
	    else if (throw_flag) {
		puts("G[: THROW܂󂯎̃^O܂\n");
		throw_flag = FALSE;
	    }
	    else if (block_throw_flag) {
		puts("G[: RETURN-FROM̎󂯎悪܂\n");
		block_throw_flag = FALSE;
	    }
        }
	else {
	    errorlisp(loc,locn);
	}
    }
    debug_level--;
    return NULL;
}

void clisp::ReadEvalPrintLoop(void)
{
    CELL *top;
    int i = 1;

    for (;;) {
        printf("[%d]",i++);
        lineStr[0] = '\0';
        if (GetOneS()) {
            SkipTerm(lineStr);
            if (!MakeFunctionText(lineStr,BUFLEN - 1)) break;
            if (!MakeQuoteText(lineStr,BUFLEN - 1)) break;
            if (!MakeBackQuoteText(lineStr,BUFLEN - 1)) break;
            if (!MakeCommaText(lineStr,BUFLEN - 1)) break;
            if (!MakeAstText(lineStr,BUFLEN - 1)) break;
            top = readFromString(lineStr);
            if (top == NULL) break;

	    IncCellStack(NULL);
	    IncCellStack(top);

            top = eval(top,NULL,0,0);

	    PopCellStack();

            if (top != NULL) {
                if (top->mtop == NULL) { /*cfunprintS(top);*/printS(top); printf("\n"); }
                else MultiPrintS(top->mtop);
	    }
	    else if (throw_flag) {
		puts("G[: THROW܂󂯎̃^O܂\n");
		throw_flag = FALSE;
	    }
	    else if (block_throw_flag) {
		puts("G[: RETURN-FROM̎󂯎悪܂\n");
		block_throw_flag = FALSE;
	    }
//            DumpString();
        }
	else {
	    errorlisp(NULL,0);
	}
    }
}

BOOL clisp::ReadEvalLoop(void)
{
    CELL *top;
    int i;

    for (i = 0;i < READ_EVAL_NUM;i++) {
            strcpy(lineStr,ReadEvalData[i]);
            SkipTerm(lineStr);
            if (!MakeFunctionText(lineStr,BUFLEN - 1)) return (BOOL)NULL;
            if (!MakeQuoteText(lineStr,BUFLEN - 1)) return FALSE;
            if (!MakeBackQuoteText(lineStr,BUFLEN - 1)) return FALSE;
            if (!MakeCommaText(lineStr,BUFLEN - 1)) return FALSE;
            if (!MakeAstText(lineStr,BUFLEN - 1)) return FALSE;
//            printf("%s\n",lineStr);
            top = readFromString(lineStr);
            if (top == NULL) return FALSE;

	    IncCellStack(NULL);
	    IncCellStack(top);

            top = eval(top,NULL,0,0);

	    PopCellStack();

            if (top == NULL) return FALSE;
    }
    return TRUE;
}

void clisp::SetKeyword(void)
{
    long n2;

    QUOTE = getStringCell("QUOTE");
    CAR = getStringCell("CAR");
    CDR = getStringCell("CDR");
    CONS = getStringCell("CONS");
    PRINT = getStringCell("PRINT");
    PRIN1 = getStringCell("PRIN1");
    SET = getStringCell("SET");

    NILCell = getStringCell("NIL");
    TCell = getStringCell("T");

    NIL = getStringCell("NIL");
    n2 = getSymVal2(NIL);
    sv[n2].sym = NIL;
    sv[n2].val = NILCell;

    T = getStringCell("T");
    n2 = getSymVal2(T);
    sv[n2].sym = T;
    sv[n2].val = TCell;

    BACKQUOTE = getStringCell("BACKQUOTE");
    BACK_QUOTE = getStringCell("`");
    COMMA = getStringCell(",");
    AST = getStringCell("@");

    DEFUN = getStringCell("DEFUN");
    AMPREST = getStringCell("&REST");
    LET = getStringCell("LET");
    LETAST = getStringCell("LET*");
    SETQ = getStringCell("SETQ");
    PROG = getStringCell("PROG**");
    GO = getStringCell("GO");
    RETURN = getStringCell("RETURN");

    AND = getStringCell("AND");
    OR = getStringCell("OR");
    ONEPLUS = getStringCell("1+");
    ONEMINUS = getStringCell("1-");
    COND = getStringCell("COND");
    NUMBEREQUAL = getStringCell("=");
    DEFMACRO = getStringCell("DEFMACRO");
    AMPBODY = getStringCell("&BODY");
    ATOM = getStringCell("ATOM");
    LIST = getStringCell("LIST");
    EQ = getStringCell("EQ");
    EVAL = getStringCell("EVAL");
    NULLP = getStringCell("NULL");
    PROGN = getStringCell("PROGN");
    IF = getStringCell("IF");
    TREAD = getStringCell("READ");
    TOPEN = getStringCell("OPEN");
    COLDIRECTION = getStringCell(":DIRECTION");
    COLINPUT = getStringCell(":INPUT");
    COLOUTPUT = getStringCell(":OUTPUT");
    TCLOSE = getStringCell("CLOSE");
    PROG1 = getStringCell("PROG1");
    PLUS = getStringCell("+");
    MINUS = getStringCell("-");
    MINUSP = getStringCell("MINUSP");
    EQL = getStringCell("EQL");
    CHAR = getStringCell("CHAR");
    STRINGP = getStringCell("STRINGP");
    LENGTH = getStringCell("LENGTH");
    COERCE = getStringCell("COERCE");
    STRING = getStringCell("STRING");
    PRINC = getStringCell("PRINC");
    FUNCTION = getStringCell("FUNCTION");
    LAMBDA = getStringCell("LAMBDA");
    SPACES = getStringCell("SPACES");
    TERPRI = getStringCell("TERPRI");
    STRINGEQUAL = getStringCell("STRING-EQUAL");
    READLINE = getStringCell("READ-LINE");
    GC = getStringCell("GC");
    BYE = getStringCell("BYE");
    QUIT = getStringCell("QUIT");
    EXIT = getStringCell("EXIT");
    DEFPARAMETER = getStringCell("DEFPARAMETER");
    DEFVAR = getStringCell("DEFVAR");
    CONSP = getStringCell("CONSP");
    NUMBERP = getStringCell("NUMBERP");
    TIMES = getStringCell("*");
    DIVIDE = getStringCell("/");
    VALUES = getStringCell("VALUES");
    MULTIPLEVALUELIST = getStringCell("MULTIPLE-VALUE-LIST");
    EVAL1 = getStringCell("EVAL1");
    APPLY = getStringCell("APPLY");
    FUNCALL = getStringCell("FUNCALL");
    RPLACA = getStringCell("RPLACA");
    RPLACD = getStringCell("RPLACD");
    SYMBOLP = getStringCell("SYMBOLP");
    CHARACTERP = getStringCell("CHARACTERP");
    CHAREQUAL = getStringCell("CHAR-EQUAL");
    CHARLESS = getStringCell("CHAR<");
    MACROEXPAND = getStringCell("MACROEXPAND");

    INTEGERP = getStringCell("INTEGERP");
    SETARRAY = getStringCell("SET-ARRAY");
//    AREF = getStringCell("AREF");
//    PREAREF = getStringCell("PREAREF");
    CHARACTER = getStringCell("CHARACTER");
    ARRAYCAR = getStringCell("ARRAY-CAR");
    ARRAYCDR = getStringCell("ARRAY-CDR");
    ARRAYP = getStringCell("ARRAYP");

    RANDOM = getStringCell("RANDOM");

    SQRT = getStringCell("SQRT");
    EXPT = getStringCell("EXPT");
    EXP = getStringCell("EXP");
    LOG = getStringCell("LOG");
    SIN = getStringCell("SIN");
    COS = getStringCell("COS");
    TAN = getStringCell("TAN");
    ASIN = getStringCell("ASIN");
    ACOS = getStringCell("ACOS");
    ATAN = getStringCell("ATAN");

    EVENP = getStringCell("EVENP");
    FLOOR = getStringCell("FLOOR");

    NOW = getStringCell("NOW");
    LOCALTIME = getStringCell("LOCAL-TIME");
    MAKETIME = getStringCell("MAKE-TIME");
    CHARCODE = getStringCell("CHAR-CODE");
    CODECHAR = getStringCell("CODE-CHAR");

    CATCHLISP = getStringCell("CATCH");
    THROWLISP = getStringCell("THROW");

    UNWINDPROTECT = getStringCell("UNWIND-PROTECT");

    ABORT = getStringCell("ABORT");
    ERRORLISP = getStringCell("ERROR");

    BREAKLISP = getStringCell("BREAK");
    CONTINUELISP = getStringCell("CONTINUE");

    SYMBOLNAME = getStringCell("SYMBOL-NAME");
    TINTERN = getStringCell("INTERN");
    TINTERNAL = getStringCell(":INTERNAL");
    TINTERNWIDE = getStringCell("INTERN-WIDE");

    SYMBOLFUNCTION = getStringCell("SYMBOL-FUNCTION");
    MACRO = getStringCell("MACRO");

    BLOCKLISP = getStringCell("BLOCK");
    RETURNFROMLISP = getStringCell("RETURN-FROM");
}

void clisp::DispMessage(void)
{
    printf("jpnslisp{Lisp version3.14\nCopyright(c) 2014-2017 S&S\n");
}

int main(void)
{
//    long i = 0L;
//    for (;;) { printf("%ld\n",i); i += 10000L; }
//    LONGDOUBLE a = 0.000000000000345;
//    printf("%Le %Lf %Lg\n",a,a,a);
//    printf("%s\n",ReadEvalData[0]);

    clisp lisp;
    lisp.Init();
    if (!lisp.AllocMemory()) return 1;
    lisp.IncCellStack(NULL);
    lisp.IncLocStack(NULL,0);
    lisp.SetKeyword();
    if (!lisp.ReadEvalLoop()) return 1;
    lisp.DispMessage();
    lisp.ReadEvalPrintLoop();
    return 0;
}

// end of file

